/*
 * main.c
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

#include "maindefs.h"

#ifdef ENV_POSIX
 #include <sys/socket.h>       /*  socket definitions        */
 #include <sys/types.h>        /*  socket types              */
 #include <arpa/inet.h>        /*  inet (3) funtions         */
 #include <unistd.h>           /*  misc. UNIX functions      */
 #include <pwd.h>
#endif
#ifdef ENV_WINDOWS
 #include <windows.h>
#endif
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <stdio.h>
#include <stddef.h>

#ifndef ENV_QNX
#include "getopt.h"
#endif
#include "net.h"
#include "util.h"
#include "platforms.h"
#include "common.h"
#include "mysqlaccess.h"

#define REC_BUFFER_SIZE 10000

#define PODDY_CODE_OK   "OK"
#define PODDY_CODE_FAIL "FAIL"


/** This structure is used for reading data from the socket, it contains a buffer and information about the filling level of that buffer so
    that a complete package can be read in many cycles until it is complete. Every client connection uses exactly one structure of this type */
struct list_entry 
   {
   int                sock;                // the socket to read the data from
   struct list_entry *next;                // the next structure in the list
   char              *recBuffer;           // a buffer to store the received data into
   int                readLen,lastReadLen; // the data lengths read from the socket
   bool               packetComplete;      // this value signals if a complete package was received and can be processed now
   char              *remoteIP;            // the IP of the remote connection
   };



/** This structure is the head for the list of elements of type struct list_entry */
struct list_type 
   {
   struct list_entry *data; // the first element of type list_entry
   unsigned count;          // the total number of entries stored by this list
   };




char                   *bindToIP2=NULL,*bindToIP=NULL,*settingsFile=NULL,*userName=NULL;
static unsigned short   mainport=26175,mainport2=0;
static struct list_type list={NULL,0};

#ifdef ENV_POSIX
static bool            daemonMode=false,verbose=false;
#endif

#ifdef ENV_QNX
#define MSG_NOSIGNAL 0
#endif

FILE                  *sysLog=NULL;



/**
 * This function creates a new list_entry and adds a new client to the internal list
 * of connections
 * @param list the list where the new entry has to be added
 * @param sock the socket descriptor of the new client connection
 * @parem remoteIP the remote IP of the client
 * @return 0 when the operation could be finished successfully or an error code otherwise
 */
int addClient(struct list_type *list, int sock,char *remoteIP)
   {
   struct list_entry *n;
   char   *buf;

   buf=(char*)malloc(REC_BUFFER_SIZE+2);
   if (!buf) return ERR_NO_MEMORY;
   n = malloc(sizeof(*n));
   if (!n) 
      {
      free(buf);
      return ERR_NO_MEMORY;
      }
   n->sock = sock;
   n->recBuffer=buf;
   if (remoteIP) n->remoteIP=strdup(remoteIP);
   else n->remoteIP=strdup("unknown host");
   n->readLen=0;
   n->packetComplete=false;
   n->next = list->data;
   list->data = n;
   list->count++;
   return 0;
   }



/**
 * Whenever a client closes its connection it has to be removed from the list of active
 * clients and the resources of the related list_entry-structure have to be released.
 * That has to be done by calling this function.
 * @param list the list of entries where the client has to be removed from
 * @param entry the clients list entry that has to be removed
 * @return 0 in case the operation could be completed successfully or an error code
 *         otherwise
 */
int removeClient(struct list_type *list,struct list_entry *entry)
   {
   struct list_entry *lz, *lst = NULL;
   
   if (!entry) return 0;

   if (!list->count) return ERR_LIST_EMPTY;
   for (lz = list->data; lz; lz = lz->next)
      {
      if (lz->sock == entry->sock) break;
      lst = lz;
      }
   if (!lz) return ERR_NO_SUCH_ELEMENT;
   if (lst) lst->next = lz->next;
   else list->data = lz->next;
   if (lz->recBuffer) free(lz->recBuffer);
   if (lz->remoteIP) free(lz->remoteIP);
   free(lz);
   list->count--;
   return 0;
   }



/**
 * This function removes all entries from a given list of connections
 * @param list the list of entries that has to be flushed completely
 */
void freeList(struct list_type *list)
   {
   while ((list->data) && (list->data->sock)) removeClient(list,list->data);
   }




/**
 * This function us used together with the socet function select(), it fills the file
 * descriptor set that is required by set depending on the available socket connections
 * @parem fds the file descriptor set structure to be filled
 * @param list the list of clients from which the socket information have to be used
 * @return the maximum socket file descriptor number
 */
int fillSet(fd_set *fds, struct list_type *list)
   {
   int max = 0;
   struct list_entry *lz;

   for (lz = list->data; lz; lz = lz->next)
   {
      if (lz->sock > max) max = lz->sock;
      FD_SET(lz->sock, fds);
   }
   return max;
}



/**
 * This function can be used after a call to the socekt function select(), it retrieves
 * that client out of a given list where a event occured that was watched by select().
 * @param fds the file descriptor set structure that was modified by select()
 * @param the related list of client conmnections that belongs to the preceding caqll of
 *        select and that was used by a former call to fillSet()
 * @return the client connection that caused select() to leave its waiting state or NULL
 *        in case there is no client socket that fits to the result
 */
struct list_entry *getSender(fd_set *fds,struct list_type *list)
   {
   int                i=0;
   struct list_entry *lz;

   while(!FD_ISSET(i, fds)) i++;
   lz=list->data;
   while (lz)
      {
      if (lz->sock==i) return lz;
      lz=lz->next;
      }
   return NULL;
   }



/** definitions for the command line options */
static char const shortopts[] = "vVhC:";
#ifndef ENV_QNX
static struct option const longopts[] = {
  {"version", no_argument, NULL, 'v'},
#ifdef ENV_LINUX
  {"verbose", no_argument, NULL, 'V'},
#endif  
  {"help",    no_argument, NULL, 'h'},
  {"settings",required_argument,NULL,'C'},
  {NULL, no_argument, NULL, 0}
};
#endif



/** definitions for the command line help output */
#ifdef ENV_QNX
static char const *const option_help[] = {
  " -C     path to config file (overwrites command line params); default: /etc/poddyd.conf",
  " -V     print out some debugging information",
  " -v     show version info",
  " -h     show this help",
  0
};
#else
static char const *const option_help[] = {
  #ifndef ENV_WINDOWS
  " -C  --config <path>        path to config file (overwrites command line params); default: /etc/poddyd.conf",
  #else
  " -C  --config <path>        path to config file (overwrites command line params); default: poddyd.conf",
  #endif
  #ifdef ENV_POSIX
  " -V  --verbose              print out some debugging information",
  #endif
  " -v  --version              show version info",
  " -h  --help                 show this help",
  0
};
#endif


/** print out version information */
static void version(void)
{
printf ("%s %s %s\n%s\n", FCOMMON_NAME,FCOMMON_VERSION,FCOMMON_URL,FCOMMON_COPYRIGHT);
}



/** print out command line help information */
static void usage (char *pname)
{
printf ("usage: %s [OPTIONS]\n",pname);
printf (" --help for more information\n");
return;
}



/**
 * Get the switches and related values out of the command line parameters
 * @param argc the number of arguments as handed over by main()
 * @param argv the arguments as handed over by main()
 */
static void getSwitches(int argc, char *argv[])
{
unsigned int   optc;
char const    *const *p;

if (optind == argc) return;
#ifdef ENV_QNX
while ((optc = getopt(argc, argv, shortopts)) != -1)
#else
while ((optc = getopt_long (argc, argv, shortopts, longopts, (int *) 0)) != -1)
#endif
   {
   switch (optc)
      {
#ifndef ENV_WINDOWS
      case 'V':
         verbose=true;
         break;
#endif
      case 'C':
         settingsFile=strdup(optarg);
         break;
      case 'v':
         version();
         exit (0);
         break;
      case 'h':
         usage (argv[0]);
         for (p = option_help; *p; p++) printf ("%s\n", *p);
         exit (0);
         break;
      default:
         usage (argv[0]);
      }
   }
}



/**
 * This function reads from the socket and tries to fetch a full packet of data. When no end of
 * a package is found it reads data only for a given time, stores them and tries to finish receiving
 * during a next call. When a full package of data could be read successfully the member
 * packageComplete of the clients list entry structure is set to TRUE so that it can ba handled by
 * the main loop.
 * @param entry the list entry of the client where data have to be read for
 * @param timeout the maximum time in milliseconds that is allowed to be used to receive data from
 *        the clients socket
 * @return true in case a full package could be read successfully or false otherwise
 */
static bool readPacket(struct list_entry *entry, long timeout) 
{
long /*size_t*/   rc;
long     ctr=0;
#ifdef ENV_WINDOWS
long     err;
#endif

entry->lastReadLen=0;
while ((entry->readLen<REC_BUFFER_SIZE) && 
       (!entry->packetComplete))
   {
   rc = recv(entry->sock,entry->recBuffer+entry->readLen,REC_BUFFER_SIZE-entry->readLen,MSG_NOSIGNAL);   
   if (rc>0)
      {
      entry->readLen+=rc;
      entry->lastReadLen=0;
      entry->recBuffer[entry->readLen]=0;
      if ((strstr(entry->recBuffer,"poddy0\n")) || (strstr(entry->recBuffer,"poddy0\r")))
       entry->packetComplete=true;
      }
   else if (rc==0) return false;
   else
      {
#ifdef ENV_WINDOWS
      err=GetLastError();
      if (err!=EAGAIN)
#else
      if ((errno!=EAGAIN) && (errno!=EINPROGRESS) && (errno!=0))
#endif
       return false;
      ctr+=ThreadSleep(10);
      }
   if (ctr>timeout)
      {
      entry->lastReadLen++;
      return false;   
      }
      
   }
return true;
}



/**
 * This function processes a string and removes all CR/LF at the end of it
 * @param[in/out] inBuffer the buffer of chars that has to be processed, the modification
 *        of the data is done in-place
 */
static void remCRLF(char *inBuffer)
{
if (inBuffer[strlen(inBuffer)-1]=='\n') inBuffer[strlen(inBuffer)-1]=0;
if (inBuffer[strlen(inBuffer)-1]=='\r') inBuffer[strlen(inBuffer)-1]=0;
}



/**
 * Within this function the settings file is read and several internal variables are set#
 * according to the values read here
 * @param[in] fname the name of the configuration file that has to be read
 * @return true when the given file could be opened successfully, false in case
 *         it could not be accessed
 */
static bool loadSettings(char *fname)
{
#define INBUFFER_SIZE DATASIZE*2

char  inBuffer[INBUFFER_SIZE];
FILE *fh;

fh=fopen(fname,"r");
if (fh)
   {
   while (fgets(inBuffer,INBUFFER_SIZE,fh))
      {
      if ((inBuffer[0]=='#') || (inBuffer[0]=='\r') || (inBuffer[0]=='\n'))
         {
         //drop comment/empty line
         }
      #ifndef ENV_WINDOWS
      else if (strstr(inBuffer,"uid=")==inBuffer)
         {
         remCRLF(inBuffer);
         userName=strdup(inBuffer+4);
         }
      #endif
      else if (strstr(inBuffer,"port=")==inBuffer) mainport=atoi(inBuffer+5);
      else if (strstr(inBuffer,"bindto-ip=")==inBuffer)
         {
         if (bindToIP) free(bindToIP);
         remCRLF(inBuffer);
         bindToIP=strdup(inBuffer+10);
         }

      else if (strstr(inBuffer,"port2=")==inBuffer) mainport2=atoi(inBuffer+6);
      else if (strstr(inBuffer,"bindto-ip2=")==inBuffer)
         {
         if (bindToIP2) free(bindToIP2);
         remCRLF(inBuffer);
         bindToIP2=strdup(inBuffer+11);
         }
      }
   fclose(fh);
   return true;
   }
return false;
}



/**
 * This function has to be called when the application exits, it releases several used
 * buffers
 */
static void freeMem(void)
{
if (userName) free(userName);         userName=NULL;
if (bindToIP) free(bindToIP);         bindToIP=NULL;
if (bindToIP2) free(bindToIP2);       bindToIP2=NULL;
if (settingsFile) free(settingsFile); settingsFile=NULL;
}



/**
 * A Poddy data package consists of a line "poddy1", followed by several lines of data and
 * it is finished by a line that contains the text "poddy0". So both texts "poddyx" are the
 * delimiters for a complete package, the number specifies the end of a package (=0) or the
 * beginning (=1, used as version number so future releases can have a different number).
 * This funktion searches for the beginning signature within a data package.
 * @param[in] e the clients list entry structure that contains the data to search into
 * @return true when the beginning of a data package could be found, false otherwise
 */
static bool findID(struct list_entry *e)
   {
   char *pos;
   
   pos=strstr(e->recBuffer,"poddy1");
   if (!pos) return false;
   e->readLen=(pos-e->recBuffer)+7;
   while ((e->recBuffer[e->readLen]!=0) && ((e->recBuffer[e->readLen]=='\r') || (e->recBuffer[e->readLen]=='\n')))
    e->readLen++;
   return true;
   }



/**
 * The data within a Poddy data package are structured as simple text lines that are terminated by a LF
 * character. Using this function one single line can be read out of the current package that is stored
 * within a clients list entry structure
 * @param[in] e the entry where the next line has to be fetched from
 * @return the next line, the LF character is already removed from it; NULL in case the end of a data
 *         block was found
 */
static char *getNextLine(struct list_entry *e)
   {
   char *pos;
   
   pos=&e->recBuffer[e->readLen];
   while (e->recBuffer[e->readLen]!=0)
      {
      if ((e->recBuffer[e->readLen]=='\r') || (e->recBuffer[e->readLen]=='\n'))
         {
         e->recBuffer[e->readLen]=0;
         e->readLen++;
         break;
         }
      e->readLen++;
      }
   while ((e->recBuffer[e->readLen]!=0) && ((e->recBuffer[e->readLen]=='\r') || (e->recBuffer[e->readLen]=='\n')))
      {
      e->readLen++;
      }
   if (strstr(pos,"poddy0")==pos) return NULL;
   return pos;
   }



/**
 * The response messages that are sent from the Poddy daemon to the Poddy Home application
 * are structured in the same way like client data packages. within this function a defined
 * response data package is sent to a client
 * @param[in] e the client the response has to be sent to
 * @param[in] code a short text that acts as reason code for the response
 * @param[in] a better, human readable message that contains a more descriptive reason for
 *            the current response
 */
static bool sendResponseMessage(struct list_entry *e,char *code,char *msg)
   {
   char buffer[REC_BUFFER_SIZE];
   int  sendLen=0;

   if (code)
      {
      if (msg) sprintf(buffer,"poddy1\n%s\n%s\npoddy0\n",code,msg);
      else sprintf(buffer,"poddy1\n%s\npoddy0\n",code);
      }
   else sprintf(buffer,"%s\n",msg);
   sendLen=ssys_send(e->sock,buffer,strlen(buffer),MSG_NOSIGNAL,1000,__FILE__,__LINE__);
   if (sendLen==strlen(buffer)) return true;
   return false;
   }



/**
 * This is the main loop, it is called by main() after several necessary initializations have been done.
 * This main loop accepts new incoming connections, tries to read data from all available connections,
 * parses received data blocks, invokes the required (re)action for the received data and sends a
 * response back to the client.
 */
static bool mainLoop(void* hInstance)
   {
   unsigned long      on=1;
   int                list_s,list_s2;                /*  listening socket          */
   unsigned long      remoteIP;
   struct in_addr     in_address;
   int                c, max;
   struct list_entry *sender;
   fd_set             fds;
   MYSQL_RES         *res=NULL;
   char               query[10000],result[20000];
   char              *cmd,*uname,*pwd,*id1,*id2;
   char             **row;

#ifdef ENV_POSIX
if (daemonMode)
#endif
 initApplication(hInstance);

sysLog=fopen("/var/log/poddyd.log","a");
if (!sysLog)
   {
   #ifdef ENV_WINDOWS
   errno=GetLastError();
   #else
   perror("");
   #endif
   printf("ERROR: could not open logfile (%d)!\n",errno);
   }
fmtLogTime();
showLog(sysLog,"%s: Starting %s...",logtime,FCOMMON_NAME);
if (!bindToIP) bindToIP=strdup("0.0.0.0");
list_s=listenOnPort(mainport,bindToIP);
if (list_s<=0)
   {
   #ifdef ENV_WINDOWS
   errno=GetLastError();
   #else
   perror("");
   #endif
   showLog(sysLog,"%s: ERROR: could not create listening socket %s:%d, %d!",logtime,bindToIP,mainport,errno);
   freeMem();
   return false;
   }
if (setsockopt(list_s,SOL_SOCKET,SO_KEEPALIVE,(char*)&on,sizeof(on))<0)
    {
    #ifdef ENV_WINDOWS
    errno=GetLastError();
    #else
    perror("");
    #endif
    showLog(sysLog,"%s: WARNING: could not set socket options, %d!",logtime,errno);
    }
showLog(sysLog,"%s: INFO: binding to %s:%d",logtime,bindToIP,mainport);

if ((bindToIP2) && (mainport2>0))
   {
   list_s2=listenOnPort(mainport2,bindToIP2);
   if (list_s2<=0)
      {
      #ifdef ENV_WINDOWS
      errno=GetLastError();
      #else
      perror("");
      #endif
      showLog(sysLog,"%s: WARNING: could not create alternative listening socket %s:%d, %d!",logtime,bindToIP2,mainport2,errno);
      if (setsockopt(list_s2,SOL_SOCKET,SO_KEEPALIVE,(char*)&on,sizeof(on))<0)
         {
         #ifdef ENV_WINDOWS
         errno=GetLastError();
         #else
         perror("");
         #endif
         showLog(sysLog,"%s: WARNING: could not set alternative socket options, %d!",logtime,errno);
         }
      }
   }
else list_s2=-1;

#ifdef ENV_POSIX
if (userName)
   {
   struct passwd *pwd;

   pwd=getpwnam(userName);
   if (!pwd) fprintf(stderr,"Error: unknown username %s!\n",optarg);
   else
      {
      if (setgid(pwd->pw_uid)!=0) perror("ERROR: Setting group failed");
      if (setuid(pwd->pw_uid)!=0) perror("ERROR: Setting user failed");
      }
   free(userName);
   }

if (daemonMode) daemon(0,0);
#endif
while (!LeaveServer(0,0,0,1))
   {
   FD_ZERO(&fds);
   max = fillSet(&fds, &list);
   FD_SET(list_s, &fds);
   if (list_s > max) max = list_s;
   select(max + 1, &fds, NULL, NULL, NULL);
   
   if (FD_ISSET(list_s, &fds))
      {
      c=acceptConnection(list_s, &remoteIP);
      if (c<=0) c=acceptConnection(list_s2, &remoteIP);
      if (c!=0)
         {
         in_address.s_addr=htonl(remoteIP);
         fmtLogTime();
         showLog(sysLog,"%s: Incoming connection from %s",logtime,inet_ntoa(in_address));
         setBlocking(c,false);
         addClient(&list, c,inet_ntoa(in_address));

/*         if (setsockopt(c,SOL_SOCKET,SO_KEEPALIVE,(char*)&on,sizeof(on))<0)
            {
            #ifdef ENV_WINDOWS
            errno=GetLastError();
            #else
            perror("");
            #endif
            showLog(sysLog,"%s: WARNING: could not set socket options, %d!",logtime,errno);
            }*/
         }
      }
   else
      {
      // get the sender that sends new data at the moment
      sender = getSender(&fds,&list);
      if (sender)
      {
         if ((!readPacket(sender,150)) && (sender->lastReadLen==0))
            {
	    // remove a client if the connection was lost
            fmtLogTime();
            showLog(sysLog,"%s: Connection lost to %s",logtime,sender->remoteIP);
            removeClient(&list, sender);
            }
         else if (sender->packetComplete)
            {
            sender->readLen=0;
            if (findID(sender))
               {
               cmd=getNextLine(sender);
               if (cmd)
               {
	          // Init a SQL connection to manage the data for a client; it seems to be necessary to open/close such a
		  // connection more often. When a connection is idle for a too long time it is closed by the server also
		  // in case of connections on the localhost. 
                  initSQLConnection();
                  /******************************************************************************************************************/
                  if (strstr(cmd,"CLOSE")==cmd) // close command from the client, all communication has been done and the client disconnects regulary
                     {
                     fmtLogTime();
                     showLog(sysLog,"%s: Connection closed from %s",logtime,sender->remoteIP);
                     removeClient(&list, sender);
                     }
                  /******************************************************************************************************************/
                  else if (strstr(cmd,"AUTH")==cmd) // authentication data from the client, the first line contains the user name and the second one the password hash
		                                    // "AUTH" is only a part of the full command, after the authentication is done the full command is parsed below to
						    // find out which operation is requested 
                     {
                     int   userid=-1;
                     char *station,*channel;
                     
                     uname=getNextLine(sender);
                     if (uname) pwd=getNextLine(sender);
                     if ((uname) && (pwd))
                        {
                        sprintf(query,"SELECT alias, userid FROM "DB_TABLE_USERDATA" WHERE alias='%s' AND pwd='%s'",uname,pwd);
                        res=doSQLQuery(query);
                        if (res)
                           {
                           row=getNextSQLResult(res);
                           if ((row) && (row[1]))
                              {
                              userid=atoi(row[1]);
                              freeSQLQuery(res);
                              
                              if (strstr(cmd,"AUTHSETSTAT")==cmd) // set a new list of TV stations for this user
                                 {
                                 //delete existing station list of user and set new one
                                 sprintf(query,"DELETE FROM "DB_TABLE_STATIONS" WHERE userid='%d'",userid);
                                 doSQLQuery(query);
                                 do
                                    {
                                    station=getNextLine(sender);
                                    if (station) channel=getNextLine(sender);
                                    if ((station) && (channel))
                                       {
                                       sprintf(query,"INSERT INTO "DB_TABLE_STATIONS" (userid, station, channel) VALUES ('%d', '%s', '%s')",userid,station,channel);
                                       doSQLQuery(query);
                                       }
                                    }
                                 while ((station) && (channel));
                                 sendResponseMessage(sender,PODDY_CODE_OK,NULL);
                                 }
                              /*******************************************************************************************************/
                              else if (strstr(cmd,"AUTHSETDELSCHED")==cmd) // delete an existing schedule
                                 {
                                 //set new remote schedule with synced flag = 1
                                 char *stime=NULL,*rtime=NULL,*station=NULL,*name=NULL;
                                 
                                 //TODO: check if this schedule already exists to give an appropriate rrror message in case there is no such dataset
                                 stime=getNextLine(sender);
                                 if (stime)    rtime=getNextLine(sender);
                                 if (rtime)    station=getNextLine(sender);
                                 if (station)  name=getNextLine(sender);
                                 if ((stime) && (rtime) && (station) && (name))
                                    {
                                    sprintf(query,"DELETE FROM "DB_TABLE_SCHEDULE" where userid='%d' AND stime='%s' AND rtime='%s' AND station='%s' AND name='%s'",userid,stime,rtime,station,name);
                                    doSQLQuery(query);
                                    sendResponseMessage(sender,PODDY_CODE_OK,NULL);                                    
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");

                                 }
                              /*******************************************************************************************************/
                              else if (strstr(cmd,"AUTHSETHL")==cmd) // set a new schedule
                                 {
                                 //set new remote schedule with synced flag = 1
                                 char *name=NULL,*dlurl=NULL,*errtext=NULL;
                                 
                                 name=getNextLine(sender);
                                 if (name)    dlurl=getNextLine(sender);
                                 if (dlurl)   errtext=getNextLine(sender);
                                 if ((name) && (dlurl) && (errtext))
                                    {
                                    if ((!strcmp(errtext,"null")) || (!strcmp(errtext,"(null)"))) sprintf(query,"DELETE FROM "DB_TABLE_HOMELOAD" WHERE userid='%d' AND name='%s' AND dlurl='%s'",userid,name,dlurl);
                                    else sprintf(query,"UPDATE "DB_TABLE_HOMELOAD" SET errtext='%s' WHERE userid='%d' AND name='%s' AND dlurl='%s'",errtext,userid,name,dlurl);
                                    doSQLQuery(query);
                                    sendResponseMessage(sender,PODDY_CODE_OK,NULL);                                    
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                                 }
                              else if (strstr(cmd,"AUTHGETHL")==cmd) // get web-programmed downloads (currently not supported in Poddy Home)
                                 {
                                 //retrieve all downloads with synced = 0 and set this flag to 1 afterwards
                                 sprintf(query,"SELECT name, dlurl FROM "DB_TABLE_HOMELOAD" WHERE userid='%d' AND synced='0' and deleted='0'",userid);
                                 res=doSQLQuery(query);
                                 if (res)
                                    {
                                    row=getNextSQLResult(res);                                    
                                    if (row)
                                       {
                                       sendResponseMessage(sender,NULL,"poddy1");
                                       sendResponseMessage(sender,NULL,PODDY_CODE_OK);
                                       while (row)
                                          {
                                          sprintf(result,"%s\n%s",row[0],row[1]);
                                          sendResponseMessage(sender,NULL,result);
                                          row=getNextSQLResult(res);                                    
                                          }                                       
                                       sendResponseMessage(sender,NULL,"poddy0");
                                       freeSQLQuery(res);                                    
                                       sprintf(query,"UPDATE "DB_TABLE_HOMELOAD" set synced='1' WHERE userid='%d' AND synced='0' and deleted='0'",userid);
                                       doSQLQuery(query);
                                       }
                                    else
                                       {
                                       freeSQLQuery(res);                                    
                                       sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW HOMELOADS");
                                       }
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW HOMELOADS");
                                 }
                              else if (strstr(cmd,"AUTHGETDELHL")==cmd) // get deleted downloads (currently not supported in poddy home)
                                 {
                                 sprintf(query,"SELECT name, dlurl FROM "DB_TABLE_HOMELOAD" WHERE userid='%d' AND synced='0' and deleted='1'",userid);
                                 res=doSQLQuery(query);
                                 if (res)
                                    {
                                    row=getNextSQLResult(res);                                    
                                    if (row)
                                       {
                                       sendResponseMessage(sender,NULL,"poddy1");
                                       sendResponseMessage(sender,NULL,PODDY_CODE_OK);
                                       while (row)
                                          {
                                          sprintf(result,"%s\n%s",row[0],row[1]);
                                          sendResponseMessage(sender,NULL,result);
                                          row=getNextSQLResult(res);                                    
                                          }                                       
                                       sendResponseMessage(sender,NULL,"poddy0");
                                       sprintf(query,"DELETE FROM "DB_TABLE_HOMELOAD" WHERE userid='%d' AND synced='0' and deleted='1'",userid);
                                       res=doSQLQuery(query);
                                       freeSQLQuery(res);                                    
                                       }
                                    else
                                       {
                                       freeSQLQuery(res);                                    
                                       sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW DELETED HOMELOADS");
                                       }
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW DELETED HOMELOADS");
                                 }
                              /*******************************************************************************************************/
                              else if (strstr(cmd,"AUTHGETDELSCHED")==cmd) // get deleted schedules to synchronize with the schedules that are already stored in the database
                                 {
                                 sprintf(query,"SELECT stime, rtime, station, name FROM "DB_TABLE_SCHEDULE" WHERE userid='%d' AND synced='0' and deleted='1'",userid);
                                 res=doSQLQuery(query);
                                 if (res)
                                    {
                                    row=getNextSQLResult(res);                                    
                                    if (row)
                                       {
                                       sendResponseMessage(sender,NULL,"poddy1");
                                       sendResponseMessage(sender,NULL,PODDY_CODE_OK);
                                       while (row)
                                          {
                                          sprintf(result,"%s\n%s\n%s\n%s",row[0],row[1],row[2],row[3]);
                                          sendResponseMessage(sender,NULL,result);
                                          row=getNextSQLResult(res);                                    
                                          }    
                                       sendResponseMessage(sender,NULL,"poddy0");
                                       sprintf(query,"DELETE FROM "DB_TABLE_SCHEDULE" WHERE userid='%d' AND synced='0' and deleted='1'",userid);
                                       res=doSQLQuery(query);
                                       freeSQLQuery(res);                                    
                                       }
                                    else
                                       {
                                       freeSQLQuery(res);                                    
                                       sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW DELETED SCHEDLUES");
                                       }
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW DELETED SCHEDLUES");
                                 }
                              else if (strstr(cmd,"AUTHSETSCHED")==cmd) // set new schedule to synchronize with the database
                                 {
                                 //set new remote schedule with synced flag = 1
                                 char *stime=NULL,*rtime=NULL,*station=NULL,*name=NULL;
                                 
                                 //TODO: check if this schedule already exists
                                 stime=getNextLine(sender);
                                 if (stime)    rtime=getNextLine(sender);
                                 if (rtime)    station=getNextLine(sender);
                                 if (station)  name=getNextLine(sender);
                                 if ((stime) && (rtime) && (station) && (name))
                                    {
                                    sprintf(query,"INSERT INTO "DB_TABLE_SCHEDULE" (userid, stime, rtime, station, name, synced) VALUES ('%d', '%s', '%s', '%s', '%s', '1')",userid,stime,rtime,station,name);
                                    doSQLQuery(query);
                                    sendResponseMessage(sender,PODDY_CODE_OK,NULL);                                    
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                                 }
                              else if (strstr(cmd,"AUTHGETSCHED")==cmd) // get web-programmed schedule; as response the client expects the schedule data
                                 {
                                 //retrieve all schedules with synced = 0 and set this flag to 1 afterwards
                                 sprintf(query,"SELECT stime, rtime, station, name FROM "DB_TABLE_SCHEDULE" WHERE userid='%d' AND synced='0' and deleted='0'",userid);
                                 res=doSQLQuery(query);
                                 if (res)
                                    {
                                    row=getNextSQLResult(res);                                    
                                    if (row)
                                       {
                                       sendResponseMessage(sender,NULL,"poddy1");
                                       sendResponseMessage(sender,NULL,PODDY_CODE_OK);
                                       while (row)
                                          {
                                          sprintf(result,"%s\n%s\n%s\n%s",row[0],row[1],row[2],row[3]);
                                          sendResponseMessage(sender,NULL,result);
                                          row=getNextSQLResult(res);                                    
                                          }                                       
                                       sendResponseMessage(sender,NULL,"poddy0");
                                       freeSQLQuery(res);                                    
                                       sprintf(query,"UPDATE "DB_TABLE_SCHEDULE" set synced='1' WHERE userid='%d' AND synced='0' and deleted='0'",userid);
                                       doSQLQuery(query);
                                       }
                                    else
                                       {
                                       freeSQLQuery(res);                                    
                                       sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW SCHEDLUES");
                                       }
                                    }
                                 else sendResponseMessage(sender,PODDY_CODE_FAIL,"NO NEW SCHEDLUES");
                                 }
                              else sendResponseMessage(sender,PODDY_CODE_OK,NULL);
                              }
                           else 
                              {
                              freeSQLQuery(res);
                              sendResponseMessage(sender,PODDY_CODE_FAIL,"AUTH FAILURE (B)");
                              }
                           }
                        else
                           {
                           char s[200];
                           
                           sprintf(s,"AUTH FAILURE %s %s",uname,pwd);
                           sendResponseMessage(sender,PODDY_CODE_FAIL,s);//"AUTH FAILURE (A)");
                           }
                        }
                     else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                     }
                  /******************************************************************************************************************/
                  else if (strstr(cmd,"SETDVD")==cmd) // this command does not require an authentication, here (possibly new) DVD data are submitted from the client to the global database
                     {
                     bool  alreadyExists=false;
                     char *ean=NULL,*time=NULL,*year=NULL,*title=NULL,*genre=NULL,*season=NULL,*disc=NULL;
                     
                     id1=getNextLine(sender);
                     if (id1)
                        {
                        sprintf(query,"SELECT * FROM "DB_TABLE_DVDINFO" WHERE crcid='%s'",id1);
                        res=doSQLQuery(query);
                        if (res)
                           {
                           row=getNextSQLResult(res);
                           if (row) alreadyExists=true;
                           freeSQLQuery(res);
                           }
                        if (!alreadyExists)
                           {
                           ean=getNextLine(sender);
                           if (ean)    time=getNextLine(sender);
                           if (time)   year=getNextLine(sender);
                           if (year)   title=getNextLine(sender);
                           if (title)  genre=getNextLine(sender);
                           if (genre)  season=getNextLine(sender);
                           if (season) disc=getNextLine(sender);
                           if ((ean) && (time) && (year) && (title) && (genre) && (season) && (disc))
                              {
                              sprintf(query,"INSERT INTO "DB_TABLE_DVDINFO" (crcid, ean, time, year, title, genre, season, disc) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')",id1,ean,time,year,title,genre,season,disc);
                              doSQLQuery(query);
                              sendResponseMessage(sender,PODDY_CODE_OK,NULL);
                              }
                           else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                           }
                        else sendResponseMessage(sender,PODDY_CODE_FAIL,"DATASET EXISTS");
                        }
                     else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                     }
                  else if (strstr(cmd,"GETDVD")==cmd) // when a new DVD is inserted for copying on a client this command is sent in order to retrieve the related information about this DVD
                     {
                     id1=getNextLine(sender);
                     if (id1)
                        {
                        sprintf(query,"SELECT * FROM "DB_TABLE_DVDINFO" WHERE crcid='%s'",id1);
                        res=doSQLQuery(query);
                        if (res)
                           {
                           row=getNextSQLResult(res);
                           if ((row) && (row[8]))
                              {
                              sprintf(result,"time=%s\nyear=%s\ntitle=%s\ngenre=%s\nseason=%s\ndisc=%s",row[2],row[3],row[5],row[6],row[7],row[8]);
                              sendResponseMessage(sender,PODDY_CODE_OK,result);
                              }
                           else sendResponseMessage(sender,PODDY_CODE_FAIL,"NOT FOUND");
                           freeSQLQuery(res);
                           }
                        else sendResponseMessage(sender,PODDY_CODE_FAIL,"NOT FOUND");
                        }
                     else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                     }
                  /******************************************************************************************************************/
                  else if (strstr(cmd,"SETCD")==cmd) // this command does not require an authentication, here (possibly new) CD data are submitted from the client to the global database
                     {
                     int       i;
                     bool      alreadyExists=false;
                     char     *ean=NULL,*time=NULL,*year=NULL,*title=NULL,*genre=NULL,*cat=NULL,*artist=NULL;
                     char     *t[35];
                     char      empty=0;
                     
                     for (i=0; i<35; i++) t[i]=NULL;
                     id1=getNextLine(sender);
                     if (id1) id2=getNextLine(sender);
                     if ((id1) && (id2))
                        {
                        sprintf(query,"SELECT * FROM "DB_TABLE_CDINFO" WHERE discid='%s' AND tracksid='%s'",id1,id2);
                        res=doSQLQuery(query);
                        if (res)
                           {
                           row=getNextSQLResult(res);
                           if (row) alreadyExists=true;
                           freeSQLQuery(res);
                           }
                        if (!alreadyExists)
                           {
                           ean=getNextLine(sender);
                           if (ean)    time=getNextLine(sender);
                           if (time)   year=getNextLine(sender);
                           if (year)   title=getNextLine(sender);
                           if (title)  genre=getNextLine(sender);
                           if (genre)  cat=getNextLine(sender);
                           if (cat)    artist=getNextLine(sender);
                           if (artist) t[0]=getNextLine(sender);
                           if (t[0])
                            for (i=1; i<35; i++)
                              {
                              t[i]=getNextLine(sender);
                              if (!t[i]) break;
                              }
                           for (i=1; i<35; i++)
                            if (!t[i]) t[i]=&empty;

                           if ((ean) && (time) && (year) && (title) && (genre) && (cat) && (artist) && (t[0]))
                              {
                              sprintf(query,"INSERT INTO "DB_TABLE_CDINFO" (discid, tracksid, ean, time, year, title, genre, cat, artist, t01, t02, t03, t04, t05, t06, t07, t08, t09, t10, t11, t12, t13, t14, t15, t16, t17, t18, t19, t20, t21, t22, t23, t24, t25, t26, t27, t28, t29, t30, t31, t32, t33, t34, t35) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')",
                                      id1,id2,ean,time,year,title,genre,cat,artist,t[0],t[1],t[2],t[3],t[4],t[5],t[6],t[7],t[8],t[9],t[10],t[11],t[12],t[13],t[14],t[15],t[16],t[17],t[18],t[19],t[20],t[21],t[22],t[23],t[24],t[25],t[26],t[27],t[28],t[29],t[30],t[31],t[32],t[33],t[34]);
                              doSQLQuery(query);
                              sendResponseMessage(sender,PODDY_CODE_OK,NULL);
                              }
                           else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                           }
                        else sendResponseMessage(sender,PODDY_CODE_FAIL,"DATASET EXISTS");
                        }
                     else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                     }
                  else if (strstr(cmd,"GETCD")==cmd) // when a new CD is inserted for copying on a client this command is sent in order to retrieve the related information
		                                     // about this CD; comparing to a request to the CDDB this method has a big advantage: here CDs can be identified
						     // non-ambiguous so that there can exist exactly one entry within the database so that the user does not have to choose
						     // which entry _could_ be the best one for a CD
                     {
                     id1=getNextLine(sender);
                     if (id1) id2=getNextLine(sender);
                     if ((id1) && (id2))
                        {
                        sprintf(query,"SELECT * FROM "DB_TABLE_CDINFO" WHERE discid='%s' AND tracksid='%s'",id1,id2);
                        row=NULL;
                        res=doSQLQuery(query);                        
                        if (res) row=getNextSQLResult(res);
                        if (!row)
                           {
                           freeSQLQuery(res);
                           sprintf(query,"SELECT * FROM "DB_TABLE_CDINFO" WHERE discid='%s'",id1);
                           res=doSQLQuery(query);                        
                           if (res) row=getNextSQLResult(res);
                           }
                        if ((res) && (row) && (row[44]))
                           {
                           sprintf(result,"time=%s\nyear=%s\ntitle=%s\ngenre=%s\ncat=%s\nt01=%s\nt02=%s\nt03=%s\nt04=%s\nt05=%s\nt06=%s\nt07=%s\nt08=%s\nt09=%s\nt10=%s\nt11=%s\nt12=%s\nt13=%s\nt14=%s\nt15=%s\nt16=%s\nt17=%s\nt18=%s\nt19=%s\nt20=%s\nt21=%s\nt22=%s\nt23=%s\nt24=%s\nt25=%s\nt26=%s\nt27=%s\nt28=%s\nt29=%s\nt30=%s\nt31=%s\nt32=%s\nt33=%s\nt34=%s\nt35=%s\nartist=%s",
                                   row[2],row[3],row[5],row[6],row[8],row[9],row[10],row[11],row[12],row[13],row[14],row[15],row[16],row[17],row[18],row[19],row[20],row[21],row[22],row[23],row[24],row[25],row[26],row[27],row[28],row[29],row[30],row[31],row[32],row[33],row[34],row[35],row[36],row[37],row[38],row[39],row[40],row[41],row[42],row[43],row[44]);
                           sendResponseMessage(sender,PODDY_CODE_OK,result);
                           freeSQLQuery(res);
                           }
                        else sendResponseMessage(sender,PODDY_CODE_FAIL,"NOT FOUND");
                        }
                     else sendResponseMessage(sender,PODDY_CODE_FAIL,"FORMAT ERROR");
                     }
                  /******************************************************************************************************************/
                  else if (strstr(cmd,"INFO")==cmd) // get general information about the server
                     {
                     sprintf(result,"%s %s %s\n%s", FCOMMON_NAME,FCOMMON_VERSION,FCOMMON_URL,FCOMMON_COPYRIGHT);
                     sendResponseMessage(sender,PODDY_CODE_OK,result);
                     }
                  /******************************************************************************************************************/
                  else
                     {
                     sendResponseMessage(sender,PODDY_CODE_FAIL,"COMMAND ERROR");
                     fmtLogTime();
                     showLog(sysLog,"%s: Unknown command \"%s\" received from %s",logtime,cmd,sender->remoteIP);
//                  removeClient(&list, sender);
                     }
                  deinitSQLConnection();
                  }
               else
                  {
                  sendResponseMessage(sender,PODDY_CODE_FAIL,"DATABLOCK ERROR");
                  fmtLogTime();
                  showLog(sysLog,"%s: Invalid datablock received from %s",logtime,sender->remoteIP);
//                  removeClient(&list, sender);
                  }
               }
            sender->readLen=0;
            sender->packetComplete=false;            
            }
         }
      }
   }
   freeList(&list);
return true;
}



#ifdef ENV_WINDOWS
WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
WSADATA   wsaData;

#else
int main(int argc,char *argv[])
{
void*           hInstance=NULL;
#endif
#ifdef ENV_WINDOWS
int             argc;
char           *argv[100];

WSAStartup((MAKEWORD(1, 1)), &wsaData);
argc=splitCmdLine(lpCmdLine,argv);
#endif

getSwitches(argc,argv);
if (!settingsFile)
#ifdef ENV_WINDOWS
 settingsFile=strdup("poddyd.conf");
#else
 settingsFile=strdup("/etc/poddyd.conf");
#endif

#ifdef ENV_POSIX
if (verbose) printf("Loading settings %s...\n",settingsFile);
#endif
if (!loadSettings(settingsFile))
   {
#ifdef ENV_WINDOWS
   errno=GetLastError();
#else
   perror("");
#endif
   fprintf(stderr, "Error: could not load configuration, %d!\n",errno);
   }

   {
   srand(time(NULL));

   #ifdef ENV_POSIX
   if (verbose) printf("Starting mainloop...\n");
   #endif
   if (mainLoop(hInstance))
      {
      #ifdef ENV_WINDOWS
      while (!LeaveServer(0,0,0,0))
         {
         ThreadSleep(100);
         }
      #endif
      }
//   deinitSQLConnection();
   }
#ifdef ENV_WINDOWS
WSACleanup();
#endif
#ifdef ENV_POSIX
if (verbose) printf("Exiting mainloop...\n");
#endif

#ifdef ENV_POSIX
if (verbose) printf("Releasing buffered data...\n");
#endif
fmtLogTime();
showLog(sysLog,"%s: Exiting server!",logtime);               

if (sysLog) fclose(sysLog);
freeMem();
return 0;
}

