/*
 * mysqlaccess.c
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


#include <mysql.h>
#include <stdio.h>

#include "mysqlaccess.h"


/* server IP, user name, user password and name of the database that is accessed by the poddy-daemon */
static char  *server = "localhost",*user = "USERNAME",*password = "PASSWORD",*database = "PODDYDATABASE";
static MYSQL *conn=NULL;



/**
 * Initializes a connection to the MySQL server. This function uses the connection and
 * authentication data defined above. When a connection could be opened the related
 * connection handler is stored in the local variable "conn".
 * @return 0 in case the connection could be established successfully or
 *         ERR_MYSQL_CONNECT_FAILED otherwise
 */
int initSQLConnection()
{
   conn = mysql_init(NULL);
   if (!conn) return ERR_MYSQL_INIT_FAILED;

   if (!mysql_real_connect(conn, server,user, password, database, 0, NULL, 0)) 
   {
      fprintf(stderr, "%s\n", mysql_error(conn));
      return ERR_MYSQL_CONNECT_FAILED;
   }
   return 0;
}



/**
 * This function performs a query to the database
 * @param[in] query a text that contains the SQL statement for the query
 * @return the result of the query or NULL in case of an error
 */
void *doSQLQuery(char *query)
{
   if (!conn) return NULL;//ERR_MYSQL_NOT_INITIALIZED;
   if (mysql_query(conn,query)) 
   {
      fprintf(stderr, "%s\n", mysql_error(conn));
      return NULL;//ERR_MYSQL_QUERY_FAILED;
   }
   return mysql_use_result(conn);
//   return 0;
}



/**
 * When a request was sent to the data base that results in responses that consist
 * of several parts this function can be used repeatedly to fetch the next part of
 * that result
 * @param[in] res the result set to fetch the next part from
 * @return the next part of the resulting data or NULL in case of an error or in
 *         case there are no more data
 */
char **getNextSQLResult(void *res)
{
   if (!conn) return NULL;
   return mysql_fetch_row(res);
}



/**
 * After a query was processed completely and isn't used any longer its data have
 * to be released by calling this function.
 * @param[in] the result set to be released
 */
int freeSQLQuery(void *res)
{
   if (!res) return ERR_MYSQL_INVALID_RESULTSET;
   mysql_free_result(res);
   return 0;
}



/**
 * At the end of a communication cycle the connection to the data base server has
 * to be closed and deinitialized by calling this function
 */
int deinitSQLConnection(void)
{
   if (!conn) return ERR_MYSQL_NOT_INITIALIZED;
   mysql_close(conn);
   conn=NULL;
   return 0;
}



