/*
 * Copyright (c) 2004-2009 by VWP
 */

#include "maindefs.h"

#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>

#ifdef ENV_WINDOWS
 #include <windows.h>
#else
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <netdb.h>
#endif
 
#include "net.h"
#include "util.h"
#include "platforms.h"



int ssys_send(int sock, const char *msg, size_t len, int flags,int msecs,char *file,int line)
{
size_t rlen=0;
int    ctr=0,val;
#ifdef ENV_WINDOWS
int errno;
#else

errno=0;
#endif
while ((rlen<len) && (ctr<msecs))
   {
   val=send(sock,msg+rlen,len-rlen,flags);
   if (val>=0) rlen+=val;
   else
      {
      #ifndef ENV_WINDOWS
      if (errno==EAGAIN) ctr-=2; // in case of eagain we expect a longer send-timeout
      #else
      errno=WSAGetLastError();
      if (errno==WSAEWOULDBLOCK) ctr-=2; // in case of eagain we expect a longer send-timeout
      #endif
      else if (errno!=0)
         {
         rlen=-1;
         break;
         }
      #ifndef ENV_WINDOWS
      errno=0;
      #endif
      }
   if (rlen<len)
      {
      ThreadSleep(2);
      ctr+=2;
      }
   if ((rlen==0) && (ctr>msecs/2)) break;
   }
/*#ifndef SRV_WINDOWS
if (val<0) return 0;
#endif*/
return rlen;
}



void fc_closesocket (int sock)
{
if (shutdown (sock, SHUT_RDWR)<0) perror("shutdown failed");
#ifdef ENV_WINDOWS
closesocket(sock);
#else
if (close (sock)<0) perror("close failed");
#endif
}



#ifndef ENV_FCASTD

int connectto2(char *address,int connect_port)
{
struct sockaddr_in a;
struct hostent *host;
int             s;
unsigned long   nl;

s = socket (AF_INET, SOCK_STREAM, 0);
if (s<0)
   {
   perror("Can't create socket");
   #ifdef ENV_WINDOWS
   showLog(NULL,"Can't create socket, error: %d\n",GetLastError());
   #endif
   #ifdef ENV_SPARCSOLARIS
   printf("Error: %d\n",errno);
   #endif
   return -1;
   }

memset (&a, 0, sizeof (a));
a.sin_port = htons (connect_port);
a.sin_family = AF_INET;
a.sin_addr.s_addr =inet_addr(address);
if (a.sin_addr.s_addr==-1)
   {
   host = gethostbyname (address);
   if (host)
      {
      memcpy (&nl, host->h_addr, sizeof(unsigned long));
      a.sin_addr.s_addr = nl;
      }
   else
      {
      perror("Getting hostname");
      #ifdef ENV_SPARCSOLARIS
      printf("Getting hostname: %d\n",errno);
      #endif
      #ifdef ENV_WINDOWS
      showLog(NULL,"Getting hostname: %d\n",GetLastError());
      closesocket(s);
      #else
      close (s);
      #endif
      return -1;
      }
   }
if (connect (s, (struct sockaddr *) &a, sizeof (a)) < 0)
   {
   perror("No connection");
   #ifdef ENV_WINDOWS
   showLog(NULL,"No connection to server, error: %d\n",GetLastError());
   #endif
   #ifdef ENV_SPARCSOLARIS
   printf("Error: %d\n",errno);
   #endif
   fc_closesocket (s);
   return -1;
   }
return s;
}

#endif



#ifndef ENV_FCTERM
#ifndef ENV_FSUPPD
#ifndef ENV_FLYENCD

void setBlocking(int sock,char block)
{
int flags;

#ifndef ENV_WINDOWS
flags=fcntl(sock,F_GETFL, 0);
if (block) flags&=~O_NONBLOCK;
else flags|=O_NONBLOCK;
fcntl(sock,F_SETFL, flags);
#else
if (block) flags=0;
else flags=13;
ioctlsocket(sock,FIONBIO,(unsigned long*)&flags);
#endif
}



int acceptConnection (int sock, u_long * remote_ip)
{
unsigned int       len;
int                newsock;
struct sockaddr_in clientaddr;
unsigned long      ip;

len = sizeof (clientaddr);
newsock = accept (sock, (struct sockaddr *) &clientaddr, &len);
if (newsock >= 0)
   {
   ip = ntohl (clientaddr.sin_addr.s_addr);
   *remote_ip = ip;
   setBlocking(newsock,false);
   return newsock;
   }
return -1;
}



int listenOnPort(unsigned short port,char *bindToIP)
{
int           sock;
struct sockaddr_in name;
int           bindval,listenval;
unsigned long op;

sock = socket(AF_INET, SOCK_STREAM, 0);
if (sock >= 0)
   {
   op=1;
   setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(char *)&op,sizeof(op)); 
   name.sin_family = AF_INET;
   name.sin_port = htons (port);   
   name.sin_addr.s_addr=inet_addr(bindToIP); // htonl (INADDR_ANY);
   bindval = bind(sock, (struct sockaddr *) &name, sizeof (name));
   if (bindval >= 0)
      {
      listenval = listen (sock,3);
      if (listenval>=0)
         {
         setBlocking(sock,false);
         return sock;
         }
      else
         {
         perror("Can't listen on socket");
         #ifdef ENV_WINDOWS
         showLog(NULL,"%d\n",WSAGetLastError());
         #endif
         #ifdef ENV_SPARCSOLARIS
         printf("Error: %d\n",errno);
         #endif
         }
      }
   else
      {
      perror("Can't bind socket to port");
      #ifdef ENV_WINDOWS
      showLog(NULL,"%d\n",WSAGetLastError());
      #endif
      #ifdef ENV_SPARCSOLARIS
      printf("Error: %d\n",errno);
      #endif
      }
   fc_closesocket (sock);
   }
else
   {
   perror("Can't create socket");
   #ifdef ENV_WINDOWS
   showLog(NULL,"%d\n",WSAGetLastError());
   #endif
   #ifdef ENV_SPARCSOLARIS
   printf("Error: %d\n",errno);
   #endif
   }
return -1;
}



int connect_udp (u_long ip, int port)
{
struct sockaddr_in rem_addr;
int                sock,err;

rem_addr.sin_family = AF_INET;
rem_addr.sin_port = htons ((u_short) port);
*(u_long *) (&rem_addr.sin_addr) = htonl (ip);
sock = socket (PF_INET, SOCK_DGRAM, 0);
if (sock < 0) return -1;
err = connect (sock, (struct sockaddr *) &rem_addr, sizeof (rem_addr));
if (err < 0) return -1;
//err = 1;
setBlocking(sock,false);
return sock;
}

#endif
#endif
#endif

