package base;

import java.io.*;
import java.util.*;

import javax.swing.DefaultListModel;



public class CDPlayManager extends Thread 
{
   public static final int   CMD_SHUFFLE_ALBUM=1,CMD_PLAY_ALBUM=2, CMD_PLAY_ARTIST=3,          CMD_SHUFFLE_ARTIST=4,
                             CMD_PLAY_CDS=5,     CMD_SHUFFLE_CDS=6,CMD_PLAY_ALBUM_FROM_TRACK=7;
   
   private int               workCmd=0,itemNumSelected=-1;
   private boolean           randomPlay=false;
   private CDPlay            cdPlay;
   private SQLLocalAccess    SQL;
   private BottomField       BottomPanel;
   private TopField          TopPanel;
   private Vector<MenuField> menuList;
   
   public CDPlayManager(int workCmd,SQLLocalAccess SQL,BottomField BottomPanel,TopField TopPanel,Vector<MenuField> menuList,int itemNumSelected)
   {
	   this.workCmd=workCmd;
	   this.SQL=SQL;
	   this.BottomPanel=BottomPanel;
	   this.menuList=menuList;
	   this.TopPanel=TopPanel;
	   this.itemNumSelected=itemNumSelected;
	   start();
   }
   
   
   
   public void stopPlay()
   {
	  randomPlay=false;
      if (cdPlay!=null)
      {
    	   cdPlay.stopPlay(true,true);
	      cdPlay=null;      
      }	   
   }
   
   
   
   public void nextTrack()
   {
      if ((workCmd==CMD_SHUFFLE_CDS) || (workCmd==CMD_SHUFFLE_ARTIST) || (workCmd==CMD_SHUFFLE_ALBUM))
      {
         cdPlay.stopPlay(true,true);
         cdPlay=null;
      }
      else if (((workCmd==CMD_PLAY_CDS) || (workCmd==CMD_PLAY_ARTIST)) && (cdPlay.currentTrack()+1==cdPlay.lastTrack()))
      {
         cdPlay.stopPlay(true,true);
         cdPlay=null;
      }       
      else if (cdPlay!=null) cdPlay.nextTrack();  
   }
   
   
   
   public void previousTrack()
   {
      if ((workCmd==CMD_SHUFFLE_CDS) || (workCmd==CMD_SHUFFLE_ARTIST) || (workCmd==CMD_SHUFFLE_ALBUM))
      {
         cdPlay.stopPlay(true,true);
         cdPlay=null;
      }
      else if (cdPlay!=null) cdPlay.previousTrack();      
   }
   
   
   
   public int getWorkCmd()
   {
	   return workCmd;
   }
   
	private void playAlbumDirect(String albumDir,CDInfo cdInfo,int startPos,boolean turnOffBottomPanelAtEnd)
	{
      String[]         tracks;
      File             cddir=new File(albumDir);
      DefaultListModel result;
      TrackInfo        trackInfo=new TrackInfo(-1);
      
      tracks=cddir.list(new DirFilter());
      if ((tracks==null) || (tracks.length==0)) return;
      Arrays.sort(tracks);
      if (cdPlay!=null) cdPlay.stopPlay(true,true);
      cdPlay=null;
      result=SQL.doSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" WHERE artist='"+cdInfo.artist+"' and album='"+cdInfo.album+"'");
      if ((result!=null) && (result.size()>0)) cdInfo=(CDInfo)result.elementAt(0);
      else cdInfo.set(menuList.elementAt(itemNumSelected).getCDInfo());
      TopPanel.setText(cdInfo.artist);
      cdPlay=new CDPlay(cddir.getAbsolutePath(),tracks,cdInfo,trackInfo,startPos,turnOffBottomPanelAtEnd);   
	}

	
	
	private void playArtistDirect(String artistdir,CDInfo cdInfo)
	{
      String[]         cds;
      File             artists=new File(artistdir);

      randomPlay=true;
      cds=artists.list(new DirFilter());
      if ((cds==null) || (cds.length==0)) return;
      Arrays.sort(cds);
      for (int i=0; i<cds.length; i++)
      {
         cdInfo.album=cds[i];
         playAlbumDirect(artistdir+"/"+cds[i],cdInfo,0,false);
         while ((randomPlay) && (cdPlay!=null) && (cdPlay.isPlaying))
         {
            com.vwp.util.Tools.threadSleep(250);
         }
         if (!randomPlay) break;
      }	   
	}
	
	
	private void playAlbumShuffle(String albumDir,CDInfo cdInfo,boolean onlyOneTrack)
	{
      
      String[]         tracks;
      File             cddir=new File(albumDir);
      int              sel,lastSel=-1;
      Random           rnd=new Random();
      DefaultListModel result;
      TrackInfo        trackInfo=new TrackInfo(-1);

      randomPlay=true;
      rnd.setSeed(System.currentTimeMillis());
      
      tracks=cddir.list(new DirFilter());
      if ((tracks==null) || (tracks.length==0)) return;
      if (cdPlay!=null) cdPlay.stopPlay(true,true);
      cdPlay=null;
      do
      {
         do
         {
            sel=Math.abs(rnd.nextInt()/100)%tracks.length;
         }
         while ((sel==lastSel) && (tracks.length>1));
         lastSel=sel;
         result=SQL.doSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" WHERE artist='"+cdInfo.artist+"' and album='"+cdInfo.album+"'");
         if ((result!=null) && (result.size()>0)) cdInfo=(CDInfo)result.elementAt(0);
         else cdInfo.set(menuList.elementAt(itemNumSelected).getCDInfo());
         trackInfo.name=tracks[sel];
         trackInfo.trackNum=sel;
         TopPanel.setText(cdInfo.artist);
         cdPlay=new CDPlay(cddir.getAbsolutePath()+"/"+tracks[sel],cdInfo,trackInfo);
//               ((MenuField)menuList.elementAt(itemNumSelected)).getTrackInfo());
         while ((randomPlay) && (cdPlay!=null) && (cdPlay.isPlaying))
//         do
         {
            com.vwp.util.Tools.threadSleep(250);
            if (!randomPlay) break;
         }
//         while (cdPlay!=null);
      }
      while ((randomPlay) && (!onlyOneTrack));
	}

	
	
	private void playArtistShuffle(String artistDir,CDInfo cdInfo,boolean onlyOneAlbum)
	{
      String[]         albums;
      File             artists=new File(artistDir);
      int              sel,lastSel=-1;
      Random           rnd=new Random();

      randomPlay=true;
      rnd.setSeed(System.currentTimeMillis());

      albums=artists.list(new DirFilter());
      if ((albums==null) || (albums.length==0)) return;
      do
      {
         do
         {
            sel=Math.abs(rnd.nextInt()/100)%albums.length;
         }
         while ((sel==lastSel) && (albums.length>1));
         lastSel=sel;
         cdInfo.album=albums[sel];
         playAlbumShuffle(artistDir+"/"+albums[sel],cdInfo,true);
         while ((randomPlay) && (cdPlay!=null) && (cdPlay.isPlaying))
         {
            com.vwp.util.Tools.threadSleep(250);
         }
         if (!randomPlay) break;
      }
      while ((randomPlay) && (!onlyOneAlbum));	   
	}

   
   public void run()
   {
	     if (workCmd==CMD_PLAY_CDS)
         {
            String[]  artists;
            File      all=new File(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/cds");
            CDInfo    cdInfo=new CDInfo();

            BottomPanel.setLockHidePanel(true);
            randomPlay=true;
            artists=all.list(new DirFilter());
            Arrays.sort(artists);
            for (int i=0; i<artists.length; i++)
            {
               cdInfo.artist=artists[i];
//               ((MenuField)menuList.elementAt(itemNumSelected)).readAlbumInfo(((MenuField)menuList.elementAt(itemNumSelected)).getDir()+"/"+cds[i],trackInfo);
               playArtistDirect(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/cds"+"/"+artists[i],cdInfo);
               while ((randomPlay) && (cdPlay!=null) && (cdPlay.isPlaying))
               {
                  com.vwp.util.Tools.threadSleep(250);
               }
               if (!randomPlay) break;
            }
            BottomPanel.setLockHidePanel(false);
            BottomPanel.hidePanel("MainWin.run()");
         }
         else if (workCmd==CMD_PLAY_ARTIST)
		 {
            BottomPanel.setLockHidePanel(true);
            playArtistDirect(menuList.elementAt(itemNumSelected).getDir(),
                             menuList.elementAt(itemNumSelected).getCDInfo());
            BottomPanel.setLockHidePanel(false);
            BottomPanel.hidePanel("MainWin.run()");
		 }
		 else if (workCmd==CMD_PLAY_ALBUM)
		 {
		    CDInfo cdInfo;
		      
		    cdInfo=menuList.elementAt(itemNumSelected).getCDInfo();

          BottomPanel.setLockHidePanel(true);
    	    playAlbumDirect(menuList.elementAt(itemNumSelected).getDir(),cdInfo,0,true);
		 }
         else if (workCmd==CMD_PLAY_ALBUM_FROM_TRACK)
         {
            int    menuNum=-1,i;
            
            BottomPanel.setLockHidePanel(true);
            for (i=0; i<menuList.size(); i++)
            {
               if (menuList.elementAt(i).getID()==PoddyFrame.ID_CD_ARTIST_ALBUM_TRACK)
               {
                  menuNum=i;
                  break;
               }
            }
            if (menuNum>-1)
            {
               menuList.elementAt(itemNumSelected).getTrackInfo().trackNum=itemNumSelected-menuNum;
               playAlbumDirect(menuList.elementAt(itemNumSelected).getDir(),
                               menuList.elementAt(itemNumSelected).getCDInfo(),
                               menuList.elementAt(itemNumSelected).getTrackInfo().trackNum,true);
            }
//            BottomPanel.setLockHidePanel(false);
//            BottomPanel.hidePanel("MainWin.run()");
         }
         else if (workCmd==CMD_SHUFFLE_CDS)
         {
            String[]  artists;
            File      allfile=new File(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/cds");
            CDInfo    cdInfo=new CDInfo();
            int       sel,lastSel=-1;
            Random    rnd=new Random();

            randomPlay=true;
            BottomPanel.setLockHidePanel(true);
            rnd.setSeed(System.currentTimeMillis());

            randomPlay=true;
            artists=allfile.list(new DirFilter());
            if ((artists==null) || (artists.length==0)) return;
            do
            {
               do
               {
                  sel=Math.abs(rnd.nextInt()/100)%artists.length;
               }
               while ((sel==lastSel) && (artists.length>1));
               lastSel=sel;
               cdInfo.artist=artists[sel];
//               ((MenuField)menuList.elementAt(itemNumSelected)).readAlbumInfo(((MenuField)menuList.elementAt(itemNumSelected)).getDir()+"/"+cds[i],trackInfo);
               playArtistShuffle(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/cds"+"/"+artists[sel],cdInfo,true);
               while ((randomPlay) && (cdPlay!=null) && (cdPlay.isPlaying))
               {
                  com.vwp.util.Tools.threadSleep(250);
               }
               if (!randomPlay) break;
            }
            while(randomPlay);
            BottomPanel.setLockHidePanel(false);
            BottomPanel.hidePanel("MainWin.run()");
         }
         else if (workCmd==CMD_SHUFFLE_ARTIST)
         {
            BottomPanel.setLockHidePanel(true);
            playArtistShuffle(menuList.elementAt(itemNumSelected).getDir(),
                              menuList.elementAt(itemNumSelected).getCDInfo(),
                               false);
            BottomPanel.setLockHidePanel(false);
            BottomPanel.hidePanel("MainWin.run()");
         }
		   else if (workCmd==CMD_SHUFFLE_ALBUM)
         {
            BottomPanel.setLockHidePanel(true);
		      playAlbumShuffle(menuList.elementAt(itemNumSelected).getDir(),
                             menuList.elementAt(itemNumSelected).getCDInfo(),false);
            BottomPanel.setLockHidePanel(false);
            BottomPanel.hidePanel("MainWin.run()");
         }   
   }
}
