/*
 * ConfigData.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

import java.io.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

/**
 * This class handles reading of the configuration file and stores the values read from
 * there. these values are accessed by the application not only in package system.
 *
 */
public class ConfigData
{
   private static final String PREFS_NODE="poddy/";
   public int           x_offset=10, y_offset=10,x_size=700,y_size=500,av_offset=0;
   public boolean       is16_9=true,use_movies=true,use_series=true,use_videos=true,use_cds=true,use_vcr=true,use_usb=true;
   public String        cd_device="/dev/cdrom",dvd_device="/dev/dvd",video_device="/dev/video0",audio_device="/dev/dsp",usb_mount="/mnt/flash",
                        video_dir="/path/to/videodir",main_dir="/path/to/maindir",bgimage=null,tv_norm="PAL",freq_table="europe-west",
                        uname=null,pwd=null;
   private String       checkPrefs=null;
   
   public ConfigData()
   {
   }
   
   

   /**
    * This is a simple encoding method for the password - it is not very secure but good enough to keep
    * stupid password scanners away (and to help the user in case he has used the same password for more
    * important services
    * @param in the plain password
    * @return the encoded password
    */
   private String encodePwd(String in)
   {
	   String out="";
	   byte[] b;
	   int    i,v;
	   
	   b=in.getBytes();
	   for (i=0; i<b.length; i++)
	   {
	      v=((b[i]+i)^73)-i;
	      if (v<10) out=out+"00"+v;
	      else if (v<100) out=out+"0"+v;
	      else out=out+v;
	   }
	   return out;
   }
   
   
   
   /**
    * This is a simple encoding method for the password - it is not very secure but good enough to keep
    * stupid password scanners away (and to help the user in case he has used the same password for more
    * important services
    * @param in the encoded password
    * @return the plain password
    */
   private String decodePwd(String in)
   {
	   byte[] b;
	   int    i,v;
	   
	   if ((in==null) || (in.length()<=0)) return null;
	   b=new byte[in.length()/3];
	   for (i=0; i<b.length; i++)
	   {
	      v=Integer.parseInt(in.substring(i*3,i*3+3));
	      b[i]=(byte)(((v+i)^73)-i);
	   }
	   return new String(b);
   }

   
   
   /**
    * Reads and parses the configuration file
    * @return true when the configuration could be read successfully, false otherwise
    */
   public boolean readConfig()
   {
      BufferedReader in;
      String     line;
      
      Preferences prefs = Preferences.systemRoot();
      
      x_offset    =prefs.getInt(PREFS_NODE+"x-offset",x_offset);
      y_offset    =prefs.getInt(PREFS_NODE+"y-offset",y_offset);
      x_size      =prefs.getInt(PREFS_NODE+"x-size",x_size);
      y_size      =prefs.getInt(PREFS_NODE+"y-size",y_size);
      cd_device   =prefs.get(PREFS_NODE+"cd-device",cd_device);
      dvd_device  =prefs.get(PREFS_NODE+"dvd-device",dvd_device);
      video_device=prefs.get(PREFS_NODE+"video-device",video_device);      
      audio_device=prefs.get(PREFS_NODE+"audio-device",audio_device);      
      usb_mount  =prefs.get(PREFS_NODE+"usb-device",usb_mount);      
      is16_9      =prefs.getBoolean(PREFS_NODE+"aspect",is16_9);
      video_dir   =prefs.get(PREFS_NODE+"video-dir",video_dir);
      main_dir    =prefs.get(PREFS_NODE+"main-dir",main_dir);
      use_videos  =prefs.getBoolean(PREFS_NODE+"use-videos",use_videos);
      use_movies  =prefs.getBoolean(PREFS_NODE+"use-movies",use_movies);
      use_series  =prefs.getBoolean(PREFS_NODE+"use-series",use_series);
      use_cds     =prefs.getBoolean(PREFS_NODE+"use-cds",use_cds);
      use_vcr     =prefs.getBoolean(PREFS_NODE+"use-vcr",use_vcr);
      use_usb     =prefs.getBoolean(PREFS_NODE+"use-usb",use_usb);
      bgimage     =prefs.get(PREFS_NODE+"bgimage",bgimage);
      tv_norm     =prefs.get(PREFS_NODE+"tv-norm",tv_norm);
      freq_table  =prefs.get(PREFS_NODE+"freq-table",freq_table);
      av_offset   =prefs.getInt(PREFS_NODE+"av-offset",av_offset);
      uname       =prefs.get(PREFS_NODE+"uname",null);
      pwd=decodePwd(prefs.get(PREFS_NODE+"pwd",null));
      checkPrefs  =prefs.get(PREFS_NODE+"check_prefs",null);
      
      if (checkPrefs==null) try
      {
         in=new BufferedReader(new FileReader("poddy.conf"));
         
         do
         {
            line=in.readLine();
            if ((line!=null) && (line.length()>0))
            {
               line=line.trim();
               
               if      (line.startsWith("x-offset="))     x_offset=Integer.parseInt(line.substring(9));
               else if (line.startsWith("y-offset="))     y_offset=Integer.parseInt(line.substring(9));
               else if (line.startsWith("x-size="))       x_size=  Integer.parseInt(line.substring(7));
               else if (line.startsWith("y-size="))       y_size=  Integer.parseInt(line.substring(7));
               else if (line.startsWith("cd-device="))    cd_device=line.substring(10);
               else if (line.startsWith("dvd-device="))   dvd_device=line.substring(11);
               else if (line.startsWith("video-device=")) video_device=line.substring(13);
               else if (line.startsWith("audio-device=")) audio_device=line.substring(13);
               else if (line.startsWith("usb-device="))   usb_mount=line.substring(11);
               else if (line.startsWith("aspect=16:9"))   is16_9=true;
               else if (line.startsWith("aspect=4:3"))    is16_9=false;
               else if (line.startsWith("video-dir="))    video_dir=line.substring(10);
               else if (line.startsWith("main-dir="))     main_dir=line.substring(9);
               else if (line.startsWith("use-videos=1"))  use_videos=true;
               else if (line.startsWith("use-videos=0"))  use_videos=false;
               else if (line.startsWith("use-movies=1"))  use_movies=true;
               else if (line.startsWith("use-movies=0"))  use_movies=false;
               else if (line.startsWith("use-series=1"))  use_series=true;
               else if (line.startsWith("use-series=0"))  use_series=false;
               else if (line.startsWith("use-cds=1"))     use_cds=true;
               else if (line.startsWith("use-cds=0"))     use_cds=false;
               else if (line.startsWith("use-vcr=1"))     use_vcr=true;
               else if (line.startsWith("use-vcr=0"))     use_vcr=false;
               else if (line.startsWith("use-usb=1"))     use_usb=true;
               else if (line.startsWith("use-usb=0"))     use_usb=false;
               else if (line.startsWith("bgimage="))      bgimage=line.substring(8);
               else if (line.startsWith("tv-norm="))      tv_norm=line.substring(8);
               else if (line.startsWith("freq-table="))   freq_table=line.substring(11);
               else if (line.startsWith("av-offset="))    av_offset=Integer.parseInt(line.substring(10));
               else if (line.startsWith("uname="))        uname=line.substring(6);
               else if (line.startsWith("pwd="))          pwd=decodePwd(line.substring(4));
            }
         }
         while ((line!=null) && (line!=""));
         in.close();
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return false;
      }
      return true;
   }

   /**
    * Writes the configuration file
    * @return true when the configuration could be written successfully, false otherwise
    */
   public boolean writeConfig()
   {
      Preferences prefs = Preferences.systemRoot();
	      
	  prefs.putInt(PREFS_NODE+"x-offset",x_offset);
	  prefs.putInt(PREFS_NODE+"y-offset",y_offset);
	  prefs.putInt(PREFS_NODE+"x-size",x_size);
	  prefs.putInt(PREFS_NODE+"y-size",y_size);
	  prefs.put(PREFS_NODE+"cd-device",cd_device);
	  prefs.put(PREFS_NODE+"dvd-device",dvd_device);
	  prefs.put(PREFS_NODE+"video-device",video_device);      
      prefs.put(PREFS_NODE+"audio-device",audio_device);      
      prefs.put(PREFS_NODE+"usb-device",usb_mount);      
	  prefs.putBoolean(PREFS_NODE+"aspect",is16_9);
	  prefs.put(PREFS_NODE+"video-dir",video_dir);
	  prefs.put(PREFS_NODE+"main-dir",main_dir);
	  prefs.putBoolean(PREFS_NODE+"use-videos",use_videos);
	  prefs.putBoolean(PREFS_NODE+"use-movies",use_movies);
	  prefs.putBoolean(PREFS_NODE+"use-series",use_series);
	  prefs.putBoolean(PREFS_NODE+"use-cds",use_cds);
	  prefs.putBoolean(PREFS_NODE+"use-vcr",use_vcr);
	  prefs.putBoolean(PREFS_NODE+"use-usb",use_usb);
	  prefs.put(PREFS_NODE+"bgimage",bgimage);
	  prefs.put(PREFS_NODE+"tv-norm",tv_norm);
	  prefs.put(PREFS_NODE+"freq-table",freq_table);
      prefs.putInt(PREFS_NODE+"av-offset",av_offset);
	  prefs.put(PREFS_NODE+"uname",uname);
	  prefs.put(PREFS_NODE+"pwd",encodePwd(pwd));
      prefs.put(PREFS_NODE+"check_prefs","blubb");
	  try 
	  {
         prefs.flush();
      } 
	  catch (BackingStoreException e) 
	  {
         e.printStackTrace();

         BufferedWriter out;
      
         try
         {
            out=new BufferedWriter(new FileWriter("poddy.conf"));
            
          	 out.write("x-offset="+x_offset);         out.newLine();
          	 out.write("y-offset="+y_offset);         out.newLine();
          	 out.write("x-size="+x_size);             out.newLine();
          	 out.write("y-size="+y_size);             out.newLine();
          	 out.write("cd-device="+cd_device);       out.newLine();
          	 out.write("dvd-device="+dvd_device);     out.newLine();
          	 out.write("video-device="+video_device); out.newLine();
          	 out.write("audio-device="+audio_device); out.newLine();
          	 out.write("usb-device="+usb_mount);     out.newLine();
          	 if (is16_9) out.write("aspect=16:9");
          	 else out.write("aspect=4:3");         
          	                                          out.newLine();
          	 out.write("video-dir="+video_dir);       out.newLine();
          	 out.write("main-dir="+main_dir);         out.newLine();
          	 if (use_videos) out.write("use-videos=1");
          	 else out.write("use-videos=0");
           	                                      out.newLine();
          	 if (use_movies) out.write("use-movies=1");
          	 else out.write("use-movies=0");
          	                                          out.newLine();
          	 if (use_series) out.write("use-series=1");
          	 else out.write("use-series=0");
          	                                          out.newLine();
          	 if (use_cds) out.write("use-cds=1");
          	 else out.write("use-cds=0");
         	                                          out.newLine();
            if (use_vcr) out.write("use-vcr=1");
            else out.write("use-vcr=0");
                                                     out.newLine();
            if (use_usb) out.write("use-usb=1");
            else out.write("use-usb=0");
                                                     out.newLine();
            out.write("bgimage="+bgimage);           out.newLine();
            out.write("tv-norm="+tv_norm);           out.newLine();
            out.write("freq-table="+freq_table);     out.newLine();
            out.write("av-offset="+av_offset);       out.newLine();
            out.write("uname="+uname);               out.newLine();
            out.write("pwd="+encodePwd(pwd));        out.newLine();         
            out.close();
         }
         catch (IOException e2)
         {
            e2.printStackTrace();
            return false;
         }

      }
      return true;
   }
}
