/*
 * DVDInfo.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;



/**
 * This is a storage class for DVD information like they are used to identify a DVD, to
 * display information about it to the user and to find it for replaying
 */
public class DVDInfo extends MediaInfo
{
   public int    idx=-1,time=0,year=0,season=0,disc=0;
   public long   crcid=0;
   public String genre=null,imgUrl=null,ean=null;
   public boolean localOnly=false;
   
   
   
  /**
   * The default constructor of class DVDInfo
   */
   public DVDInfo()
   {
   }

   
   
  /**
   * The standard constructor for a object of type DVDInfo
   * @param idx an unique identifier for a DVD
   * @param ean the EAN (European Article Number) of a DVD
   * @param time the total runtime (in minutes) of the whole movie of the DVD; in case there
   *        are several movies/several parts on one DVD this value specifies the total time of all
   *        of them
   * @param year the year this DVD was released
   * @param crcid a checksum that uniquely identifies a DVD, the algorithm for that checksum is specified
   *        by http://dvd-db.dk
   * @param title the title of the DVD
   * @param genre the genre of the DVD
   * @param season the season this DVD belongs to; this parameter applies to series only and not to movies
   */
   public DVDInfo(int idx,String ean,int time,int year,long crcid,String title, String genre,int season,
           int disc)
   {
      this.idx=idx;
      this.title=title;
      this.genre=genre;
      this.year=year;
      this.time=time;
      this.ean=ean;
      this.crcid=crcid;   
      this.season=season;
      this.disc=disc;
   }
   
   

  /**
   * This method sets new DVD data depending on the in coming values: in case some fields of the given object
   * are not set the original values of the own object will be left untouched
   * @param info the new DVDInfo data that have to be set
   */   
   public void set(DVDInfo info)
   {
      super.set(info);
      if (info==null) return;
      if (info.idx!=0) this.idx=info.idx; 
      if (info.ean!=null) this.ean=info.ean; 
      if (info.time!=0) this.time=info.time; 
      if (info.year!=0) this.year=info.year; 
      if (info.crcid!=0) this.crcid=info.crcid; 
      if (info.season!=0) this.season=info.season; 
      if (info.disc!=0) this.disc=info.disc; 
      if (info.genre!=null) this.genre=info.genre; 
      this.localOnly=info.localOnly; 
   }
   
}
