/*
 * DialoguePanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;



@SuppressWarnings("serial")
/**
 * This class provides a simple panel that display some text. It can be used e.g. for error messages.
 */
public class DialoguePanel extends CustomPanel
{
   public static final int MSG_ERROR=1; /** error message ID */

  /**
   * The default constructor
   */	
   DialoguePanel()
   {
      
   }

   
   
   /**
    * This is the standard constructor for creating a new dialogue panel. The resulting object
    * has to be added by calling PoddyFrame.setCustomPanel();
    * @param text the text that has to be displayed
    * @param msgid an identifier that describes which kind of message has to be shown; this parameter
    *        influences the design of the panel. Here a constant of type MSG_xxx has to be given (please
    *        refer constant definitions above)
    */
   public DialoguePanel(String text, int msgid)
   {
      super();

      JPanel innerPanel=new JPanel();  
      PoddyFrame.MyWin.setLCDErrorText(text);
      
      mainPanel.setLayout(new FlowLayout(FlowLayout.CENTER,10,10));
      add(innerPanel);/*,new GridBagConstraints(0,0, 1, 1, 0.9, 0.5,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));*/
      innerPanel.setLayout(new BorderLayout(5,5));
      if (msgid==MSG_ERROR)
      {
         JLabel title=new JLabel(" "+PoddyFrame.MyWin._m.getString("error")+" ");
         title.setForeground(PoddyFrame.errorcolor);
         innerPanel.setBorder(new LineBorder(Color.BLACK));
         innerPanel.add(title,BorderLayout.NORTH);
      }
      JLabel err=new JLabel(" "+text+" ");
      err.setForeground(PoddyFrame.errorcolor);
      innerPanel.add(err,BorderLayout.CENTER);
   }
   
   
}
