/*
 * JFixedPanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

import javax.swing.*;
import java.awt.*;



@SuppressWarnings("serial")
/**
 * This class provides a special panel type with a fixed width that can't be
 * modified by any operation. It has to be used instead of normal JPanels everywhere
 * in the user interface. It ensures that - although a JScrollPane is used - there is
 * no horizontal scrolling necessary which would make usage complicated on a TV screen.
 * Beside of that this class provides exactly the same functionality like a standard
 * JPanel and has to be used in the same way. 
 */
public class JFixedPanel extends JPanel
{
   
  /**
   * The standard constructor for a fixed width panel
   */
   public JFixedPanel()
   {
      setOpaque(false);
   }
   
   
   
  /**
   * This method overwrites the related method of the JPanel in order to inform Swing about
   * the (fixed horizontal) size. Please refer to the documentation of JPanel for more details.
   * @return the minimum allowed size of this panel
   */
   public Dimension getMinimumSize()
   {
      return new Dimension(PoddyFrame.MyWin.getSize().width,super.getMinimumSize().height);
   }



  /**
   * This method overwrites the related method of the JPanel in order to inform Swing about
   * the (fixed horizontal) size. Please refer to the documentation of JPanel for more details.
   * @return the maximum allowed size of this panel
   */
   public Dimension getMaximumSize()
   {
      return new Dimension(PoddyFrame.MyWin.getSize().width,super.getMaximumSize().height);
   }



  /**
   * This method overwrites the related method of the JPanel in order to inform Swing about
   * the (fixed horizontal) size. Please refer to the documentation of JPanel for more details.
   * @return the preferred size of this panel
   */
   public Dimension getPreferredSize()
   {
      return new Dimension(PoddyFrame.MyWin.getSize().width,super.getPreferredSize().height);
   }



  /**
   * This method overwrites the related method of the JPanel in order to inform Swing about
   * the (fixed horizontal) size. Please refer to the documentation of JPanel for more details.
   * @return the current size of this panel
   */
   public Dimension getSize()
   {
      return new Dimension(PoddyFrame.MyWin.getSize().width,super.getSize().height);
   }

   
   
  /**
   * Using this method the selection state and therefore the layout and design of the panel
   * can be changed
   * @param selected true in case the panel is selected so that it has to be highlighted, false
   *        otherwise
   */
   public void setSelected(boolean selected)
   {
      if (selected)
      {
         setOpaque(true);
         setBackground(PoddyFrame.highlight);
      }
      else
      {
         setOpaque(false);
         setBackground(PoddyFrame.bgcolor);
      }
   }
}
