/*
 * KeyHelpPanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package base;

import java.awt.*;
import java.io.File;

import javax.swing.*;

import javax.swing.border.*;

@SuppressWarnings("serial")
/**
 * This class realizes the help that describes the keys and usage of the application. This panel is displayed
 * to the user at first application start up. It is displayed for every start of the application as long as
 * the user does not select the "Do not show again" button.<BR>
 * Here within this class the images are loaded, the horizontal rows of images and texts are created and the
 * selection of elements in vertical direction is managed.
 */
public class KeyHelpPanel extends CustomPanel 
{
	private static final int MAX_HELP_ITEMS=9;
	private JButton          upButton=new JButton(),downButton=new JButton(),leftButton=new JButton(),rightButton=new JButton(),
	                         retButton=new JButton(),f11Button=new JButton(),f12Button=new JButton(),pgupButton=new JButton(),
	                         aButton=new JButton(),numButton=new JButton(),rcButton=new JButton();
	private JButton          okButton=new JButton(PoddyFrame.MyWin._m.getString("ok")),
	                         offButton=new JButton(PoddyFrame.MyWin._m.getString("help_neveragain")); 
	private JTextArea[]      helpArea=new JTextArea[MAX_HELP_ITEMS];



       /**
        * This is the standard constructor. Here the complete layout of the panel is implemented including
	* loading of the images
	*/
	public KeyHelpPanel()
	{
	   	try
	   	{
  	   	   upButton.setIcon(new ImageIcon(getClass().getResource("/img/cup.png")));
  	   	   pgupButton.setIcon(new ImageIcon(getClass().getResource("/img/pgup.png")));
	   	   downButton.setIcon(new ImageIcon(getClass().getResource("/img/cdown.png")));
	   	   leftButton.setIcon(new ImageIcon(getClass().getResource("/img/cleft.png")));
	   	   rightButton.setIcon(new ImageIcon(getClass().getResource("/img/cright.png")));
	   	   retButton.setIcon(new ImageIcon(getClass().getResource("/img/return.png")));
	   	   f11Button.setIcon(new ImageIcon(getClass().getResource("/img/f11.png")));
	   	   f12Button.setIcon(new ImageIcon(getClass().getResource("/img/f12.png")));
	   	   aButton.setIcon(new ImageIcon(getClass().getResource("/img/a.png")));
	   	   numButton.setIcon(new ImageIcon(getClass().getResource("/img/1.png")));
	   	   rcButton.setIcon(new ImageIcon(getClass().getResource("/img/rc.png")));
	   	}
	   	catch (Exception e)
	   	{
	   	   e.printStackTrace();
	   	}

	   	JLabel label=new JLabel(PoddyFrame.MyWin._m.getString("help_keys"));
	   	label.setForeground(Color.WHITE);
	   	add(label, new GridBagConstraints(0,0,4,1,1.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 

	   	helpArea[0]=new JTextArea(PoddyFrame.MyWin._m.getString("help_updown"));
	   	helpArea[0].setWrapStyleWord(true);
	   	helpArea[0].setLineWrap(true);
	   	helpArea[0].setBorder(new EtchedBorder());
	   	helpArea[0].setEditable(false);
	   	upButton.setBorder(null);
	   	add(upButton, new GridBagConstraints(0,1,1,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	downButton.setBorder(null);
	   	add(downButton, new GridBagConstraints(1,1,1,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0)); 
	   	add(helpArea[0], new GridBagConstraints(2,1,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[1]=new JTextArea(PoddyFrame.MyWin._m.getString("help_leftright"));
	   	helpArea[1].setWrapStyleWord(true);
	   	helpArea[1].setLineWrap(true);
	   	helpArea[1].setBorder(new EtchedBorder());
	   	helpArea[1].setEditable(false);
	   	leftButton.setBorder(null);
	   	add(leftButton, new GridBagConstraints(0,2,1,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	rightButton.setBorder(null);
	   	add(rightButton, new GridBagConstraints(1,2,1,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0)); 
	   	add(helpArea[1], new GridBagConstraints(2,2,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[2]=new JTextArea(PoddyFrame.MyWin._m.getString("help_return"));
	   	helpArea[2].setWrapStyleWord(true);
	   	helpArea[2].setLineWrap(true);
	   	helpArea[2].setBorder(new EtchedBorder());
	   	helpArea[2].setEditable(false);
	   	retButton.setBorder(null);
	   	add(retButton, new GridBagConstraints(0,3,2,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[2], new GridBagConstraints(2,3,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[3]=new JTextArea(PoddyFrame.MyWin._m.getString("help_pgup"));
	   	helpArea[3].setWrapStyleWord(true);
	   	helpArea[3].setLineWrap(true);
	   	helpArea[3].setBorder(new EtchedBorder());
	   	helpArea[3].setEditable(false);
	   	pgupButton.setBorder(null);
	   	add(pgupButton, new GridBagConstraints(0,4,2,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[3], new GridBagConstraints(2,4,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[4]=new JTextArea(PoddyFrame.MyWin._m.getString("help_f11"));
	   	helpArea[4].setWrapStyleWord(true);
	   	helpArea[4].setLineWrap(true);
	   	helpArea[4].setBorder(new EtchedBorder());
	   	helpArea[4].setEditable(false);
	   	f11Button.setBorder(null);
	   	add(f11Button, new GridBagConstraints(0,5,2,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[4], new GridBagConstraints(2,5,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[5]=new JTextArea(PoddyFrame.MyWin._m.getString("help_f12"));
	   	helpArea[5].setWrapStyleWord(true);
	   	helpArea[5].setLineWrap(true);
	   	helpArea[5].setBorder(new EtchedBorder());
	   	helpArea[5].setEditable(false);
	   	f12Button.setBorder(null);
	   	add(f12Button, new GridBagConstraints(0,6,2,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[5], new GridBagConstraints(2,6,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[6]=new JTextArea(PoddyFrame.MyWin._m.getString("help_alpha"));
	   	helpArea[6].setWrapStyleWord(true);
	   	helpArea[6].setLineWrap(true);
	   	helpArea[6].setBorder(new EtchedBorder());
	   	helpArea[6].setEditable(false);
	   	aButton.setBorder(null);
	   	add(aButton, new GridBagConstraints(0,7,2,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[6], new GridBagConstraints(2,7,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[7]=new JTextArea(PoddyFrame.MyWin._m.getString("help_num"));
	   	helpArea[7].setWrapStyleWord(true);
	   	helpArea[7].setLineWrap(true);
	   	helpArea[7].setBorder(new EtchedBorder());
	   	helpArea[7].setEditable(false);
	   	numButton.setBorder(null);
	   	add(numButton, new GridBagConstraints(0,8,2,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[7], new GridBagConstraints(2,8,2,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	helpArea[8]=new JTextArea(PoddyFrame.MyWin._m.getString("help_rc"));
	   	helpArea[8].setWrapStyleWord(true);
	   	helpArea[8].setLineWrap(true);
	   	helpArea[8].setBorder(new EtchedBorder());
	   	helpArea[8].setEditable(false);
	   	rcButton.setBorder(null);
	   	add(rcButton, new GridBagConstraints(0,9,3,1,0.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
	   	add(helpArea[8], new GridBagConstraints(3,9,1,1,0.9,0.0,
                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

	   	add(okButton, new GridBagConstraints(0,20,4,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,10,1,10),0,0)); 
	   	
	   	add(offButton, new GridBagConstraints(0,21,4,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,10,10,10),0,0)); 
	   	switchVertical(-1);
	}
	

          /**
	   * This method handles the change of selection in vertical direction. It is called from the main
	   * application depending on the current user input. Here within this panel the selection of the
	   * next or previous row is performed so that the user can see where he jumped to using his keyboard
	   * or remote control
	   * @param dir the direction the current selection changed to, a value of +1 means the user jumped
	   *        one step up, -1 means he jumped one step down
	   */	
	   public boolean switchVertical(int dir)
	   {
	      if (dir>0)
	      {
	         if (selected==0) return false;
	         selected--;
	      }
	      else if (dir<0)
	      {
	         if (selected>=MAX_HELP_ITEMS+1) return false;            
	         selected++;
	      }
	      if ((prevSelected>-1) && (prevSelected<MAX_HELP_ITEMS)) helpArea[prevSelected].setBackground(Color.WHITE);
	      okButton.setBackground(Color.WHITE);
	      offButton.setBackground(Color.WHITE);
	      if (selected==MAX_HELP_ITEMS+1)
	      {
	         offButton.setBackground(PoddyFrame.highlight);
	         scrollComponentToVisible(offButton);
	      }
	      else if (selected==MAX_HELP_ITEMS)
	      {
	         okButton.setBackground(PoddyFrame.highlight);
	         scrollComponentToVisible(okButton);
	      }
	      else
	      {
	         helpArea[selected].setBackground(PoddyFrame.highlight);
	         scrollComponentToVisible(helpArea[selected]);
	      }
	      prevSelected=selected;
	      return true;
	   }
	   
	   
	  /**
	   * Whenever the panel is no longer required this method is called by the main application. Here
	   * everythin is deinitialized and depending on the button (euqlas row) that was selected last
	   * the information is stored if this panel has to be displayed again during next application.
	   * start up or not. The procedure to do that depends on the row number stored in variable
	   * "selected": when it is the last available help element plus 1 (= MAX_HELP_ITEMS = "OK"-button)
	   * the formerly created file .firststart is deleted so that the application has to assume the
	   * application was never started before. When the last element (= MAX_HELP_ITEMS+1 = "Do not
	   * show again"-button) was selected the panel cal be closed without changing anything, in this
	   * case the main application handles the .firststart file correctly.  
	   */
	   public boolean closePanel()
	   {
	      super.closePanel();
	      if (selected==MAX_HELP_ITEMS+1) return true;
	      else if (selected==MAX_HELP_ITEMS)	    	  
	      {
	    	  File file=new File(".firststart");
	    	  file.delete(); // delete this file to enable the help for the next time again
	    	  return true;
	      }
	      return false;
	   }

}
