/*
 * LCDController.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import java.text.*;
import java.util.*;

import javax.swing.*;

import com.vwp.jLCD.*;


/**
 * This class comtains the functionality to send data to an optionally connected LCD. This is done via
 * lcdproc and the related classes of com.vwp.jLCD of the Java XTools. This class consisnts of several
 * service methods and an onw thread. The thread displays some interesting statistic information after
 * a timeout. This timeout elapses only when no other, usage-relevant data are displayed via the other
 * service methods meanwhile.<BR>
 * Within the display two text lines are created, the first line is a scrolling widget that is able to
 * hold texts that are larger than the display, the second one is a fixed widget where thexts should
 * be set only that fit into the display, elsewhere the right part will be skipped.
 */
public class LCDController extends Thread
{
   private ScrollerWidget   text1=null;
   private StringWidget     text2=null;
   private boolean          isErrorText=false,isMsgText=false,isSaverText;
   private String           saveText1,saveText2;
   private long             lastPing=System.currentTimeMillis()+10000;
   private int              numOfTracks=-1,numOfAlbums=-1,numOfArtists=-1,numOfSeries=-1,numOfMovies=-1,displayPos=0;
   private DefaultListModel top10tracks=null,top10albums=null;
   private LCD              lcd;
   private Screen           textScreen=new Screen("T");
   
   
  /**
   * The standard constructor for an object of type LCDController establishes the connection
   * to the lcdproc server and starts the internal thread that checks for the user input
   * timeout
   * @param title the name for the lcdproc scoller widget in first line of the LCD
   */
   public LCDController(String title)
   {
      try
      {
         lcd=new LCD(13666,"127.0.0.1");
   
         textScreen.setHeartbeat(false);
         textScreen.setPriority(3);
         lcd.addScreen(textScreen);

         text1=new ScrollerWidget(1, 1,lcd.getWidth(),3,ScrollerWidget.SCROLL_HORIZONTAL,4,title);
         text2=new StringWidget(1,2,"");  
         textScreen.addWidget(text1);
         textScreen.addWidget(text2);
         
         start();
      }
      catch (Exception e)
      {
//         e.printStackTrace();
      }
   }
   
   
   public void run()
   {
      DefaultListModel result;
      SimpleDateFormat formatter1 = new SimpleDateFormat ("EEEE, d. MMMM yyyy");
      SimpleDateFormat formatter2 = new SimpleDateFormat ("HH:mm");
      TrackInfo        trackInfo;
      CDInfo           cdInfo;
      
      while (true)
      {
         if (lastPing+60000<System.currentTimeMillis())
         {
            switch (displayPos)
            {
            case 0:
               if (numOfTracks<0)
               {
                  result=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_COUNT,"SELECT COUNT (*) FROM "+SQLAccess.CDTracksTable);
                  if ((result!=null) && (result.size()>0)) numOfTracks=((Integer)result.elementAt(0)).intValue();
               }
               if (numOfTracks>=0) this.setSaverTexts(PoddyFrame.MyWin._m.getString("num_of_tracks")+" "+numOfTracks,"");
               break;
            case 1:
               if (numOfAlbums<0)
               {
                  result=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_COUNT,"SELECT COUNT (*) FROM "+SQLAccess.CDTable);
                  if ((result!=null) && (result.size()>0)) numOfAlbums=((Integer)result.elementAt(0)).intValue();
               }
               if (numOfAlbums>=0) this.setSaverTexts(PoddyFrame.MyWin._m.getString("num_of_albums")+" "+numOfAlbums,"");
               break;               
            case 2:
               if (numOfArtists<0)
               {
                  result=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_TEXT,"SELECT artist, COUNT (*) FROM "+SQLAccess.CDTable+" GROUP BY artist");
                  if ((result!=null) && (result.size()>0)) numOfArtists=result.size();
               }
               if (numOfArtists>=0) this.setSaverTexts(PoddyFrame.MyWin._m.getString("num_of_artists")+" "+numOfArtists,"");
               break;               
            case 3:
               if (numOfSeries<0)
               {
                  result=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_TEXT,"SELECT title, COUNT (*) FROM "+SQLAccess.DVDTable+" WHERE season>0 GROUP BY title");
                  if ((result!=null) && (result.size()>0)) numOfSeries=result.size();//((Integer)result.elementAt(0)).intValue();
               }
               if (numOfSeries>=0) this.setSaverTexts(PoddyFrame.MyWin._m.getString("num_of_series")+" "+numOfSeries,"");
               break;               
            case 4:
               if (numOfMovies<0)
               {
                  result=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_COUNT,"SELECT COUNT (*) FROM "+SQLAccess.DVDTable+" WHERE season<=0");
                  if ((result!=null) && (result.size()>0)) numOfMovies=((Integer)result.elementAt(0)).intValue();
               }
               if (numOfMovies>=0) setSaverTexts(PoddyFrame.MyWin._m.getString("num_of_movies")+" "+numOfMovies,"");
               break;
            case 5:
            case 16:
            case 21:
            case 32:
            case 43:
               Date currentTime = new Date();
               setSaverTexts(formatter1.format(currentTime),formatter2.format(currentTime));
               break;
            case 6:
            case 7:
            case 8:
            case 9:
            case 10:
            case 11:
            case 12:
            case 13:
            case 14:
            case 15:
               if (top10tracks==null)
                top10tracks=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_CDTRACKS,"SELECT * FROM "+SQLAccess.CDTracksTable+" WHERE cnt>0 ORDER BY cnt DESC LIMIT 10");
               if (top10tracks==null) displayPos=15;
               else if (top10tracks.size()+6-displayPos<=0) displayPos=15;
               else
               {
                  trackInfo=(TrackInfo)top10tracks.getElementAt(displayPos-6);
                  result=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" where idx='"+trackInfo.cdidx+"'");
                  if ((result==null) || (result.size()<=0)) displayPos=15;
                  else
                  {
                     cdInfo=(CDInfo)result.elementAt(0);
                     setSaverTexts(cdInfo.artist+": "+trackInfo.name,PoddyFrame.MyWin._m.getString("top_10_songs")+" "+(displayPos-5));
                  }
               }
               break;
            case 17:
               setSaverTexts("Credits go to...","");
               break;
            case 18:
               setSaverTexts("Volker Berlin for SmallSQL","www.smallsql.de");
               break;
            case 19:
               setSaverTexts("David R. Robinson and others for jFLAC","jflac.sf.net");
               break;
            case 20:
               setSaverTexts("James Gosling and SUN for Java","java.sun.com");
               break;
            case 22:
            case 23:
            case 24:
            case 25:
            case 26:
            case 27:
            case 28:
            case 29:
            case 30:
            case 31:
               if (top10albums==null)
                top10albums=PoddyFrame.MyWin.doLocalSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" WHERE cnt>0 ORDER BY cnt DESC LIMIT 10");
               if (top10albums==null) displayPos=31;
               else if (top10albums.size()+22-displayPos<=0) displayPos=31;
               else
               {
                  cdInfo=(CDInfo)top10albums.getElementAt(displayPos-22);
                  setSaverTexts(cdInfo.artist+": "+cdInfo.album,PoddyFrame.MyWin._m.getString("top_10_albums")+" "+(displayPos-21));
               }
               break;
            default:
               break;
            }
            displayPos++;
            if (displayPos>32) displayPos=0;
         }
         com.vwp.util.Tools.threadSleep(5000);
      }
   }
   
   
   
  /**
   * This method has to be called for every user interaction that does not change the data displayed
   * within the LCD so that it is able to restart the timeout counter without changing anything within
   * the display or - when a timeout is active - to display the last data that have been active before
   * the current timeout ocurred 
   */
   public void ping()
   {
      lastPing=System.currentTimeMillis();
      numOfTracks=-1;
      numOfAlbums=-1;
      numOfArtists=-1;
      numOfSeries=-1;
      numOfMovies=-1;
      top10tracks=null;
      top10albums=null;
      resetSaverTexts();
   }
   
  
   
  /**
   * Sets two standard texts to be displayed left-aligned
   * @param t1 the text for the first line within the display
   * @param t2 the text for the second line within the display
   */
   public void setTexts(String t1,String t2)
   {
      setTexts(t1,t2,false);
   }
   
   
   
  /**
   * Sets two standard texts to be displayed
   * @param t1 the text for the first line within the display
   * @param t2 the text for the second line within the display
   * @param center when this value is set to false the texts are displayed left-aligned, when it
   *        is true they are centered
   */
   public void setTexts(String t1,String t2,boolean center)
   {
      if ((!isErrorText) && (!isMsgText) && (!isSaverText))
      {
         if ((text1!=null) && (t1!=null))
            {
            if (center) text1.setText(t1,Widget.ALIGN_CENTER); 
            else text1.setText(t1);
            }
         if ((text2!=null) && (t2!=null)) 
         {
            if (center) text2.setText(t2,Widget.ALIGN_CENTER);
            else text2.setText(t2);
         }
      }
   }
   
   
  /**
   * Fetches a line of text that is currently set as standard text. When a timeout is active
   * and other information are displayed this method does NOT return these other information
   * but the standard texts that would have been displayed withiut that timeout
   * @param num the line for which the text has to be fetched
   * @return the text stored at this line or null when an invalid line number was given
   */
   public String getText(int num)
   {
      if ((num==1) && (text1!=null)) return text1.getText();
      if ((num==2) && (text2!=null)) return text2.getText();
      return null;
   }
   
   
   
  /**
   * Sets two error texts to be displayed left-aligned and with a bigger priority than the standard
   * texts; this method does not overwrite the standard texts, with a call to resetErrorText() the
   * error text is removed and the previously used standard texts are displayed again
   * @param t1 the error text for the first line within the display
   * @param t2 the error text for the second line within the display
   */
   public void setErrorText(String text)
   {
      if (!isErrorText)
      {
         if (text1!=null) saveText1=text1.getText();
         else saveText1=null;
         if (text2!=null) saveText2=text2.getText();
         else saveText2=null;
      }
      setTexts(text,PoddyFrame.MyWin._m.getString("error"));
      isErrorText=true;
   }
   
   
   
  /**
   * Removes the currently displayed error texts and switch back to the formerly shown standard
   * information
   */
   public void resetErrorText()
   {
      ping();
      if (isErrorText)
      {
         isErrorText=false;
         setTexts(saveText1,saveText2);
      }
   }

   
   
  /**
   * Sets two message texts to be displayed left-aligned and with a bigger priority than the
   * standard texts; this method does not overwrite the standard texts, with a call to
   * resetMessageText() the message texts are removed and the previously used standard texts
   * are displayed again
   * @param t1 the message text for the first line within the display
   * @param t2 the message text for the second line within the display
   */
   public void setMessageText(String text)
   {
      if (!isMsgText)
      {
         if (text1!=null) saveText1=text1.getText();
         else saveText1=null;
         if (text2!=null) saveText2=text2.getText();
         else saveText2=null;
      }
      setTexts(text,PoddyFrame.MyWin._m.getString("info"));
      isMsgText=true;
   }
   
   
   
  /**
   * Removes the currently displayed message texts and switch back to the formerly shown standard
   * information
   */
   public void resetMessageText()
   {
      ping();
      if (isMsgText)
      {
         isMsgText=false;
         setTexts(saveText1,saveText2);
      }
   }

   

  /**
   * Sets two saver texts to be displayed left-aligned and with a bigger priority than the
   * standard texts; this method does not overwrite the standard texts, with a call to
   * resetSaverText() the message texts are removed and the previously used standard texts
   * are displayed again. Saver texts are an additional category for special texts that
   * temporarily may be displayed during operation but they exist in parallel to the standard
   * texts, they are not as important than errors or messages.
   * @param t1 the saver text for the first line within the display
   * @param t2 the saver text for the second line within the display
   */
   private void setSaverTexts(String t1,String t2)
   {
      if (!isSaverText)
      {
         if (text1!=null) saveText1=text1.getText();
         else saveText1=null;
         if (text2!=null) saveText2=text2.getText();
         else saveText2=null;
      }
      isSaverText=false;
      setTexts(t1,t2,true);
      isSaverText=true;
   }
   
   
   
  /**
   * Removes the currently displayed saver texts and switch back to the formerly shown standard
   * information
   */
   public void resetSaverTexts()
   {
      if (isSaverText)
      {
         isSaverText=false;
         setTexts(saveText1,saveText2);
      }
   }
}
