/*
 * NEJField.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package base;

import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;



@SuppressWarnings("serial")
/**
 * This is a default text field that has to be used within the application instead of the normal
 * JTextField. It avoids the normal selection and editing of its data what would collide with the
 * remote control/keyboard based usage principle.
 * TODO: implement functionality to react on number keys of remote control
 */
public class NEJField extends JPanel
{
   public static final int MODE_LEFTRIGHT=0x01; /** operation mode flag; the input is able to handle left/right commands to display a different pre-selection element */
   public static final int MODE_INPUT=0x02;     /** operation mode flag; the input is able to handle input of real data from keyboard */
   
   private JTextComponent field;

   

  /**
   * NEJField objects never should bne used directly. They are generated via their default constructor
   * by the classes that inherit from NEJField. Because of that the parameters have to be set via this
   * method by these classes.
   * @param mode OR-concatenated MODE_xxx input flags that define the behaviour of the input
   * @param field the input field that has to be used for operation
   * @param icon an image icon that visualises the input mode
   */   
   void init(int mode,JTextComponent field,ImageIcon icon)
   {
      this.field=field;
      JButton btn1=new JButton(),btn2=new JButton();
      
      field.setEditable(false);
      setLayout(new GridBagLayout());
      add(field,new GridBagConstraints(0,0,1,1,1.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
      super.setBackground(Color.WHITE);
      
      btn1.setBorder(null);
      if ((mode & MODE_LEFTRIGHT)!=0) btn1.setIcon(PoddyFrame.MyWin.iconLR);
      else btn1.setIcon(PoddyFrame.MyWin.iconEmpty);
      add(btn1,new GridBagConstraints(1,0,1,1,0.0,0.0,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));

      btn2.setBorder(null);
      if ((mode & MODE_INPUT)!=0) btn2.setIcon(icon);
      else btn2.setIcon(PoddyFrame.MyWin.iconEmpty);
      add(btn2,new GridBagConstraints(2,0,1,1,0.0,0.0,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(0,0,0,1),0,0));
   }
   
   
  /**
   * Sets a new text that is displayed to the user and can be edited.
   * @param text the new text to be set
   */
   public void setText(String text)
   {
      field.setText(text);
   }
   
   
  /**
   * Gets the currently displayed text
   * @return the text that was entered
   */
   public String getText()
   {
      return field.getText();
   }
   
   
  /**
   * Fetches the currently used input object
   * @return the input element that is used by this object
   */
   public JTextComponent getInputField()
   {
      return field;
   }
   
   
  /**
   * This method can be used to change the background colour of the input object
   * @param color the new background colour
   */
   public void setBackground(Color color)
   {
      if (field!=null) field.setBackground(color);
   }

}
