/*
 * NEXIntegerField.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package base;

import com.vwp.swing.*;


@SuppressWarnings("serial")
/**
 * This is a default integer field that has to be used within the application instead of the normal
 * XIntegerField. It avoids the normal selection and editing of its data what would collide with the
 * remote control/keyboard based usage principle.
 * TODO: implement functionality to react on number keys of remote control to enter texts in a similar
 * way like it is known from cellphones
 */public class NEXIntegerField extends NEJField
{
   private XIntegerField intField; /** an object of type XIntegerField is used because allowed input for this class is numerical only */

   
   
  /**
   * Constructor for objects of type NEXIntegerField
   * @param mode operation mode flags of type NEJField.MODE_xxx that are necessary to visualise what the
   *        user can do with this input field 
   */
   public NEXIntegerField(int mode)
   {
      super();
      intField=new XIntegerField();
      init(mode,intField,PoddyFrame.MyWin.iconNum);
   }

   
   
  /**
   * Constructor for objects of type NEXIntegerField
   * @param value a numerical value the new object has to be initialised with
   * @param MinVal the minimum allowed value a user can enter
   * @param MaxVal the maximum allowed value a user can enter, this value must be greater than MinVal
   * @param mode operation mode flags of type NEJField.MODE_xxx that are necessary to visualise what the
   *        user can do with this input field 
   */
   public NEXIntegerField(int value, int MinVal, int MaxVal,int mode)
   {
      super();
      intField=new XIntegerField(value,MinVal,MaxVal);
      init(mode,intField,PoddyFrame.MyWin.iconNum);
   }

   
   
  /**
   * Objects of type NEXIntegerField handle numbers instead of thexts. Because of that new values
   * have to be set using this method.
   * @param value the new number that has to be set
   */
   public void setValue(int value)
   {
      intField.setValue(value);
   }
   
   
   
  /**
   * Returns the stored number
   * @return the number currently stored by the object
   */
   public int getValue()
   {
	  return intField.getValue();	   
   }

}
