/*
 * PoddyFrame.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;
import javax.swing.*;

import com.vwp.awt.image.ImageUtils;

import system.*;




/**
 * This is the base class for the main window of the main and setup application. It defines several methods
 * that are used by the main and setup application. So the main implementation of these methods is done
 * within the classes SetupWin and MainWin that inherit from this base class. Resulting from that implementations
 * done here within this base class have to provide a general functionality that is valid for both.
 */

@SuppressWarnings("serial")
public class PoddyFrame extends JFrame
{   
   public static final int USEFLAG_UP     =0x01,
                           USEFLAG_DOWN   =0x02,
                           USEFLAG_LEFT   =0x04,
                           USEFLAG_RIGHT  =0x08,
                           USEFLAG_RETURN =0x10,
                           USEFLAG_ALPHA  =0x20,
                           USEFLAG_NUM    =0x40;
   public static final int ID_TOP_SERIES  =0x001,
                           ID_TOP_MOVIES  =0x002,
                           ID_TOP_CD=      0x003,
                           ID_TOP_VIDEOS  =0x005,
                           ID_TOP_VCR=     0x007,
                           ID_TOP_POWEROFF=0x008;
   public static final int ID_CD_ARTIST_ALBUM_SHUFFLE=         0x01001,
                           ID_CD_ARTIST_ALBUM_PLAY=            0x01002,
                           ID_CD_ARTIST_ALBUM_TRACK=           0x01003,
                           ID_CD_ARTIST_ALBUM_MOVE_TRACKS=     0x01004,
                           ID_CD_ARTIST_ALBUM_DELETE      =    0x01005,
                           ID_CD_ARTIST_ALBUM_MOVE_TRACKS_HERE=0x01006,
                           ID_CD_ARTIST_ALBUM_CANCEL_OPERATION=0x01007,
                           ID_CD_ARTIST_ALBUM_EDIT        =    0x01008,
                           ID_CD_ARTIST_ALBUM_TO_USB      =    0x01009;
   
   public static PoddyFrame     MyWin;                                    /** the main window as created by MainWin or SetupWin, that inherits from PoddyFrame */

   public        ResourceBundle _m=ResourceBundle.getBundle("texts");     /** handles the external text resources */
   public        ConfigData     config=new ConfigData();                  /** this object contains the data out of the configuration file */
   public        ASymKey        keyEventHandler=new ASymKey();
   public        long           lastKeyPress=System.currentTimeMillis();  /** stores the last time a key event was received */
   public        SystemAccess   sysAccess;
   public        boolean        processingKeyEvent=false;                 /** as long as this variable is true no new key event has to be sent */
   public        int            customSelected;
   
   public        CustomPanel    customPanel;                              /** holds a custom panel that is displayed temporarily */
   public        StationsList   stationsList=new StationsList();
   public        BufferedImage  useImage=null,bgImage=null;
   public        ImageUtils     iu;

   /* following several colour definitions are made that are used within the application. TODO: put these values into the configration
      so that deisgn and layout can be edited freely */
   public final static Color          highlight=new Color(145,145,255);     /** color for highlighting entries */
   public final static Color          topentry=new Color(130,130,230);      /** top managing entries color */
   public final static Color          stdentry=new Color(240,240,255);      /** standard entries color, must be different than disabledentry */
   public final static Color          disabledentry=new Color(140,140,150); /** color for a disabled entry, must be different than stdentry */
   public final static Color          serviceentry=new Color(225,125,125);  /** service entry, located at bottom of lists */
   public final static Color          errorcolor=new Color(150,10,10);      /** color for error texts */
   public final static Color          bgcolor=new Color(0,0,100);           /** standard background color */
   public              ImageIcon      iconAlpha,iconNum,iconLR,iconEmpty;

   // next variables used only in MainWin but for SetupWin...it doesn't disturbs - so what? ;-)
   public              SQLLocalAccess SQL=null; 	
   public              long           nextPoddydSyncTime=System.currentTimeMillis();
   
   
  /**
   * The standard constructor for the applications main window. Here all resources are created,
   * initialized and loaded that are required for the application
   */
   public PoddyFrame()
   {
      UIDefaults uiDefaults = UIManager.getDefaults();
      /* following lines modify the standard sizes of GUI elements in order to have a nice look on
         TV screens; these values are optimized for 16:9 PAL screens (and should fit for NTSC/SECAM too)
	 TODO: put these values into the configuration so that a user can adapt them according to its
	 screen size */
      uiDefaults.put( "Button.font",((Font)uiDefaults.get("Button.font")).deriveFont(21f) );
      uiDefaults.put( "Label.font",((Font)uiDefaults.get("Label.font")).deriveFont(21f) );
      uiDefaults.put( "TextField.font",((Font)uiDefaults.get("TextField.font")).deriveFont(21f) );
      uiDefaults.put( "TextArea.font",((Font) uiDefaults.get("TextArea.font")).deriveFont(18f) );
      config.readConfig();
   
      if ((config.bgimage!=null) && (config.bgimage.length()>1)) try
      {
         BufferedInputStream in=new BufferedInputStream(new FileInputStream(config.bgimage));
         byte                b[];
         
         b=new byte[in.available()];
         in.read(b);
         in.close();
         iu=new ImageUtils();
         bgImage=iu.createBufferedImage(b,1.0);
      }
      catch (IOException ioe)
      {
         ioe.printStackTrace();
      }
      try
      {
    	 iconNum=new ImageIcon(getClass().getResource("/img/iconnum.png"));  
    	 iconAlpha=new ImageIcon(getClass().getResource("/img/iconalpha.png"));  
    	 iconLR=new ImageIcon(getClass().getResource("/img/iconlr.png"));  
    	 iconEmpty=new ImageIcon(getClass().getResource("/img/iconempty.png"));  
      }
      catch (Exception e)
      {
      }
   }


   
  /**
   * Method for handling the set-input from keyboard/remote control
   */
   public void handleSetCmd()
   {
   }
   
   
   
  /**
   * Method for handling the up-input from keyboard/remote control
   */
   public void handleUpCmd()
   {
      if (customPanel!=null)
	  {
	     if (customSelected>0) customSelected--;
	     customPanel.setSelectedField(customSelected);
	     customPanel.switchVertical(1);
      }
   }
   
   
   
  /**
   * Method for handling the down-input from keyboard/remote control
   */
   public void handleDownCmd()
   {
      if (customPanel!=null)
	  {
	     customSelected++;
	     if (!customPanel.setSelectedField(customSelected))
	      customSelected--;
	     customPanel.switchVertical(-1);
      }
   }
   
   
   
  /**
   * Method for handling the left-input from keyboard/remote control
   */
   public void handleLeftCmd()
   {
      if (customPanel!=null)
      {
         customSelected--;
         if (!customPanel.setSelectedField(customSelected))
          customSelected++;
         customPanel.switchHorizontal(-1);
      }
   }
   
   
   
  /**
   * Method for handling the right-input from keyboard/remote control
   */
   public void handleRightCmd()
   {
      if (customPanel!=null)
      {
         customSelected++;
         if (!customPanel.setSelectedField(customSelected))
          customSelected--;
         customPanel.switchHorizontal(1);
      }
   }



  /**
   * Method for handling the menu-input from keyboard/remote control
   */
   public void handleMenuCmd()
   {
   }



  /**
   * Method for handling the play-input from keyboard/remote control
   */
   public void handlePlayCmd()
   {
   }
   
   
   
  /**
   * Method for handling the stop-input from keyboard/remote control
   */
   public void handleStopCmd()
   {
   }
   
   
  /**
   * The key event handler that cares about all received key events from a possibly connected
   * keyboard
   */
   class ASymKey extends KeyAdapter
   {
      public void keyReleased(KeyEvent e)
      {
         if ((lastKeyPress+150>System.currentTimeMillis()) && (customPanel==null)) return;
         processingKeyEvent=true;
         lastKeyPress=System.currentTimeMillis();
         switch (e.getKeyCode())
         {
            case KeyEvent.VK_ENTER:
               handleSetCmd();
               break;
            case KeyEvent.VK_UP:
               handleUpCmd();
               break;
            case KeyEvent.VK_DOWN:
               handleDownCmd();
               break;
            case KeyEvent.VK_LEFT:
               handleLeftCmd();
               break;
            case KeyEvent.VK_RIGHT:
               handleRightCmd();
               break;
            case KeyEvent.VK_PAGE_UP:
               handleMenuCmd();
               break;
            case KeyEvent.VK_F11:
               handlePlayCmd();
               break;
            case KeyEvent.VK_F12:
               handleStopCmd();
               break;
            case KeyEvent.VK_0:
               handleKey(0);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_1:
               handleKey(1);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_2:
               handleKey(2);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_3:
               handleKey(3);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_4:
               handleKey(4);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_5:
               handleKey(5);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_6:
               handleKey(6);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_7:
               handleKey(7);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_8:
               handleKey(8);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            case KeyEvent.VK_9:
               handleKey(9);
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;
            default:
               if (customPanel!=null) customPanel.otherKeyEvent(e.getKeyCode(),e.getKeyChar());
               break;         
         }
         processingKeyEvent=false;
      }
      
   }
   
   
  /**
   * Whenever the user enters a number (using keyboard or remote control) that number is handed over
   * to this method for further processing
   */
   public void handleKey(int num)
   {
   }



  /**
   * When an object of type CustomPanel is created and has to be displayed instead of the standard view
   * this method has to be used. Its implementation in MainWin disables the current panel and shows the
   * custom panel. To switch back to normal view or to switch to a different custom panel
   * removeCustomPanel() has to be executed first
   * @param panel the object that inherits from CustomPanel that has to be set
   */
   public void setCustomPanel(CustomPanel panel)
   {
   }   
   
   
  /**
   * The main title bar of the application diusplays useful information about what is managed/done by the
   * application at the moment.
   * @param text the new text that has to bne displayed within the top bar
   */
   public void setTopPanelText(String text)
   {
   }


   
  /**
   * The main title bar of the application diusplays useful information about what is managed/done by the
   * application at the moment.
   * @param text the new text that has to bne displayed within the top bar
   */   
   public void setTopPanelText(String text,String path)
   {
   }

   
   
  /**
   * 
   */
   public void storeGlobalCDInfo(CDInfo info)
   {
   }
   
   
   
  /**
   * 
   */
   public boolean fetchGlobalCDInfo(CDInfo info)
   {
      return true;
   }
   
   
   
  /**
   * 
   */
   public void updateCDDBs(String artist,String album,String track)
   {
   }

   
   
  /**
   * This method applies to the local CD database, it stores a track
   */
   public void setTrackToDB(int idx,String trackname)
   {
   }

   
   
  /**
   * This method applies to the local CD database, it deletes a track from
   * the SQL database. Please note: currently the SmallSQL-DB does not support
   * data compression. Because of that deleting a dataset from the DB does not release
   * the related disk space. That means ceation and deletion of tracks has to be
   * done very carefully and only if really necessary!
   */
   public void deleteCDFromDB(int cdidx)   
   {
   }

   
   
  /**
   * 
   */
   public void setCDToDB(String artist,String album, String genre, String cat,int year,int tracks)
   {
   }

   
   
  /**
   * 
   */
   public void setDVDToDB(DVDInfo dvdInfo)
   {
   }

   
   
  /**
   * 
   */
   public void updateDVDDB(DVDInfo dvdInfo)
   {
   }


   
  /**
   * 
   */
   public int getCDDBIdx(String artist,String album)
   {
      return -1;
   }

   

  /**
   * 
   */
   public int getDVDDBIdx(long crcid)
   {
      return -1;
   }

   
   
  /**
   * 
   */
   public void setPlayInfo(CDInfo cdInfo,TrackInfo trackInfo,String src)
   {
   }
   
   
   
  /**
   * 
   */
   public void initUI()
   {
   }   
 
   
   
  /**
   * 
   */
   public void setMenu(int menuID,int stack)
   {
   }

   
   
  /**
   * This method directly accesses the LCDController and lets it show a new text on the
   * (optionally) connected display
   * @param text1 the first line of the new text to be set
   * @param text2 the second line of the new text to be set
   */
   public void setLCDText(String text1,String text2)
   {
      
   }
   
   
   
  /**
   *
   */
   public String getLCDText(int num)
   {
      return null;
   }



  /**
   * This method overwrites the text currently displayed at an (optionally) connected
   * display to display an important message instead of it. The original texts are not
   * deleted by this operation
   * @param text the error text to be displayed
   */
   public void setLCDErrorText(String text)
   {
      
   }
   


  /**
   * This method overwrites the text currently displayed at an (optionally) connected
   * display to display an important message instead of it. The original texts are not
   * deleted by this operation
   * @param text the message to be displayed
   */
   public void setLCDMessageText(String text)
   {
      
   }
   
   
   
   public DefaultListModel doLocalSQLQuery(int table,String query)
   {
      return SQL.doSQLQuery(table,query);
   }
	
   
   
   public void doLocalSQLOperation(String query)
   {
      SQL.doSQLOperation(query);
   }
   
   
   
   public void turnOffBottomPanel()
   {
   }

   

   public void setStationsList(StationsList newList)
   {
	   
   }
   
/*   public int getTopOffset()
   {   
      return 20;
   }   */

   
   
   public void saveStations(StationsList list)
   {
      SQL.saveStations(list,config.uname,getPwdHash());
   }

   
   
   public void saveRecording(RecordInfo info)
   {
      SQL.saveRecording(info,config.uname,getPwdHash());
      nextPoddydSyncTime=System.currentTimeMillis();
   }

   
   
   public String getPwdHash()
   {
	   return "";
   }
   
   public void scrollToObjectASync(JComponent component)
   {
   }
   
   
}

