/*
 * PoddydClient.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import java.net.*;
import java.util.Calendar;
import java.io.*;

import javax.swing.*;


/**
 * This class implements the client connection to the remote poddy-server to retrieve/update CD and DVD data
 * and to synchronize with a users remote account that contains scheduled recording information. Here within
 * this class all functionalities for connectiong with the poddy-server "poddyd" and the poddy-protocol is
 * implemented
 */
public class PoddydClient
{
   private Socket           sock; 
   private PrintWriter      out;
   private BufferedReader   in;
   
   
   
  /**
   * Default constructor for a PoddydClient object, here the socket connection to the poddy
   * server is established to communicate with it in further method calls
   */
   public PoddydClient()
   throws Exception
   {
      sock=new Socket("virtualworlds.de",26175);
      in=new BufferedReader(new InputStreamReader(sock.getInputStream(),"ISO-8859-1"));
      out=new PrintWriter(new OutputStreamWriter(sock.getOutputStream(),"ISO-8859-1"));
   }

   
   
  /**
   * Retrieves the current time in a standardised format
   * @param the current time in seconds
   */
   private long getCurrentTime()
   {
      return (System.currentTimeMillis()-Calendar.getInstance().getTimeZone().getRawOffset())/1000;
   
   }
   
   
  /**
   * Reads the poddyd protocol header information from the currently used socket connection
   */
   private boolean readHeader()
   throws IOException
   {
      String ln;
      
      out.flush();
      ln=in.readLine(); ln.trim();
      if (!ln.equals("poddy1")) return false;
      ln=in.readLine(); ln.trim();
      if (!ln.equals("OK"))
      {
         System.err.println(ln+": "+in.readLine());
         return false;
      }
      return true;
   }

   
  /**
   * Performs a full synchronisation of all remote and local userdata with new and deleted
   * schedules
   * @param SQL local SQL connection object that handles the stored scheduling information
   * @param uname remote username to authenticate with
   * @param hash remote password hash value to authenticate with 
   */
   public void syncAll(SQLLocalAccess SQL,String uname,String hash)
   {
      DefaultListModel result;
      int              i;
      RecordInfo       info;
      String           ln;
      
      if ((uname.length()<=1) || (hash.length()<=3)) return;
      try
      {
         // send programmed schedules
         result=SQL.doSQLQuery(SQLAccess.T_SCHEDULE,"SELECT * FROM "+SQLAccess.ScheduleTable+" WHERE stime>'"+getCurrentTime()+"' AND synced='0' AND deleted='0'");
         if ((result!=null) && (result.size()>0))
         {
            out.write("poddy1\n");
            out.write("AUTHSETSCHED\n");
            out.write(uname+"\n");
            out.write(hash+"\n");
            for (i=0; i<result.size(); i++)
            {
               info=(RecordInfo)result.elementAt(i);
               out.write(info.stime+"\n");
               out.write(info.rtime+"\n");
               out.write(info.station+"\n");
               out.write(info.name+"\n");
            }
            out.write("poddy0\n");
            if (readHeader())
             SQL.doSQLOperation("UPDATE "+SQLAccess.ScheduleTable+" SET synced='1' WHERE synced='0' AND deleted='0'");
            in.readLine(); // poddy0
         }
         
         // fetch remotely programmed schedules
         out.write("poddy1\n");
         out.write("AUTHGETSCHED\n");
         out.write(uname+"\n");
         out.write(hash+"\n");
         out.write("poddy0\n");
         if (readHeader())
         {
            ln=in.readLine();
            do
            {
               if (!ln.equals("poddy0"))
               {
                  info=new RecordInfo();
                  try
                  {
                     info.stime=Integer.parseInt(ln);
                  }
                  catch (Exception e) {e.printStackTrace();}
                  try
                  {
                     info.rtime=Integer.parseInt(in.readLine());
                  }
                  catch (Exception e) {e.printStackTrace();}
                  info.station=in.readLine();
                  info.name=in.readLine();
                  SQL.doSQLOperation("INSERT INTO "+SQLAccess.ScheduleTable+" (stime, rtime, station, name, synced) VALUES ('"+info.stime+"', '"+info.rtime+"', '"+info.station+"', '"+info.name+"', '1')");
                  ln=in.readLine();
               }
            }
            while (!ln.equals("poddy0"));
         }
         else in.readLine();

         // fetch deleted schedules
         out.write("poddy1\n");
         out.write("AUTHGETDELSCHED\n");
         out.write(uname+"\n");
         out.write(hash+"\n");
         out.write("poddy0\n");
         if (readHeader())
         {
            ln=in.readLine();
            do
            {
               if (!ln.equals("poddy0"))
               {
                  info=new RecordInfo();
                  try
                  {
                     info.stime=Integer.parseInt(ln);
                  }
                  catch (Exception e) {e.printStackTrace();}
                  try
                  {
                     info.rtime=Integer.parseInt(in.readLine());
                  }
                  catch (Exception e) {e.printStackTrace();}
                  info.station=in.readLine();
                  info.name=in.readLine();
                  SQL.doSQLOperation("DELETE FROM "+SQLAccess.ScheduleTable+" WHERE stime='"+info.stime+"' AND rtime='"+info.rtime+"' AND name='"+info.name+"' AND station='"+info.station+"'");
                  ln=in.readLine();
               }
            }
            while (!ln.equals("poddy0"));
         }
         else in.readLine();

         // send deleted schedules
         result=SQL.doSQLQuery(SQLAccess.T_SCHEDULE,"SELECT * FROM "+SQLAccess.ScheduleTable+" WHERE stime>'"+getCurrentTime()+"' AND synced='0' AND deleted='1'");
         if ((result!=null) && (result.size()>0))
         {
            out.write("poddy1\n");
            out.write("AUTHSETDELSCHED\n");
            out.write(uname+"\n");
            out.write(hash+"\n");
            for (i=0; i<result.size(); i++)
            {
               info=(RecordInfo)result.elementAt(i);
               out.write(info.stime+"\n");
               out.write(info.rtime+"\n");
               out.write(info.station+"\n");
               out.write(info.name+"\n");
            }
            out.write("poddy0\n");
            if (readHeader())
             SQL.doSQLOperation("DELETE FROM "+SQLAccess.ScheduleTable+" WHERE synced='0' AND deleted='1'");
            in.readLine(); // poddy0
         }      
      }
      catch (IOException ioe)
      {
         ioe.printStackTrace();
      }
   }
   
   
  /**
   * Sends the local list of known TV stations to the remote account of a user so that the latest
   * list is available within the web interface; this method is called every time a new scan for
   * TV stations was done, the TV stations have been re-arranged by the user or at least one TV
   * station was edited
   * @param list the new list of TV stations
   * @param uname remote username to authenticate with
   * @param hash remote password hash value to authenticate with 
   */
   public void setStations(StationsList list,String uname,String hash)
   {
      int i;
      
      if ((uname.length()<=1) || (hash.length()<=3)) return;
      try
      {
         out.write("poddy1\n");
         out.write("AUTHSETSTAT\n");
         out.write(uname+"\n");
         out.write(hash+"\n");
         for (i=0; i<list.stations.size(); i++)
         {
            out.write(list.stations.elementAt(i).name+"\n");
            out.write(list.stations.elementAt(i).channel+"\n");
         }
         out.write("poddy0\n");
         readHeader();
         in.readLine(); // poddy0
      }
      catch (IOException ioe)
      {     
         ioe.printStackTrace();
      }
   }
   
   

  /**
   * Sends information about a possibly new CD; this functionality is not related to a user account, therefore
   * no authentication is required
   * @param info the new CDInfo data to be stored within the global database; these values are rejected on
   *        server side in case there already exists a CD with this identifier
   */   
   public void setCD(CDInfo info)
   {
      int i;
      
      try
      {
         out.write("poddy1\n");
         out.write("SETCD\n");
         out.write(info.discid+"\n");
         out.write(info.tracksid+"\n");
         out.write("null\n"); // ean
         out.write("null\n"); // time
         out.write(info.year+"\n");      
         out.write(info.album+"\n");      
         out.write(info.genre+"\n");
         out.write(info.cat+"\n");
         out.write(info.artist+"\n");
         if (info.getTracksNumber()>0)
          for (i=0; i<info.getTracksNumber(); i++)
           out.write(info.getTrackAt(i).name+"\n");
         out.write("poddy0\n");
         readHeader();
         in.readLine(); // poddy0
      }
      catch (IOException ioe)
      {     
      }
   }
   
   
  /**
   * Tries to retrieve information about a CD from the CD database; this functionality is not related to a user
   * account, therefore no authentication is required
   * @param info a CDInfo object that contains discid and tracksid values that uniquely identify this CD
   * @return a CDInfo object with valid information for this CD or null if no entry for this CD exists in the
   *         poddy-database 
   */
   public CDInfo getCD(CDInfo info)
   {
      String    ln;
      CDInfo    newInfo=new CDInfo();  
      TrackInfo track;
      int       i;
      
      if ((info.discid==null) || (info.tracksid==null)) return null;
      newInfo.set(info);
      try
      {
         out.write("poddy1\n");
         out.write("GETCD\n");
         out.write(info.discid+"\n");
         out.write(info.tracksid+"\n");
         out.write("poddy0\n");
         /**********************************/
         if (!readHeader()) return null;
         if (newInfo.trackList!=null) newInfo.trackList.removeAllElements();
         do
         {
            ln=in.readLine(); ln.trim();
            if (ln.startsWith("poddy0")) break;
            else if (ln.startsWith("title=")) newInfo.album=ln.substring(6);
            else if (ln.startsWith("genre=")) newInfo.genre=ln.substring(6);
            else if (ln.startsWith("artist=")) newInfo.artist=ln.substring(7);
            else if (ln.startsWith("year="))
            {
               try
               {
                  newInfo.year=Integer.parseInt(ln.substring(5));
               }
               catch (Exception e)
               {
                  e.printStackTrace();   
               }
            }
            else for (i=1;i<=35; i++) 
            {
               String tag;
                
               if (i<10) tag="t0"+i+"=";
               else tag="t"+i+"=";
               if ((ln.startsWith(tag)) && (ln.length()>tag.length()))
               {
                  track=new TrackInfo(i);
                  track.name=ln.substring(tag.length());
                  newInfo.addTrack(track);
                  break;
               }  
            }
         }
         while (true);
         return newInfo;
      }
      catch (IOException ioe)
      {
      }
      return null;
   }

   
   
  /**
   * Sends information about a possibly new DVD; this functionality is not related to a user account, therefore
   * no authentication is required
   * @param info the new DCDInfo data to be stored within the global database; these values are rejected on
   *        server side in case there already exists a DVD with this identifier
   */   
   public void setDVD(DVDInfo info)
   {
      try
      {
         out.write("poddy1\n");
         out.write("SETDVD\n");
         out.write(info.crcid+"\n");
         out.write(info.ean+"\n"); // ean
         out.write(info.time+"\n"); // time
         out.write(info.year+"\n");      
         out.write(info.title+"\n");      
         out.write(info.genre+"\n");
         out.write(info.season+"\n");
         out.write(info.disc+"\n");
         out.write("poddy0\n");
         readHeader();
         in.readLine(); // poddy0
      }
      catch (IOException ioe)
      {     
      }
   }

   
   
  /**
   * Tries to retrieve information about a  DVD from the DVD database; this functionality is not related to a user
   * account, therefore no authentication is required
   * @param info a DVDInfo object that contains crcid value that uniquely identify this DVD
   * @return a DVDInfo object with valid information for this DVD or null if no entry for this DVD exists in the
   *         poddy-database 
   */
   public DVDInfo getDVD(DVDInfo info)
   {
      String    ln;
      DVDInfo   newInfo=new DVDInfo();  
      
      if (info.crcid==0) return null;
      newInfo.set(info);
      try
      {
         out.write("poddy1\n");
         out.write("GETDVD\n");
         out.write(info.crcid+"\n");
         out.write("poddy0\n");
         /**********************************/
         if (!readHeader()) return null;
         do
         {
            ln=in.readLine(); ln.trim();
            if (ln.startsWith("poddy0")) break;
            else if (ln.startsWith("time=")) 
            {
               try
               {
                  newInfo.time=Integer.parseInt(ln.substring(5));
               }
               catch (Exception e)
               {
                  e.printStackTrace();
               }
            }
            else if (ln.startsWith("title=")) newInfo.title=ln.substring(6);
            else if (ln.startsWith("genre=")) newInfo.genre=ln.substring(6);
            else if (ln.startsWith("year="))
            {
               try
               {
                  newInfo.year=Integer.parseInt(ln.substring(5));
               }
               catch (Exception e)
               {
                  e.printStackTrace();   
               }
            }
            else if (ln.startsWith("season="))
            {
               try
               {
                  newInfo.season=Integer.parseInt(ln.substring(7));
               }
               catch (Exception e)
               {
                  e.printStackTrace();   
               }
            }
            else if (ln.startsWith("disc="))
            {
               try
               {
                  newInfo.disc=Integer.parseInt(ln.substring(5));
               }
               catch (Exception e)
               {
                  e.printStackTrace();   
               }
            }
         }
         while (true);
         return newInfo;
      }
      catch (IOException ioe)
      {
      }
      return null;
   }

   
   
  /**
   * Closes the connection to the poddy-server; after this method has been called the PoddydClient
   * object canT be used any longer
   */
   public void close()
   {
      try
      {
         sock.close();
      } 
      catch (IOException e1)
      {
         e1.printStackTrace();
      }  
   }
}
