/*
 * QuestionPanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

import javax.swing.*;

import java.awt.*;



/**
 * This class can be used to ask simple questions that can be answered with one of two possible
 * options. The result of the positive answer is returned by getResultID(). 
 */
@SuppressWarnings("serial")
public class QuestionPanel extends DialoguePanel
{

   private int okID=-1;
   private JButton okButton=new JButton(),cancelButton=new JButton();
   
   
   
   /**
    * Default constructor for a question panel
    * @param object the element the question is asked for; if null is handed over here this value is ignored 
    * @param message a short message text / the question that has to be asked
    * @param yesID the unique Id that has to be returned by getResultID() in case the positive answer was selected
    * @param yesText the text for the positive answer
    * @param noText the text for the negative answer
    */
   public QuestionPanel(String object,String message,int yesID,String yesText,String noText)
   {
      super();
      
      JLabel msgLabel;
      this.okID=yesID;
      
      PoddyFrame.MyWin.setLCDMessageText(message);
      if (object!=null)
      {
    	  JLabel objLabel=new JLabel(object+":");
    	  objLabel.setForeground(PoddyFrame.highlight);
          add(objLabel,new GridBagConstraints(0,0, 1, 1, 0.0, 0.0,
           GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 10, 1, 10), 0, 0));
      }
      msgLabel=new JLabel(message);
      msgLabel.setForeground(PoddyFrame.stdentry);
      add(msgLabel,new GridBagConstraints(0,1, 1, 1, 0.0, 0.0,
                              GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 10, 1, 10), 0, 0));
      okButton.setText(yesText);
      add(okButton,new GridBagConstraints(0,2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 10, 1, 10), 0, 0));
      cancelButton.setText(noText);
      add(cancelButton,new GridBagConstraints(0,3, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 10, 1, 10), 0, 0));
      add(new JLabel(),new GridBagConstraints(0,4, 1, 1, 0.5, 0.5,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 10, 1, 10), 0, 0));
      okButton.setBackground(Color.WHITE);
      cancelButton.setBackground(Color.WHITE);
   }
   
   
   
   /**
    * This method is called either by the up/down commands of the remote control or by
    * pressing the cursor keys for up/down and modifies the selection of the answer
    * elements within the panel
    * @param dir specifies the direction the selection has to be changed into, here
    *        values 1 and -1 are allowed 
    */
   public boolean switchVertical(int dir)
   {
	  super.switchVertical(dir);
      if (dir>0)
      {
         if (selected==0) return false;
         selected--;
      }
      else if (dir<0)
      {
         if (selected>=1) return false;            
         selected++;
      }
      
      okButton.setBackground(Color.WHITE);
      cancelButton.setBackground(Color.WHITE);
      if (selected==0) 
      {
         okButton.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(okButton);
      }
      else if (selected==1) 
      {
         cancelButton.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(cancelButton);
      }
      return true;
   }

   
   
   /**
    * This method returns an Id that specifies the result of the operation
    * within the question panel. In case the positive answer was selected
    * the related yesID is returned, -1 otherwise.
    */
   public int getResultID()
   {
      super.getResultID();
      if (selected!=0) return -1;
      return okID;
   }
   
   
   
   /**
    * This method specifies if the panel can be aborted: if it returns true the
    * main application exits this panel in case the menu button was pressed on the
    * remote control or in case the F12 key was used  
    */
   public boolean canBeAborted()
   {
      super.canBeAborted();
      return true;
   }
   
   
}
