/*
 * RecordInfo.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;



/**
 * This is a storage class for recording information of the video recorder.
 */
public class RecordInfo
{
   public long        stime=0,rtime=0; /** stard and end tiome of a scheduled recording */
   public int         id=-1;           /** unique identifier of a scheduled recording; this id is equal to the database index */
   public String      name=null,station=null; /** name of the recording and name of the TV station where it has to be recorded at */



  /**
   * This is the default constructor
   */   
   public RecordInfo()
   {
   }
   
   
   
  /**
   * Constructor for an object of type RecordInfo
   * @param day the day the schedule starts at
   * @param shour the hour the shedule starts at
   * @param smin the minute if the hour the shedule starts at
   * @param ehour the hour the shedule ends at
   * @param emin the minute within the hour the shedule ends at
   * @param name the name for this recording; this name is also used for the name of the video file
   * @param station the name of the TV station where to record from
   */
   public RecordInfo(long day,int shour,int smin,int ehour,int emin,String name,String station)
   {
      day/=1000;
      stime=day+(shour*3600)+(smin*60);
      rtime=((ehour*3600)+(emin*60))-((shour*3600)+(smin*60));
      this.name=name.trim();
      this.name+=" "+(rtime/60);
      this.station=station;
   }
   
   
   
  /**
   * Constructor for an object of type RecordInfo
   * @param stime the starting time (in seconds) for the schedule
   * @param rtime the total recording time for the schedule
   * @param name the name for this recording; this name is also used for the name of the video file
   * @param station the name of the TV station where to record from
   * @param id a unique identifier for this object (also used by the SQL DB)
   */
   public RecordInfo(int stime,int rtime,String name,String station,int id)
   {
      this.stime=stime;
      this.rtime=rtime;
      this.name=name;
      this.station=station;
      this.id=id;
   }

}
