/*
 * RecordPanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.TimeZone;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.JTextComponent;

import com.vwp.swing.XIntegerField;



/**
 * This class implements a custom panel that can be used to program a new video recorder schedule. Here the selection
 * of the TV station, the name of the recording and the start and end time (via the vertical time table) is done
 */
@SuppressWarnings("serial")
public class RecordPanel extends CustomPanel
{
   private static final int   MAX_OTHER_REC=5; /** the maximum number of schedules that can be displayed within the time table of the same day */
   private StationsList       list;
   private NEJTextField       stationField=new NEJTextField(NEJField.MODE_LEFTRIGHT),
                              dayField=new NEJTextField(NEJField.MODE_LEFTRIGHT),
                              nameField=new NEJTextField(NEJField.MODE_INPUT);
   private JLabel             nameLabel=new JLabel();
   private JButton            stationButton=new JButton(),okButton=new JButton(PoddyFrame.MyWin._m.getString("ok"));
   private JLabel[]           hourLabel=new JLabel[27],minuteLabel=new JLabel[12];
   private JPanel[]           hourPanel=new JPanel[27],minutePanel=new JPanel[12];
   private int                stationSel=0,minuteSelected=-1,prevMinuteSelected=-1;
   private int                startHour=-1,startMin=-1,endHour=-1,endMin=-1;
   private long               date,startDate;
   private GridBagConstraints recConstraints=new GridBagConstraints(2,0,1,1,0.2,0.0,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(1,1,1,1), 0, 0);
   private JPanel             recPanel=new JPanel(),namePanel=new JPanel();
   private JLabel             recLabel=new JLabel();
   private SimpleDateFormat   formatter =new SimpleDateFormat("EEEE, d. MMMM yyyy (z Z)"),
                              formatter2=new SimpleDateFormat("yy.MM.dd");
   private JPanel[]           otherRecPanel=new JPanel[MAX_OTHER_REC];
   private JButton[]          otherStationButton=new JButton[MAX_OTHER_REC];
   private SQLLocalAccess     SQL;
   private DefaultListModel   result=null;
   private boolean            doASyncScroll=false;



  /**
   * The standard constructor for a RecordPAnel object
   * @param list the list of stations that is currently known and where the user can choose from
   * @param SQL the object to access the local database with for retrieving current schedules that possibly
   *        have to be displayed and for storing a new schedule when the panel is left
   */
   public RecordPanel(StationsList list,SQLLocalAccess SQL)
   {
      JLabel label;
      int    i;
      
      this.list=list;
      this.SQL=SQL;
      date=Calendar.getInstance().getTimeInMillis();

      date=date-(date%60000);    // remove seconds from day
      date=date-(date%3600000);  // ...minutes
      date=date-(date%86400000); // ...hours
      date-=Calendar.getInstance().getTimeZone().getRawOffset();
      startDate=date;

      recLabel.setForeground(Color.WHITE);
      recPanel.setBackground(Color.BLUE);
      recPanel.setBorder(new BevelBorder(BevelBorder.RAISED));
      recPanel.add(recLabel);
      
      label=new JLabel(PoddyFrame.MyWin._m.getString("station"));
      label.setForeground(PoddyFrame.stdentry);
      add(label,new GridBagConstraints(0,0, 1, 1, 0.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14,1,1), 0, 0));
      
      add(stationField,new GridBagConstraints(1, 0,2+MAX_OTHER_REC, 1, 1.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));
      add(new JLabel(),new GridBagConstraints(11,0, 1, 1, 0.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,14), 0, 0));
      setStation(0);
      
      label=new JLabel(PoddyFrame.MyWin._m.getString("date"));
      label.setForeground(PoddyFrame.stdentry);
      add(label,new GridBagConstraints(0,1, 1, 1, 0.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14,1,1), 0, 0));
      dayField.setText(formatter.format(date));
      add(dayField,new GridBagConstraints(1, 1,2+MAX_OTHER_REC, 1, 0.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));

      label=new JLabel(PoddyFrame.MyWin._m.getString("title"));
      label.setForeground(PoddyFrame.stdentry);
      add(label,new GridBagConstraints(0,2, 1, 1, 0.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14,1,1), 0, 0));
      setName();
      nameLabel.setForeground(Color.WHITE);
      namePanel.setOpaque(false);
      namePanel.setLayout(new BorderLayout());
      namePanel.add(nameField,BorderLayout.CENTER);
      namePanel.add(nameLabel,BorderLayout.EAST);
      add(namePanel,new GridBagConstraints(1,2,2+MAX_OTHER_REC, 1, 0.4, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));
   
      stationButton.setBorder(null);
      stationButton.setBackground(Color.WHITE);
      add(stationButton,new GridBagConstraints(2,3,1, 1, 0.2, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5,1,1,1), 0, 0));
      
      for (i=0; i<27; i++)
      {
         if (i<24) hourLabel[i]=new JLabel(""+i+":00");
         else  hourLabel[i]=new JLabel(""+(i-24)+":00");
         hourLabel[i].setForeground(PoddyFrame.stdentry);
         hourPanel[i]=new JPanel();
         hourPanel[i].setOpaque(false);
         hourPanel[i].setLayout(new BorderLayout());
         hourPanel[i].add(hourLabel[i],BorderLayout.CENTER);
         add(hourPanel[i],new GridBagConstraints(0,i*12+10, 1, 1, 0.0, 0.0,
               GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,14,1,1), 0, 0));         
      }
      for (i=0; i<12; i++)
      {
         if (i<2) minuteLabel[i]=new JLabel(":0"+(i*5));
         else minuteLabel[i]=new JLabel(":"+(i*5));
         minuteLabel[i].setForeground(PoddyFrame.stdentry);
         minutePanel[i]=new JPanel();
         minutePanel[i].setOpaque(false);
         minutePanel[i].setLayout(new BorderLayout());
         minutePanel[i].add(minuteLabel[i],BorderLayout.CENTER);
      }
      for (i=0; i<MAX_OTHER_REC; i++)
      {
         otherRecPanel[i]=new JPanel();
         otherRecPanel[i].setBorder(new BevelBorder(BevelBorder.RAISED));
         otherRecPanel[i].setBackground(Color.DARK_GRAY);
         
         otherStationButton[i]=new JButton();
         otherStationButton[i].setBorder(null);
         otherStationButton[i].setBackground(Color.WHITE);
      }
      add(okButton,new GridBagConstraints(0,1000,12,1,1.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,14,14,14), 0, 0));      
      okButton.setBackground(Color.WHITE);
      setOtherRecordings(date);
      switchVertical(-1);  
   }

   
   
  /**
   * This method accesses the local database and fetches information about all recordings for a day. When there are
   * other schedules programmed for this day they are added to the time table so that the user is able to see if
   * the new schedule would overlap with existing ones or not
   * @param date the day to look for other schedules
   */
   private void setOtherRecordings(long date)
   {
      int              i,startPos,endLen;
      RecordInfo       info;
      
      for (i=0; i<MAX_OTHER_REC; i++) 
         {
         remove(otherRecPanel[i]);
         remove(otherStationButton[i]);
         }
      result=SQL.doSQLQuery(SQLAccess.T_SCHEDULE,"SELECT * FROM "+SQLAccess.ScheduleTable+" WHERE deleted='0' AND stime>='"+(date/1000)+"' AND stime<'"+((date/1000)+93600)+"' AND stime+rtime>'"+(Calendar.getInstance().getTimeInMillis()-Calendar.getInstance().getTimeZone().getRawOffset())/1000+"' ORDER BY stime");

      if ((result!=null) && (result.size()>0)) for (i=0; i<MAX_OTHER_REC && i<result.size(); i++)
      {
         info=(RecordInfo)result.elementAt(i);

         startPos=(int)(info.stime-(date/1000));
         startPos=startPos/300;
         endLen=(int)(info.rtime/300);
         
//         otherRecLabel[i].setText(timeFormatter.format(info.stime*1000)+"-"+timeFormatter.format(info.stime*1000+info.rtime*1000));
         add(otherRecPanel[i],new GridBagConstraints(3+i,10+startPos,1, endLen, 0.1, 0.0,
               GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(1,1,1,1), 0, 0));

         otherStationButton[i].setIcon(new ImageIcon(Toolkit.getDefaultToolkit().createImage(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/img/"+info.station.toLowerCase().replaceAll(" ","")+".gif")));
         add(otherStationButton[i],new GridBagConstraints(3+i,3,1, 1, 0.1, 0.0,
               GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5,1,1,1), 0, 0));

      }
      else result=null;
      revalidate();
      repaint();
   }


  /**
   * Sets a new icon for a station; this method is called during initialization of the panel and whenever the user
   * witches over to a different TV station
   * @param num the number of the TV station within the stations list
   */
   private void setStation(int num)
   {
      stationField.setText(list.stations.elementAt(num).name);
      stationButton.setIcon(new ImageIcon(Toolkit.getDefaultToolkit().createImage(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/img/"+list.stations.elementAt(num).name.toLowerCase().replaceAll(" ","")+".gif")));
   }

   
   
  /**
   * Sets a new name for the schedule that is currently edited, this name is automatically created
   * out of the start time of the schedule and the TV station name
   */
   public void setName()
   {
      nameLabel.setText(" "+formatter2.format(date)+" "+stationField.getText());
   }
   
   
  /**
   * This is the local implementation of the method defined in the base class. Here all
   * key events are processed that are not handled by other methods. These key events are
   * related to text input for the name of the schedule, so there all character inputs,
   * shift, and backspace is processed in order to get a comfortable text input behaviour.
   * @param keyCode the VK_xxx key code constant of the pressed key
   * @param keyChar the character of the pressed key (if there exists a printable character)
   */
   public void otherKeyEvent(int keyCode,char keyChar)
   {
      super.otherKeyEvent(keyCode, keyChar);
      
      JTextComponent focusedInput=null;
      
      if (selected==2) focusedInput=nameField.getInputField();
      else return;
      
      if (keyCode==KeyEvent.VK_SHIFT) return;
      if (keyCode==KeyEvent.VK_BACK_SPACE)
      {
         if (focusedInput instanceof XIntegerField)
         {
            ((XIntegerField)focusedInput).setValue(((XIntegerField)focusedInput).getValue()/10);                        
         }
         else
         {
            focusedInput.setText(focusedInput.getText().substring(0,focusedInput.getText().length()-1));            
         }
      }
      else
      {
         if (focusedInput instanceof XIntegerField) try
         {
            int value;
            
            value=Integer.parseInt(""+keyChar);
            ((XIntegerField)focusedInput).setValue((((XIntegerField)focusedInput).getValue()*10)+value);            
         }
         catch (Exception e)
         {
            e.printStackTrace();
         }
         else
         {
            focusedInput.setText(focusedInput.getText()+keyChar);
         }
      }
   }
   


 
  /**
   * This is the local implementation of the method defined in the base class. Here the
   * horizontal selection of elements is handled whenever the user presses the left/right
   * cursor keys.<BR><BR>
   * This method checks if an element is selected that supports the horizontal selection
   * of data and - if yes - switches the related value according to the selection direction.
   * @param dir specifies the direction the horizontal switch is performed, this parameter
   *        is 1 for direction right and -1 for left
   */
   public boolean switchHorizontal(int dir)
   {
      if (selected==0) // TV station selection
      {
         stationSel+=dir;
         if (stationSel<0) stationSel+=list.stations.size();
         else if (stationSel>=list.stations.size()) stationSel-=list.stations.size();
         setStation(stationSel);
         setName();
      }
      else if (selected==1) // day selection
      {
         date+=(dir*86400000);         
         if (date<startDate) date=startDate;
         dayField.setText(formatter.format(date));
         setOtherRecordings(date);
         setName();
      }
      else if (minuteSelected<0)
      {
         selected+=(dir*10);
         if (selected<=4) selected=4;
         else if (selected>=28) selected=28;
         switchVertical(-dir);
      }
      return true;
   }
   
   
   
  /**
   * This is the local implementation of the method defined in the base class. Here the
   * vertical selection of elements/menu fields is handled whenever the user presses the
   * up/down cursor keys.<BR><BR>
   * This method switches through all available elements and counts its position to know which
   * one is the current one. Comparing to other custom panels here also the sub-item selection
   * is handled that allows the user to switch to minute-level within a choosen hour to
   * select an exact time.
   * @param dir specifies the direction the vertical switch is performed, this parameter
   *        is 1 for direction up and -1 for down
   */
   public boolean switchVertical(int dir)
   {
      if (minuteSelected>=0)
      {
         if (dir>0)
         {
            if (minuteSelected==0)
            {
               disableMinutes();
               switchVertical(0);
               return false;
            }
            minuteSelected--;
         }
         else if (dir<0)
         {
            if (minuteSelected>=11)
            {
               disableMinutes();
               switchVertical(dir);
               return false;            
            }
            minuteSelected++;
         }         
      }
      else
      {
         if (dir>0)
         {
            if (selected==0) return false;
            selected--;
         }
         else if (dir<0)
         {
            if (selected>=list.stations.size()) return false;            
            selected++;
         }
      }
      stationField.setBackground(Color.WHITE);
      dayField.setBackground(Color.WHITE);
      nameField.setBackground(Color.WHITE);
      okButton.setBackground(Color.WHITE);
      if ((prevSelected>=3) && (prevSelected<30))
      {
         hourPanel[prevSelected-3].setOpaque(false);
         hourPanel[prevSelected-3].setBackground(PoddyFrame.bgcolor);
      }
      if (prevMinuteSelected>=0)
      {
         minutePanel[prevMinuteSelected].setOpaque(false);
         minutePanel[prevMinuteSelected].setBackground(PoddyFrame.bgcolor);         
      }
      if (minuteSelected>=0)
      {
         minutePanel[minuteSelected].setOpaque(true);
         minutePanel[minuteSelected].setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(minutePanel[minuteSelected],doASyncScroll);
         doASyncScroll=false;
      }
      else if (selected==0)
      {
         stationField.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(stationField,doASyncScroll);
         doASyncScroll=false;
      }
      else if (selected==1) 
      {
         dayField.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(dayField,doASyncScroll);
         doASyncScroll=false;
      }
      else if (selected==2) 
      {
         nameField.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(nameField,doASyncScroll);
         doASyncScroll=false;
      }
      else if ((selected>=3) && (selected<30))
      {
         hourPanel[selected-3].setOpaque(true);
         hourPanel[selected-3].setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(hourPanel[selected-3],doASyncScroll);
         doASyncScroll=false;
      }
      else if (selected==30) 
      {
         okButton.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(okButton,doASyncScroll);
         doASyncScroll=false;
      }

      if (dir!=0)
      {
         prevSelected=selected;
         prevMinuteSelected=minuteSelected;
      }
      if (endHour>=0)
      {
         int    h;
         String s;
         
         if (endMin>=0) h=(endHour*12+(endMin/5)) - (startHour*12+(startMin/5));
         else 
         {
            h=(endHour*12+minuteSelected) - (startHour*12+(startMin/5));

            s=getTimeString(startHour,startMin)+"-";
            s+=getTimeString(endHour,(minuteSelected*5));
            recLabel.setText(s);
         }
         if (h>0) recConstraints.gridheight=h;

         add(recPanel,recConstraints);
      }
      else if (startHour>=0)
      {
         recConstraints.gridy=10+(startHour*12);
         if (startMin>=0) recConstraints.gridy+=(startMin/5);
         else if (minuteSelected>0)
         {
            recConstraints.gridy+=minuteSelected;
            recLabel.setText(getTimeString(startHour,minuteSelected*5)+"-    ");
         }

         add(recPanel,recConstraints);
      }
      return true;
   }


   
  /**
   * Formats a given time to format hh:mm
   * @param hour the hour of the time to be formatted
   * @param min the minute of the time to be formatted
   * @return the string containing the formatted time
   */
   String getTimeString(int hour,int min)
   {
      String s;

      if (hour>23) hour-=24;
      s=hour+":";
      if (min<10) s+="0"+min;
      else s+=min;
      return s;
   }
   
   
  /**
   * When this method is called the minutes displayed for the currently selected
   * hour are removed, the time table is collapsed
   */
   private void disableMinutes()
   {
      int i;
      if (endMin<0) endHour=-1;
      if (startMin<0) 
      {
         startHour=-1;
         remove(recPanel);
      }
      for (i=0; i<12; i++) remove(minutePanel[i]);
      minuteSelected=-1;
      revalidate();
      repaint();
      doASyncScroll=true;
   }

   
   
  /**
   * This is the local implementation of the method defined in the base class. This
   * method is called before the RecordPanel is removed from the user interface. Here
   * the new schedule data are collected and stored within the local database. Here
   * the savong of data is done only in case the "OK" button is selected; that is evaluated
   * via the number of the "selected" menu field.
   */
   public boolean closePanel()
   {
      if (minuteSelected>=0)
      {
         if (startMin<0) startMin=minuteSelected*5;
         else endMin=minuteSelected*5;
         disableMinutes();
         switchVertical(-1);
      }
      else if ((selected>=3) && (selected<30))
      {
         int i;
         minuteSelected=0;
         for (i=0; i<12; i++)
          add(minutePanel[i],new GridBagConstraints(0,(selected-3)*12+10+i, 1, 1, 0.0, 0.0,
               GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,14,1,1), 0, 0));         
         revalidate();
         switchVertical(0);
         prevMinuteSelected=0;
         if (startHour<0)
         {
            startHour=selected-3;
            recConstraints.gridy=startHour*12+10;
            add(recPanel,recConstraints);
         }
         else 
         {
            endHour=selected-3;
            endMin=-1;
         }
      }
      if (((endHour>=0) && (endHour<startHour)) ||
          ((endHour>=0) && (endHour==startHour) && (endMin>=0) && (endMin<startMin)))
      {
         startHour=endHour; endHour=-1;
         startMin=endMin;   endMin=-1;
         switchVertical(0);
      }
      if (selected==30) 
      {
         RecordInfo info;
         
         info=new RecordInfo(date,startHour,startMin,endHour,endMin,nameField.getText()+nameLabel.getText(),stationField.getText());
         PoddyFrame.MyWin.saveRecording(info);
         return true;
      }
      return false;
   }


   
  /**
   * This is the local implementation of the method defined in the base class, it specifies
   * that the RecordPanel can be left without a result.
   */
   public boolean canBeAborted()
   {
      super.canBeAborted();
      return true;
   }

   
}
