/*
 * SQLLocalAccess.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import java.sql.*;

import javax.swing.*;



/**
 * This method inherits from SQLAccess and implements the functionalities for accessing the local
 * SmallSQL database. For a description of the inherited method splease refer to the base class.
 */
public class SQLLocalAccess extends SQLAccess
{

          
	           
  /**
   * This method sends a raw SQL statement to the database
   * @param Cmd the SQL statement that has to be sent to the database
   */
   public void doSQLOperation(String Cmd)
   {
      try
      {
         SQLSt.execute(Cmd);
      }
      catch (SQLException se)
      {
    	 errorText=se.getMessage();
//	    	 se.printStackTrace();
      }         
   }

    
    
  /**
   * This method sends a raw SQL statement to the database and puts the results in a list of objects
   * that belong to the type of the result
   * @param table a constant T_xxx that specifies the table the results come from and - out of this
   *        information - selects the correct object type to put the resulting data into
   * @param Cmd the raw SQL statement the data are requested with
   * @return a list of objects containing the resulting data or null or an empty list if no data
   *        could be found 
   */
   public DefaultListModel doSQLQuery(int table,String Cmd)
   {
      int              rctr=0;
      DefaultListModel result;
      ResultSet        rs;

      lastCmd=System.currentTimeMillis();
      result=new DefaultListModel();
      try
      {
         rs= SQLSt.executeQuery(Cmd);
         while (rs.next())
         {
            if (table==T_CDS) result.addElement(new CDInfo(rs.getInt("idx"),
                                                           rs.getString("artist"),rs.getString("album"),rs.getString("genre"),rs.getString("category"),
                                                           rs.getInt("year"),rs.getInt("tracks")));
            else if (table==T_DVDS) result.addElement(new DVDInfo(rs.getInt("idx"),"",rs.getInt("time"),rs.getInt("year"),rs.getInt("crcid"),
                                                                  rs.getString("title"),rs.getString("genre"),
                                                                  rs.getInt("season"),rs.getInt("disc")));
            else if (table==T_CDTRACKS) result.addElement(new TrackInfo(rs.getString("name"),rs.getInt("cnt"),rs.getInt("cdidx")));
            else if (table==T_STATIONS) result.addElement(new StationInfo(rs.getString("name"),rs.getString("channel")));
            else if (table==T_SCHEDULE) result.addElement(new RecordInfo(rs.getInt("stime"),rs.getInt("rtime"),rs.getString("name"),rs.getString("station"),rs.getInt("idx")));
            else if (table==T_COUNT) result.addElement(new Integer(rs.getInt(1)));
            else if (table==T_TEXT) result.addElement(rs.getString(1));
            rctr++;
         }
         return result;
      }
      catch (SQLException se)
      {
         errorText=se.getMessage();
         se.printStackTrace();
      }
      result=null;
      return null;
   }
    
	   
	   
   public DefaultListModel loadStations()
   {
      super.loadStations();
      return doSQLQuery(T_STATIONS,"SELECT * FROM "+StationsTable);   
   }

	

   public void saveStations(StationsList list,String uname,String pwd)
   {
      int i;
      PoddydClient poddyd;
      
      super.saveStations(list,uname,pwd);
      if ((list==null) || (list.stations==null) || (list.stations.size()<=0)) return;
      doSQLOperation("DROP TABLE "+StationsTable);      
      doSQLOperation("CREATE TABLE "+StationsTable+" ("+
                     "idx INT NOT NULL AUTO_INCREMENT, "+
                     "name TEXT, "+
                     "channel TEXT, "+
                     "PRIMARY KEY (idx) "+
                     ")");
      for (i=0; i<list.stations.size(); i++)
       doSQLOperation("INSERT INTO "+StationsTable+" (name, channel) VALUES ('"+list.stations.elementAt(i).name+"', '"+list.stations.elementAt(i).channel+"')");
      try
      {
         poddyd=new PoddydClient();
         poddyd.setStations(list,uname,pwd);
         poddyd.close();
      } 
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   
   
   public void saveRecording(RecordInfo info,String uname,String pwd)
   {
      super.saveRecording(info,uname,pwd);
      if (info==null) return;
      doSQLOperation("INSERT INTO "+ScheduleTable+" (rtime, station, name, stime) VALUES ('"+info.rtime+"', '"+info.station+"', '"+info.name+"', '"+info.stime+"')");
   }
   
   
       
   public boolean initJDBC()
   {
      super.initJDBC();
      try
      {
         Class.forName( "smallsql.database.SSDriver" );
         SQLCon = java.sql.DriverManager.getConnection( "jdbc:smallsql:poddydb?create=true" );
	 SQLSt = SQLCon.createStatement();
       }
       catch (Exception e1)
       {
          errorText=e1.toString();
          e1.printStackTrace();
          return false;
       }
       try
       {
          // check if the essential CDTable exists...
          SQLSt.executeQuery("SELECT * FROM "+CDTable+" WHERE idx=0");
/*	      try
	      {
	         SQLSt.executeQuery("ALTER TABLE "+CDTable+" ADD mastervolume DOUBLE default '0'");
	      }
	      catch (SQLException se2)
	      {
//	         se2.printStackTrace();
	      }*/
         }
         catch (SQLException se)
         {
	    // ..and create it if not
            doSQLOperation("CREATE TABLE "+CDTable+" ("+
       		               "idx INT NOT NULL AUTO_INCREMENT, "+
                           "cnt INT default '0', "+
                           "artist TEXT, "+
                           "album TEXT, "+
                           "genre TEXT, "+
                           "category TEXT, "+
                           "year INT, "+
                           "tracks INT, "+
                           "users INT, "+
                           "format INT, "+
                           "cust1 INT, "+
                           "cust2 INT, "+
                           "lastplayed INT default '0', "+
                           "mastervolume DOUBLE default '0', "+
                           "PRIMARY KEY (idx) "+
                           ")");
            doSQLOperation("CREATE TABLE "+CDTracksTable+" ("+
                           "idx INT NOT NULL AUTO_INCREMENT, "+
                           "cnt INT default '0', "+
                           "cdidx INT, "+
                           "name TEXT, "+
                           "users INT, "+
                           "format INT, "+
                           "cust1 INT, "+
                           "cust2 INT, "+
                           "lastplayed INT default '0', "+
                           "PRIMARY KEY  (idx) "+
                           ")");
         }
         try
         {
	    // check if the DVD table exists...
            SQLSt.executeQuery("SELECT * FROM "+DVDTable+" WHERE idx=0");
         }
         catch (SQLException se)
         {
	    // ... and create it if not
            doSQLOperation("CREATE TABLE "+DVDTable+" ("+
                            "idx INT NOT NULL AUTO_INCREMENT, "+
                            "cnt INT default '0', "+
                            "title TEXT, "+
                            "genre TEXT, "+
                            "crcid BIGINT, "+
                            "year INT, "+
                            "time INT, "+
                            "season INT, "+
                            "disc INT, "+
                            "users INT, "+
                            "format INT, "+
                            "cust1 INT, "+
                            "cust2 INT, "+
                            "lastplayed INT default '0', "+
                            "PRIMARY KEY (idx) "+
                            ")");
          }
          try
          {
	     // check if a schedule table exists...
             SQLSt.executeQuery("SELECT * FROM "+ScheduleTable+" WHERE idx=0");
          }
          catch (SQLException se)
          {
	     // ...and create it if not
             doSQLOperation("CREATE TABLE "+ScheduleTable+" ("+
                            "idx INT NOT NULL AUTO_INCREMENT, "+
                            "rtime INT default '0', "+
                            "station TEXT, "+
                            "name TEXT, "+
                            "synced INT default 0, "+
                            "deleted INT default 0, "+
                            "stime INT, "+
                            "PRIMARY KEY (idx) "+
                            ")");
            }
         return true;
      }



   public void closeJDBC()
   {
      try
      {
         if (SQLCon!=null) SQLCon.close();
      }
      catch (Exception e)
      {
      }
   }


}
