/*
 * SearchPanel.java
 *
 * Copyright (c) 2008 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import java.awt.*;
import java.awt.event.KeyEvent;

import javax.swing.*;



/**
 * This is a (currently unused base implementation of a) panel that allows it to search for special
 * items like artists, album names, track names, movies or series by their name. Entering of the name
 * can be done by using the number inputs of a remote control, they are mapped to letters like it is
 * known e.g. from mobile phones.
 * TODO: move the implementation of text input to the NEJTextField/of number input to the NEXIntegerField
 *       implement searching fully
 */
@SuppressWarnings("serial")
public class SearchPanel extends CustomPanel implements Runnable
{
   private JTextField     searchField=new JTextField();
   private int            lastNum=-1,charPos=0,resultNum=0;
   private long           lastNumSent=0;
   private String[]       chars={" `-!?.,","","ABC","DEF","GHI","JKL","MNO","PQRS","TUV","WXYZ"};
   private Thread         myThread=new Thread(this);
   private SQLLocalAccess SQL;
   private JButton        cancelButton=new JButton(PoddyFrame.MyWin._m.getString("cancel"));
   private JLabel         stretchLabel=new JLabel();

   public SearchPanel(SQLAccess SQL)
   {
      super();
      JButton button;
      
      this.SQL=(SQLLocalAccess)SQL;

      button=new JButton("<HTML><DIV ALIGN=CENTER>1<BR>&lt;-</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(0,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 10, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>2<BR>ABC</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(1,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>3<BR>DEF</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(2,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>4<BR>GHI</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(3,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>5<BR>JKL</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(4,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>6<BR>MNO</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(5,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>7<BR>PQRS</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(6,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>8<BR>TUV</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(7,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>9<BR>WXYZ</DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(8,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0));
      
      button=new JButton("<HTML><DIV ALIGN=CENTER>0<BR> _ </DIV></HTML>");
      button.setMargin(new Insets(1,1,1,1));
      add(button,new GridBagConstraints(9,0, 1, 1, 0.1, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 10), 0, 0));

      searchField.setText("_");
      add(searchField,new GridBagConstraints(0,1,10, 1, 1.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,10, 1, 10), 0, 0));
      searchField.setBackground(Color.WHITE);
      
      cancelButton.setBackground(Color.WHITE);
      add(cancelButton,new GridBagConstraints(0,2,10, 1, 1.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,10, 1, 10), 0, 0));
      
      add(stretchLabel,new GridBagConstraints(0,3,10, 1, 0.0, 1.0,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1,10, 1, 10), 0, 0));      

      myThread.start();
   }

   
   
   public boolean switchVertical(int dir)
   {
      if (dir>0)
      {
         if (selected==0) return false;
         selected--;
      }
      else if (dir<0)
      {
         if (selected>=1+resultNum) return false;            
         selected++;
      }
      cancelButton.setBackground(Color.WHITE);
      if (selected==0) 
      {
         cancelButton.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(cancelButton);
      }
      return true;      
   }
   
   
   
   
   
   public void run()
   {
      while (true)
      {
         if (lastNumSent+1500<System.currentTimeMillis())
         {
            if (!searchField.getText().endsWith("_"))
            {
               searchField.setText(searchField.getText()+"_");
               charPos=0;
            }
            lastNumSent=System.currentTimeMillis();
         }
         com.vwp.util.Tools.threadSleep(200);
      }
   }
   
   public void sendNum(int num)
   {
      super.sendNum(num);
      
      String newText;
      
      lastNumSent=System.currentTimeMillis();
      if (num==1)
      {
         doBackspace();
         return;
      }
      else if (num!=lastNum)
      {
         charPos=0;
         if (!searchField.getText().endsWith("_"))
          searchField.setText(searchField.getText()+"_");
      }
      else
      {
         charPos++;
         if (charPos>=chars[num].length()) charPos=0;
      }
      newText=searchField.getText().substring(0,searchField.getText().length()-1);
      newText+=chars[num].substring(charPos,charPos+1);
      searchField.setText(newText);
      lastNum=num;
      search();
   }

   
   
   private void doBackspace()
   {
      charPos=0;
      if (searchField.getText().length()>0)
       searchField.setText(searchField.getText().substring(0,searchField.getText().length()-1));
      if (searchField.getText().length()<=0)
       searchField.setText("_");      
   }
   
   
   public void otherKeyEvent(int keyCode,char keyChar)
   {
      super.otherKeyEvent(keyCode, keyChar);
      
      String newText;

      if (keyCode==KeyEvent.VK_SHIFT) return;
      if (keyCode==KeyEvent.VK_BACK_SPACE)
      {
         doBackspace();
         return;
      }
      
      if (searchField.getText().endsWith("_"))
       newText=searchField.getText().substring(0,searchField.getText().length()-1);
      else
       newText=searchField.getText();
      newText+=keyChar;
      searchField.setText(newText);
      search();
   }
   
   
   private void search()
   {
      DefaultListModel album_result,track_result;
      CDInfo           cdInfo;
      TrackInfo        trackInfo;
      String           s=searchField.getText();
      
      if (s.endsWith("_")) s=s.substring(0,s.length()-1);
      s.replace("'","`");

      album_result=SQL.doSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" WHERE album LIKE '"+s+"'");
      if ((album_result==null) || (album_result.size()==0))
      {
         album_result=SQL.doSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" WHERE album LIKE '"+s+"%'");
         if ((album_result==null) || (album_result.size()==0))
          album_result=SQL.doSQLQuery(SQLAccess.T_CDS,"SELECT * FROM "+SQLAccess.CDTable+" WHERE album LIKE '%"+s+"%'");            
      }
      if ((album_result!=null) && (album_result.size()>0))
      {
         for (int i=0; i<album_result.size(); i++)
         {
            cdInfo=(CDInfo)album_result.elementAt(i);
System.out.println(cdInfo.album+"  "+cdInfo.artist);            
         }
      }

   
      track_result=SQL.doSQLQuery(SQLAccess.T_CDTRACKS,"SELECT * FROM "+SQLAccess.CDTracksTable+" WHERE name LIKE '"+s+"'");
      if ((track_result==null) || (track_result.size()==0))
      {
         track_result=SQL.doSQLQuery(SQLAccess.T_CDTRACKS,"SELECT * FROM "+SQLAccess.CDTracksTable+" WHERE name LIKE '"+s+"%'");
         if ((track_result==null) || (track_result.size()==0))
          track_result=SQL.doSQLQuery(SQLAccess.T_CDTRACKS,"SELECT * FROM "+SQLAccess.CDTracksTable+" WHERE name LIKE '%"+s+"%'");            
      }
      if ((track_result!=null) && (track_result.size()>0))
      {
         for (int i=0; i<track_result.size(); i++)
         {
            trackInfo=(TrackInfo)track_result.elementAt(i);
System.out.println(trackInfo.name+"  "+trackInfo.cdidx);            
         }
      }
   
   
   }
   
   
   
}
