/*
 * SetupWin.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package base;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

import com.vwp.swing.*;

import java.awt.*;
import java.awt.event.*;



@SuppressWarnings("serial")
/**
 * This is the main window for the Poddy configuration application. It behaves similar like Poddy
 * itself and uses the same functionalities of it. Currently only Linux is supported so this setup
 * expects to act on a Linux system.
 */
public class SetupWin extends PoddyFrame
{
	private static final Color keyBgColor=new Color(15,15,15);
	private CustomPanel        rootPane;
	private JButton            keyFetchButton=new JButton(_m.getString("pref_poddy"));
	private JButton            upButton=new JButton(),downButton=new JButton(),
	                           leftButton=new JButton(),rightButton=new JButton(),
	                           retButton=new JButton(),alphaButton=new JButton();
	private JPanel             navPanel=new JPanel();
	private JTextArea          helpText=new JTextArea(); 
	
	SetupWin()
	{
	}
	
	
	/**
	 * This custom panel contains all the user interface elements
	 */
	class ConfigCustomPanel extends CustomPanel
	{
	    private NEXIntegerField leftOffsetField=new NEXIntegerField(config.x_offset,0,100,NEJField.MODE_LEFTRIGHT),
                                topOffsetField=new NEXIntegerField(config.y_offset,0,100,NEJField.MODE_LEFTRIGHT),
                                widthField=new NEXIntegerField(config.x_size,300,3000,NEJField.MODE_LEFTRIGHT),
                                avOffsetField=new NEXIntegerField(config.av_offset,-10000,10000,NEJField.MODE_LEFTRIGHT|NEJField.MODE_INPUT),
                                heightField=new NEXIntegerField(config.y_size,200,2000,NEJField.MODE_LEFTRIGHT);
        private NEJTextField    aspectField=new NEJTextField(NEJField.MODE_LEFTRIGHT),
                                mainDirField=new NEJTextField(NEJField.MODE_INPUT),
                                videoDirField=new NEJTextField(NEJField.MODE_INPUT),
                                cdDeviceField=new NEJTextField(NEJField.MODE_INPUT),
                                dvdDeviceField=new NEJTextField(NEJField.MODE_INPUT),
                                videoDeviceField=new NEJTextField(NEJField.MODE_INPUT),
                                audioDeviceField=new NEJTextField(NEJField.MODE_INPUT),
                                usbMountField=new NEJTextField(NEJField.MODE_INPUT|NEJField.MODE_LEFTRIGHT),
                                bgimageField=new NEJTextField(NEJField.MODE_INPUT),
                                tvNormField=new NEJTextField(NEJField.MODE_LEFTRIGHT),
                                freqTableField=new NEJTextField(NEJField.MODE_LEFTRIGHT),
                                unameField=new NEJTextField(NEJField.MODE_INPUT),
                                pwdField=new NEJTextField(NEJField.MODE_INPUT); 
        private JCheckBox       cdCheckBox=new JCheckBox(),movieCheckBox=new JCheckBox(),
                                seriesCheckBox=new JCheckBox(),videoCheckBox=new JCheckBox(),
                                vcrCheckBox=new JCheckBox(),usbCheckBox=new JCheckBox();
        private JButton         okButton=new JButton(_m.getString("ok")),cancelButton=new JButton(_m.getString("cancel"));
        private int             tvNormSel=0,freqTableSel=0,usbMountSel=0;
    	private String[]        freqTableDef={"","us-bcast",   "us-cable",   "us-cable-hrc","japan-bcast",
                                                 "japan-cable","europe-west","europe-east", "italy",
                                                 "newzealand" ,"australia"  ,"ireland",     "france",
                                                 "china-bcast","southafrica","argentina",   "australia-optus",
                                                 "russia"};
        private String[]        tvNormDef={"","NTSC-M","NTSC-JP","PAL","PAL-BG","PAL-DK","PAL-I","PAL-M","PAL-N",
    			                              "PAL-Nc","PAL-60","SECAM-L","SECAM-DK"};
		private JLabel          videoLabel,cdDeviceLabel,dvdDeviceLabel,videoDeviceLabel,audioDeviceLabel,usbMountLabel,tvNormLabel,freqTableLabel,avOffsetLabel;
		private String[]        usbMountDef={"","/mnt/flash","/mnt/usb","/mnt/stick"};
        
        /**
         * The standard constructor, here the user interface is set up
         */
		ConfigCustomPanel()
		{
			JLabel label;
			
		   	/*****************************************************************************************/
		   	label=new JLabel(_m.getString("pref_pos"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,0,2,1,0.0,0.0,
	                     GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		
		   	label=new JLabel(_m.getString("pref_x_offset"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,1,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(leftOffsetField, new GridBagConstraints(1,1,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 
		   	
		   	label=new JLabel(_m.getString("pref_y_offset"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,2,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(topOffsetField, new GridBagConstraints(1,2,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 
		   	
		   	label=new JLabel(_m.getString("pref_x_size"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,3,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(widthField, new GridBagConstraints(1,3,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 
		   	
		   	label=new JLabel(_m.getString("pref_y_size"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,4,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(heightField, new GridBagConstraints(1,4,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

		   	/*****************************************************************************************/
		   	label=new JLabel(_m.getString("pref_display"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,5,2,1,0.0,0.0,
	                     GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		
		   	if (config.is16_9) aspectField.setText("16:9");
		   	else aspectField.setText("4:3");
		   	label=new JLabel(_m.getString("pref_aspect"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,6,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(aspectField, new GridBagConstraints(1,6,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 
		   	
		   	tvNormDef[0]=config.tv_norm;
		   	tvNormField.setText(config.tv_norm);
		   	tvNormLabel=new JLabel(_m.getString("pref_tvnorm"));
		   	tvNormLabel.setForeground(PoddyFrame.stdentry);
		   	add(tvNormLabel, new GridBagConstraints(0,7,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(tvNormField, new GridBagConstraints(1,7,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

		   	cdCheckBox.setSelected(config.use_cds);
		   	label=new JLabel(_m.getString("pref_use_cds"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,8,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(cdCheckBox, new GridBagConstraints(1,8,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,10),0,0)); 
		   	
		   	movieCheckBox.setSelected(config.use_movies);
		   	label=new JLabel(_m.getString("pref_use_movies"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,9,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(movieCheckBox, new GridBagConstraints(1,9,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,10),0,0)); 
		   	
		   	seriesCheckBox.setSelected(config.use_series);
		   	label=new JLabel(_m.getString("pref_use_series"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,10,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(seriesCheckBox, new GridBagConstraints(1,10,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,10),0,0)); 
		   	
		   	videoCheckBox.setSelected(config.use_videos);
		   	label=new JLabel(_m.getString("pref_use_videos"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,11,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(videoCheckBox, new GridBagConstraints(1,11,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,10),0,0)); 

		   	vcrCheckBox.setSelected(config.use_videos);
		   	label=new JLabel(_m.getString("pref_use_vcr"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,12,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(vcrCheckBox, new GridBagConstraints(1,12,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,10),0,0)); 

		   	usbCheckBox.setSelected(config.use_usb);
		   	label=new JLabel(_m.getString("pref_use_usb"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,13,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(usbCheckBox, new GridBagConstraints(1,13,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,10),0,0)); 

		   	/*****************************************************************************************/
		   	label=new JLabel(_m.getString("pref_path"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,15,2,1,0.0,0.0,
	                     GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		   	
		   	mainDirField.setText(config.main_dir);
		   	label=new JLabel(_m.getString("pref_main_dir"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,16,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(mainDirField, new GridBagConstraints(1,16,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	
		   	
		   	videoDirField.setText(config.video_dir);
		   	videoLabel=new JLabel(_m.getString("pref_video_dir"));
		   	videoLabel.setForeground(PoddyFrame.stdentry);
		   	add(videoLabel, new GridBagConstraints(0,17,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(videoDirField, new GridBagConstraints(1,17,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	
		   	
		   	/*****************************************************************************************/
		   	label=new JLabel(_m.getString("pref_devices"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,20,2,1,0.0,0.0,
	                     GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		   	
		   	cdDeviceField.setText(config.cd_device);
		   	cdDeviceLabel=new JLabel(_m.getString("pref_cd_device"));
		   	cdDeviceLabel.setForeground(PoddyFrame.stdentry);
		   	add(cdDeviceLabel, new GridBagConstraints(0,21,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(cdDeviceField, new GridBagConstraints(1,21,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	
		   	
		   	dvdDeviceField.setText(config.dvd_device);
		   	dvdDeviceLabel=new JLabel(_m.getString("pref_dvd_device"));
		   	dvdDeviceLabel.setForeground(PoddyFrame.stdentry);
		   	add(dvdDeviceLabel, new GridBagConstraints(0,22,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(dvdDeviceField, new GridBagConstraints(1,22,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	

		   	videoDeviceField.setText(config.video_device);
		   	videoDeviceLabel=new JLabel(_m.getString("pref_video_device"));
		   	videoDeviceLabel.setForeground(PoddyFrame.stdentry);
		   	add(videoDeviceLabel, new GridBagConstraints(0,23,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(videoDeviceField, new GridBagConstraints(1,23,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	

            audioDeviceField.setText(config.audio_device);
            audioDeviceLabel=new JLabel(_m.getString("pref_audio_device"));
            audioDeviceLabel.setForeground(PoddyFrame.stdentry);
            add(audioDeviceLabel, new GridBagConstraints(0,24,1,1,0.0,0.0,
                   GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
            add(audioDeviceField, new GridBagConstraints(1,24,1,1,0.9,0.0,
                   GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));        

		   	usbMountDef[0]=config.usb_mount;
            usbMountField.setText(config.usb_mount);
            usbMountLabel=new JLabel(_m.getString("pref_usb_mount"));
            usbMountLabel.setForeground(PoddyFrame.stdentry);
            add(usbMountLabel, new GridBagConstraints(0,25,1,1,0.0,0.0,
                   GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
            add(usbMountField, new GridBagConstraints(1,25,1,1,0.9,0.0,
                   GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));        

		   	/*****************************************************************************************/
		   	label=new JLabel(_m.getString("pref_misc"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,30,2,1,0.0,0.0,
	                     GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		   	
		   	freqTableDef[0]=config.freq_table;
		   	freqTableField.setText(config.freq_table);
		   	freqTableLabel=new JLabel(_m.getString("pref_freqtable"));
		   	freqTableLabel.setForeground(PoddyFrame.stdentry);
		   	add(freqTableLabel, new GridBagConstraints(0,31,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(freqTableField, new GridBagConstraints(1,31,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

            avOffsetLabel=new JLabel(_m.getString("pref_avoffset"));
            avOffsetLabel.setForeground(PoddyFrame.stdentry);
            add(avOffsetLabel, new GridBagConstraints(0,32,1,1,0.0,0.0,
                   GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
            add(avOffsetField, new GridBagConstraints(1,32,1,1,0.9,0.0,
                   GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 

		   	bgimageField.setText(config.bgimage);
		   	label=new JLabel(_m.getString("pref_bgimage"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,33,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(bgimageField, new GridBagConstraints(1,33,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	

		   	unameField.setText(config.uname);
		   	label=new JLabel(_m.getString("pref_uname"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,34,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(unameField, new GridBagConstraints(1,34,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	

		   	pwdField.setText(config.pwd);
		   	label=new JLabel(_m.getString("pref_pwd"));
		   	label.setForeground(PoddyFrame.stdentry);
		   	add(label, new GridBagConstraints(0,35,1,1,0.0,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0)); 
		   	add(pwdField, new GridBagConstraints(1,35,1,1,0.9,0.0,
	                GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0)); 	   	

		   	/*****************************************************************************************/
		   	add(okButton, new GridBagConstraints(0,40,2,1,0.0,0.0,
	                     GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,10,1,10),0,0));
		   	add(cancelButton, new GridBagConstraints(0,41,2,1,0.0,0.0,
	                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,10,10,10),0,0));
			
            setEnableStates();
		   	switchVertical(-1);
		}

		
		
		private void setEnableStates()
		{
			videoLabel.setEnabled(config.use_videos);
			videoDirField.setEnabled(config.use_videos);
			cdDeviceLabel.setEnabled(config.use_cds);
			cdDeviceField.setEnabled(config.use_cds);
			dvdDeviceLabel.setEnabled(config.use_movies || config.use_series);
			dvdDeviceField.setEnabled(config.use_movies || config.use_series);
			videoDeviceLabel.setEnabled(config.use_vcr);
			videoDeviceField.setEnabled(config.use_vcr);
            audioDeviceLabel.setEnabled(config.use_vcr);
            audioDeviceField.setEnabled(config.use_vcr);
            usbMountLabel.setEnabled(config.use_usb);
            usbMountField.setEnabled(config.use_usb);
			tvNormLabel.setEnabled(config.use_vcr);
			tvNormField.setEnabled(config.use_vcr);
			freqTableLabel.setEnabled(config.use_vcr);
			freqTableField.setEnabled(config.use_vcr);
            avOffsetLabel.setEnabled(config.use_vcr);
            avOffsetField.setEnabled(config.use_vcr);
		}

		
		
		   /**
		    * This class changes the selection/contents/values of several user interface elements.
		    * It is basically triggered by cursor key left/right and can be used for easy parameter
		    * modification. 
		    */
		   public boolean switchHorizontal(int dir)
		   {
		      if (selected==0)
		      {
		    	  config.x_offset+=dir;
		    	  if (config.x_offset<0) config.x_offset=0;
		    	  leftOffsetField.setValue(config.x_offset);
		      }
		      else if (selected==1)
		      {
		    	  config.y_offset+=dir;
		    	  if (config.y_offset<0) config.y_offset=0;
		    	  topOffsetField.setValue(config.y_offset);
		      }
		      else if (selected==2)
		      {
		    	  config.x_size+=dir;
		    	  if (config.x_size<0) config.x_size=0;
		    	  widthField.setValue(config.x_size);		    	  
		    	  PoddyFrame.MyWin.invalidate();
		    	  PoddyFrame.MyWin.repaint();
		      }
		      else if (selected==3)
		      {
		    	  config.y_size+=dir;
		    	  if (config.y_size<0) config.y_size=0;
		    	  heightField.setValue(config.y_size);		    	  
		      }
		      else if (selected==4)
		      {
		    	  config.is16_9=!config.is16_9;
		    	  if (config.is16_9) aspectField.setText("16:9");
		    	  else aspectField.setText("4:3");
		      }
		      else if (selected==5)
		      {
		         tvNormSel+=dir;
		         if (tvNormSel<0) tvNormSel+=tvNormDef.length;
		         else if (tvNormSel>=tvNormDef.length) tvNormSel-=tvNormDef.length;
		         tvNormField.setText(tvNormDef[tvNormSel]);
		      }
		      else if (selected==6)
		      {
		    	  cdCheckBox.setSelected(!cdCheckBox.isSelected());
		    	  config.use_cds=cdCheckBox.isSelected();
		    	  if (!cdCheckBox.isSelected()) usbCheckBox.setSelected(false);
		      }		      
		      else if (selected==7)
		      {
		    	  movieCheckBox.setSelected(!movieCheckBox.isSelected());
		    	  config.use_movies=movieCheckBox.isSelected();
		      }
		      else if (selected==8)
		      {
		    	  seriesCheckBox.setSelected(!seriesCheckBox.isSelected());
		    	  config.use_series=seriesCheckBox.isSelected();
		      }
		      else if (selected==9)
		      {
		    	   videoCheckBox.setSelected(!videoCheckBox.isSelected());
		    	   config.use_videos=videoCheckBox.isSelected();
		    	   if (!config.use_videos)
		    	   {
		    	      config.use_vcr=false;
		    	      vcrCheckBox.setSelected(false);
		    	   }
		      }
		      else if (selected==10)
		      {
		    	   vcrCheckBox.setSelected(!vcrCheckBox.isSelected());
		    	   config.use_vcr=vcrCheckBox.isSelected();
               if (config.use_vcr)
               {
                  config.use_videos=true;
                  videoCheckBox.setSelected(true);
               }
		      }
		      else if (selected==11)
		      {
		    	  usbCheckBox.setSelected(!usbCheckBox.isSelected());
		    	  config.use_usb=usbCheckBox.isSelected();
		    	  if (usbCheckBox.isSelected()) 
	    		  {
   	    		     config.use_cds=true;
                     cdCheckBox.setSelected(true);
	    		  }
		      }
		      else if (selected==18)
		      {
		         usbMountSel+=dir;
		         if (usbMountSel<0) usbMountSel+=usbMountDef.length;
		         else if (usbMountSel>=usbMountDef.length) usbMountSel-=usbMountDef.length;
		         usbMountField.setText(usbMountDef[usbMountSel]);
		      }
		      else if (selected==19)
		      {
		         freqTableSel+=dir;
		         if (freqTableSel<0) freqTableSel+=freqTableDef.length;
		         else if (freqTableSel>=freqTableDef.length) freqTableSel-=freqTableDef.length;
		         freqTableField.setText(freqTableDef[freqTableSel]);
		      }
            else if (selected==20)
            {
               avOffsetField.setValue(avOffsetField.getValue()+(dir*10));
               if (avOffsetField.getValue()<-10000) avOffsetField.setValue(-10000);
               else if (avOffsetField.getValue()>10000) avOffsetField.setValue(10000);
            }
		      PoddyFrame.MyWin.setBounds(config.x_offset,config.y_offset,config.x_size,config.y_size);
		      setEnableStates();
		      return true;      
		   }
		
		   
		   
		   /**
		    * This method handles other key events like real keyboard inputs and sends them to the
		    * user interface elements that can react on it. 
		    */
		   public void otherKeyEvent(int keyCode,char keyChar)
		   {
		      super.otherKeyEvent(keyCode, keyChar);
		      
		      JTextComponent focusedInput=null;
		      
		      if (selected==12) focusedInput=mainDirField.getInputField();
		      else if (selected==13) focusedInput=videoDirField.getInputField();
		      else if (selected==14) focusedInput=cdDeviceField.getInputField();
		      else if (selected==15) focusedInput=dvdDeviceField.getInputField();
		      else if (selected==16) focusedInput=videoDeviceField.getInputField();
              else if (selected==17) focusedInput=audioDeviceField.getInputField();
              else if (selected==18) focusedInput=usbMountField.getInputField();
		      else if (selected==20) focusedInput=avOffsetField.getInputField();
              else if (selected==21) focusedInput=bgimageField.getInputField();
		      else if (selected==22) focusedInput=unameField.getInputField();
		      else if (selected==23) focusedInput=pwdField.getInputField();
		      
		      if (focusedInput==null) return;
		      
		      if (keyCode==KeyEvent.VK_SHIFT) return;
		      if (keyCode==KeyEvent.VK_BACK_SPACE)
		      {
		         if (focusedInput instanceof XIntegerField)
		         {
		            ((XIntegerField)focusedInput).setValue(((XIntegerField)focusedInput).getValue()/10);                        
		         }
		         else
		         {
		            if (focusedInput.getText().length()>0)
		             focusedInput.setText(focusedInput.getText().substring(0,focusedInput.getText().length()-1));            
		         }
		      }
		      else
		      {
		         if (focusedInput instanceof XIntegerField) try
		         {
		            int value;
		            
		            if (keyChar=='-') ((XIntegerField)focusedInput).setValue(((XIntegerField)focusedInput).getValue()*-1);
		            else
		            {
		               value=Integer.parseInt(""+keyChar);
		               ((XIntegerField)focusedInput).setValue((((XIntegerField)focusedInput).getValue()*10)+value);
		            }
		         }
		         catch (Exception e)
		         {
		            e.printStackTrace();
		         }
		         else
		         {
		            focusedInput.setText(focusedInput.getText()+keyChar);
		         }
		      }		      
		   }

		   
		
		   /**
		    * Here the vertical selection of the user interface elements is changed. The current selection
		    * is stored in variable "selected". Its value is used at several positions to find out which
		    * element is selected. That important especially for the "OK" and "Cancel" button.<BR>
		    * This method is triggered on every cursor up/down key press.
		    */
		   public boolean switchVertical(int dir)
		   {
		      if (dir>0)
		      {
		         if (selected==0) return false;
		         selected--;
		      }
		      else if (dir<0)
		      {
		         if (selected>=25) return false;            
		         selected++;
		      }
		      leftOffsetField.setBackground(Color.WHITE);
		      topOffsetField.setBackground(Color.WHITE);
		      widthField.setBackground(Color.WHITE);
		      heightField.setBackground(Color.WHITE);
		      aspectField.setBackground(Color.WHITE);
		      tvNormField.setBackground(Color.WHITE);
		      mainDirField.setBackground(Color.WHITE);
		      videoDirField.setBackground(Color.WHITE);
		      cdDeviceField.setBackground(Color.WHITE);
		      dvdDeviceField.setBackground(Color.WHITE);
		      videoDeviceField.setBackground(Color.WHITE);
              audioDeviceField.setBackground(Color.WHITE);
              usbMountField.setBackground(Color.WHITE);
		      cdCheckBox.setBackground(Color.WHITE);
		      movieCheckBox.setBackground(Color.WHITE);
		      seriesCheckBox.setBackground(Color.WHITE);
		      videoCheckBox.setBackground(Color.WHITE);
		      vcrCheckBox.setBackground(Color.WHITE);
		      usbCheckBox.setBackground(Color.WHITE);
		      freqTableField.setBackground(Color.WHITE);
              avOffsetField.setBackground(Color.WHITE);
		      bgimageField.setBackground(Color.WHITE);
		      unameField.setBackground(Color.WHITE);
		      pwdField.setBackground(Color.WHITE);
		      okButton.setBackground(Color.WHITE);
		      cancelButton.setBackground(Color.WHITE);
		      if (selected==0) 
		      {
		    	  leftOffsetField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(leftOffsetField);
   		  	   	  setUseFlags(USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_overscan"));	  
		      }
		      else if (selected==1)
		      {
		    	  topOffsetField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(topOffsetField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_overscan"));	  
		      }
		      else if (selected==2) 
		      {
		    	  widthField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(widthField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_overscan"));	  
		      }
		      else if (selected==3) 
		      {
		    	  heightField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(heightField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_overscan"));	  
		      }
		      else if (selected==4) 
		      {
		    	  aspectField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(aspectField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_aspect"));	  
		      }
		      else if (selected==5) 
		      {
		    	  if (!config.use_vcr) switchVertical(dir);
		    	  else
		    	  {
		    	     tvNormField.setBackground(PoddyFrame.highlight);
		             scrollComponentToVisible(tvNormField);
   		  	   	     setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
     		  	     helpText.setText(_m.getString("pref_help_tvnorm"));	  
		    	  }
		      }
		      else if (selected==6) 
		      {
		    	  cdCheckBox.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(cdCheckBox);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_usecd"));
		      }
		      else if (selected==7) 
		      {
		    	  movieCheckBox.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(movieCheckBox);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_usemovies"));
		      }
		      else if (selected==8) 
		      {
		    	  seriesCheckBox.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(seriesCheckBox);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_useseries"));
		      }
		      else if (selected==9) 
		      {
		    	  videoCheckBox.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(videoCheckBox);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_usevideos"));
		      }
		      else if (selected==10) 
		      {
		    	  vcrCheckBox.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(vcrCheckBox);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_usevcr"));
		      }		      
		      else if (selected==11) 
		      {
		    	  usbCheckBox.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(vcrCheckBox);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	      helpText.setText(_m.getString("pref_help_useusb"));
		      }		      
		      else if (selected==12) 
		      {
		    	  mainDirField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(mainDirField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	      helpText.setText(_m.getString("pref_help_maindir"));
		      }
		      else if (selected==13) 
		      {
		    	  if (!config.use_videos) switchVertical(dir);
		    	  else
		    	  {
		    	     videoDirField.setBackground(PoddyFrame.highlight);
		             scrollComponentToVisible(videoDirField);
   		  	   	     setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	   	     helpText.setText(_m.getString("pref_help_videodir"));
		    	  }		    	  
		      }
		      else if (selected==14) 
		      {
		    	  if (!config.use_cds) switchVertical(dir);
		    	  else
		    	  {
		    	     cdDeviceField.setBackground(PoddyFrame.highlight);
		             scrollComponentToVisible(cdDeviceField);
   		  	   	     setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	   	     helpText.setText(_m.getString("pref_help_cddevice"));
		    	  }
		      }
		      else if (selected==15) 
		      {
		    	  if ((!config.use_series) && (!config.use_movies)) switchVertical(dir);
		    	  else
		    	  {
		    	     dvdDeviceField.setBackground(PoddyFrame.highlight);
		             scrollComponentToVisible(dvdDeviceField);
   		  	   	     setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	   	     helpText.setText(_m.getString("pref_help_dvddevice"));
		    	  }
		      }
		      else if (selected==16) 
		      {
		    	  if (!config.use_vcr) switchVertical(dir);
		    	  else
		    	  {
   		    	     videoDeviceField.setBackground(PoddyFrame.highlight);
		             scrollComponentToVisible(videoDeviceField);
   		  	   	     setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	   	     helpText.setText(_m.getString("pref_help_videodevice"));
		    	  }
		      }
            else if (selected==17) 
            {
              if (!config.use_vcr) switchVertical(dir);
              else
              {
                 audioDeviceField.setBackground(PoddyFrame.highlight);
                 scrollComponentToVisible(audioDeviceField);
                 setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
                 helpText.setText(_m.getString("pref_help_audiodevice"));
              }
            }
            else if (selected==18) 
            {
              if (!config.use_usb) switchVertical(dir);
              else
              {
                 usbMountField.setBackground(PoddyFrame.highlight);
                 scrollComponentToVisible(usbMountField);
                 setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
                 helpText.setText(_m.getString("pref_help_usbmount"));
              }
            }
		      else if (selected==19) 
		      {
		    	  if (!config.use_vcr) switchVertical(dir);
		    	  else
		    	  {
		    	     freqTableField.setBackground(PoddyFrame.highlight);
		             scrollComponentToVisible(freqTableField);
   		  	   	     setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT);
   		  	   	     helpText.setText(_m.getString("pref_help_freqtable"));
		    	  }
		      }
            else if (selected==20) 
            {
               if (!config.use_vcr) switchVertical(dir);
               else
               {
                  avOffsetField.setBackground(PoddyFrame.highlight);
                  scrollComponentToVisible(avOffsetField);
                  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_LEFT|USEFLAG_RIGHT|USEFLAG_NUM);
                  helpText.setText(_m.getString("pref_help_avoffset"));
               }
            }
		      else if (selected==21) 
		      {
		    	  bgimageField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(bgimageField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	      helpText.setText(_m.getString("pref_help_bgimage"));
		      }
		      else if (selected==22) 
		      {
		    	  unameField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(unameField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	      helpText.setText(_m.getString("pref_help_uname"));
		      }
		      else if (selected==23) 
		      {
		    	  pwdField.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(pwdField);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_ALPHA);
   		  	      helpText.setText(_m.getString("pref_help_pwd"));
		      }
		      else if (selected==24) 
		      {
		    	  okButton.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(okButton);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_DOWN|USEFLAG_RETURN);
   		  	      helpText.setText(_m.getString("pref_help_ok"));
		      }
		      else if (selected==25) 
		      {
		    	  cancelButton.setBackground(PoddyFrame.highlight);
		          scrollComponentToVisible(cancelButton);
   		  	   	  setUseFlags(USEFLAG_UP|USEFLAG_RETURN);
   		  	      helpText.setText(_m.getString("pref_help_cancel"));
		      }
           return true;
		   }
		   
		   
		   
		   /**
		    * This method is called in case return was hit. It checks if one of the
		    * buttons "OK" or "Cancel" is selected and returns true in this case.<BR>
		    * Here is is also checked if the contents of some user interface elements
		    * are valid. If not, the related element is marked in red and leaving
	        * not allowed.
		    */
		   public boolean closePanel()
		   {
		      super.closePanel();
		      
		      if (selected==24)
		      {
		    	  if ((mainDirField.getText().length()<=3) || (!mainDirField.getText().startsWith("/")))
		    	  {
		    		  mainDirField.setBackground(PoddyFrame.errorcolor);
		    		  scrollComponentToVisible(mainDirField);
		    		  return false;
		    	  }
		    	  else if ((videoDirField.getText().length()<=3) || (!videoDirField.getText().startsWith("/")))
		    	  {
		    		  videoDirField.setBackground(PoddyFrame.errorcolor);
		    		  scrollComponentToVisible(videoDirField);
		    		  return false;
		    	  }
		    	  return true;
		      }
		      else if (selected==25) return true;
		      else return false;
		   }
		   
		   
		   
		   /**
		    * This method is called when the panel is really closed, means in case the
		    * Buttons "OK" or "Cancel" are selected. It checks if "OK" was selected and stores the
		    * configured data in this case. 
		    */
		   public void postClosePanel()
		   {
			   if (selected==24) 
			   {
			      config.main_dir=mainDirField.getText();
				  config.video_dir=videoDirField.getText();
				  config.cd_device=cdDeviceField.getText();
				  config.dvd_device=dvdDeviceField.getText();
				  config.video_device=videoDeviceField.getText();
                  config.audio_device=audioDeviceField.getText();
                  config.usb_mount=usbMountField.getText();
			 	  config.bgimage=bgimageField.getText();
				  config.tv_norm=tvNormField.getText();
				  config.freq_table=freqTableField.getText();
                  config.av_offset=avOffsetField.getValue();
				  config.uname=unameField.getText();
				  config.pwd=pwdField.getText();			  
			      config.writeConfig();
			   }
		   }
		
	}
	
	
	   public void handleSetCmd()
	   {
	      super.handleSetCmd();
          if (customPanel.closePanel()) 
          {
        	  customPanel.postClosePanel();
        	  System.exit(0);
          }
	   }
	
	
	public void initUI()
	{
		super.initUI();
		getContentPane().setBackground(PoddyFrame.bgcolor);
		setBounds(config.x_offset,config.y_offset,config.x_size,config.y_size);
	   	setUndecorated(true);
//	   	getContentPane().setLayout(new BorderLayout());
	   	setLayout(new BorderLayout());
//	   	setBackground(new Color(0,0,150));
	   	add(keyFetchButton,BorderLayout.NORTH);
	   	navPanel.setBorder(new LineBorder(Color.WHITE));
	   	navPanel.setLayout(new GridBagLayout());
	   	navPanel.setBackground(PoddyFrame.bgcolor);
	   	try
	   	{
  	   	   upButton.setIcon(new ImageIcon(getClass().getResource("/img/cup.png")));
	   	   downButton.setIcon(new ImageIcon(getClass().getResource("/img/cdown.png")));
	   	   leftButton.setIcon(new ImageIcon(getClass().getResource("/img/cleft.png")));
	   	   rightButton.setIcon(new ImageIcon(getClass().getResource("/img/cright.png")));
	   	   alphaButton.setIcon(new ImageIcon(getClass().getResource("/img/a.png")));
	   	   retButton.setIcon(new ImageIcon(getClass().getResource("/img/return.png")));
	   	}
	   	catch (Exception e)
	   	{
	   	   e.printStackTrace();
	   	}
	   	helpText.setEditable(false);
	   	helpText.setRows(2);
	   	helpText.setLineWrap(true);
	   	helpText.setWrapStyleWord(true);
	   	navPanel.add(helpText, new GridBagConstraints(0,0,6,1,1.0,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,10,1,10),0,0)); 	   	
	   	navPanel.add(leftButton, new GridBagConstraints(0,1,1,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,10,10,1),0,0)); 	   	
	   	navPanel.add(upButton, new GridBagConstraints(1,1,1,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,1,10,1),0,0));     
	   	navPanel.add(downButton, new GridBagConstraints(2,1,1,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,1,10,1),0,0));  
	   	navPanel.add(rightButton, new GridBagConstraints(3,1,1,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,1,10,1),0,0));
	   	navPanel.add(alphaButton, new GridBagConstraints(4,1,1,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,1,10,1),0,0));
	   	navPanel.add(retButton, new GridBagConstraints(5,1,1,1,0.1,0.0,
                GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(1,1,10,10),0,0));
	   	add(navPanel,BorderLayout.SOUTH);	   	
	   	keyFetchButton.addKeyListener(keyEventHandler);
		rootPane=new ConfigCustomPanel();
        add(rootPane,BorderLayout.CENTER);
	   	
	   	customPanel=rootPane;
	   	setVisible(true);
	}
	
	
	
    public void setUseFlags(int flags)
	{
	   upButton.setEnabled((flags & USEFLAG_UP)!=0);
	   if ((flags & USEFLAG_UP)!=0) upButton.setBackground(keyBgColor);
	   else upButton.setBackground(Color.GRAY);
	   
	   downButton.setEnabled((flags & USEFLAG_DOWN)!=0);
	   if ((flags & USEFLAG_DOWN)!=0) downButton.setBackground(keyBgColor);
	   else downButton.setBackground(Color.GRAY);
	   
	   leftButton.setEnabled((flags & USEFLAG_LEFT)!=0);
	   if ((flags & USEFLAG_LEFT)!=0) leftButton.setBackground(keyBgColor);
	   else leftButton.setBackground(Color.GRAY);
	   
	   rightButton.setEnabled((flags & USEFLAG_RIGHT)!=0);
	   if ((flags & USEFLAG_RIGHT)!=0) rightButton.setBackground(keyBgColor);
	   else rightButton.setBackground(Color.GRAY);
	   
	   retButton.setEnabled((flags & USEFLAG_RETURN)!=0);
	   if ((flags & USEFLAG_RETURN)!=0) retButton.setBackground(keyBgColor);
	   else retButton.setBackground(Color.GRAY);
	   
	   alphaButton.setEnabled((flags & USEFLAG_ALPHA)!=0);
	   if ((flags & USEFLAG_ALPHA)!=0) alphaButton.setBackground(keyBgColor);
	   else alphaButton.setBackground(Color.GRAY);
	   
	}
	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
       PoddyFrame.MyWin=new SetupWin();
       PoddyFrame.MyWin.initUI();

	}

}
	

