/*
 * StationsList.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package base;


import java.util.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;

import javax.imageio.*;
import javax.swing.*;

import com.vwp.awt.image.*;



/**
 * The StationsList provides functionality to manage a complete list of TV stations that is available
 * for the main application 
 */
public class StationsList
{
   public  Vector<StationInfo> stations=new Vector<StationInfo>();

   
   
  /**
   * This method moves a station within two station lists. A given station is removed from the own station
   * list of the current object and put at the end of a given stations list. So this method can be used to
   * sort the list by different criteria
   * @param newStations the new Vector where the station has to be put into
   * @param stationName the name of the element that has to be removed from the current and added to the
   *        new list
   * @return the modified, new stations list
   */
   private Vector<StationInfo> moveStation(Vector<StationInfo> newStations,String stationName)
   {
      int                 i;
      
      for (i=0; i<stations.size(); i++)
      {
         if (stations.elementAt(i).name.toLowerCase().contains(stationName))
         {
            newStations.add(stations.elementAt(i));
            stations.remove(i);
         }
      }
      return newStations;
   }
   
   
   
  /**
   * This method is called after the main application scanned for new TV stations. It performs some
   * logic presorting that tries to put the TV stations within the own stations list into a better order.<BR>
   * TODO: currently the presorting only fits for germany, a parameter has to be introduced that identifies
   *       a country and provides a different presorting list that fits better for this country
   */
   public void presort()
   {
      Vector<StationInfo> newStations=new Vector<StationInfo>();
      
      newStations=moveStation(newStations,"ard");
      newStations=moveStation(newStations,"zdf");
      newStations=moveStation(newStations,"br-3");
      newStations=moveStation(newStations,"phoenix");
      newStations=moveStation(newStations,"arte");
      newStations=moveStation(newStations,"3sat");
      newStations=moveStation(newStations,"sat1");
      newStations=moveStation(newStations,"rtl");
      newStations=moveStation(newStations,"rtl 2");
      newStations=moveStation(newStations,"superrtl");
      newStations=moveStation(newStations,"pro 7");
      newStations=moveStation(newStations,"kabel 1");
      newStations=moveStation(newStations,"vox");
      newStations=moveStation(newStations,"das vierte");
      newStations=moveStation(newStations,"tele 5");
      newStations=moveStation(newStations,"orf1");
      newStations=moveStation(newStations,"orf2");
      while (stations.size()>0)
      {
         newStations.add(stations.elementAt(0));
         stations.remove(0);
      }
      stations=newStations;
   }
   
   
   
  /**
   * This method is executed whenever new stations have been scanned or whenever the name of a station has
   * been changed by the user. It invokes downloading of the logos of all used stations in order to display
   * them within the stations list later. These logos are buffered locally.
   */
   public void downloadStationLogos()
      {
      int i;
      
      for (i=0; i<stations.size(); i++)
         {            
         PoddyFrame.MyWin.setTopPanelText(PoddyFrame.MyWin._m.getString("saving_station")+" "+stations.elementAt(i).name);
         downloadFile(stations.elementAt(i).name.toLowerCase().replaceAll(" ","")+".gif",stations.elementAt(i).name.toLowerCase()+".jpg");
         }
      }   
   
   
   
  /**
   * This method belongs to downloadStationsLogos(). While the preceding one goes through the list of available
   * stations this one is cvalled for every station and fetches and rescales the image data for the logos from
   * the internet.
   */
   private void downloadFile(String file,String file2)
   {
      URL            msn;
      URLConnection  msnc;
      InputStream    in;
      DataOutputStream out;  
      byte[]         b=null;
      int            size,read=0,w,h;
      float          f;
      ImageUtils     iu;
      BufferedImage  bi;

      try
      {
         msn=new URL("http://www.virtualworlds.de/poddy/img/"+file);
         msnc = msn.openConnection();
         msnc.setConnectTimeout(2000);
         msnc.setReadTimeout(2000);
         in =msnc.getInputStream();
         size=msnc.getContentLength();
         b=new byte[size];

         while (size>read)
         {
            read=read+in.read(b,read,size-read);
         }
         in.close();
         try
         {
            out=new DataOutputStream(new FileOutputStream(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/img/"+file));
            out.write(b,0,b.length);
            out.close();
            return;
         }
         catch (IOException ioe)
         {
            ioe.printStackTrace();
         }
      }
      catch (Exception e)
      {
//         e.printStackTrace();
      }
    
      file2=file2.replace(" ","_");
      file2=file2.replace("ü","ue");
      file2=file2.replace("ö","oe");
      file2=file2.replace("ä","ae");
      
      try
      {
         msn=new URL("http://www.lyngsat-logo.com/logo/tv/"+file2.substring(0,1)+file2.substring(0,1)+"/"+file2);
         msnc = msn.openConnection();
         msnc.setConnectTimeout(2000);
         msnc.setReadTimeout(2000);
         in =msnc.getInputStream();
         size=msnc.getContentLength();
         b=new byte[size];

         while (size>read)
         {
            read=read+in.read(b,read,size-read);
         }
         in.close();
         
         iu=new ImageUtils();
         bi=iu.createBufferedImage(b,1.0);
         w=bi.getWidth();
         h=bi.getHeight();
         f=21.0f/h;
         bi=iu.scaleBufferedImage(bi,(int)(w*f),(int)(h*f));

         ImageIO.write(bi,"gif",new File(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/img/"+file));
      }
      catch (Exception e)
      {
//         e.printStackTrace();
      }  
   }
   
   
   
  /**
   * This method replaces the current list of stations by a new one.
   * @param list the new list of TV stations that has to be managed by this object
   */
   public void set(DefaultListModel list)
      {
      int i;
      
      if ((list==null) || (list.size()==0)) return;
      stations.removeAllElements();
      for (i=0; i<list.size(); i++) stations.add((StationInfo)list.elementAt(i));
      }  
   
   
   
  /**
   * This method is used by the recording mechanism. Whenever a scheduled recording has to be activated
   * this function is called to get the channel information for that recording
   * @param info the RecordInfo object that contains the schedule data
   * @return the name of the channel that has to be used by the TV card
   */
   public String getChannel(RecordInfo info)
      {
      int i;
      
      if (info==null) return null;
      for (i=0; i<stations.size(); i++)
       if (stations.elementAt(i).name.equals(info.station)) return stations.elementAt(i).channel;
      return null;
      }
   
}

