/*
 * StationsPanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

//import com.sun.awt.AWTUtilities.*;
import com.vwp.swing.*;



/**
 * This class is a custom panel that displays the list of available stations for editing
 * their names and to exchange their positions to bring them in an order that is useful
 * and easy to remember for the user.
 */
@SuppressWarnings("serial")
public class StationsPanel extends CustomPanel
{
   private StationsList    list;
   private JButton         okButton=new JButton(PoddyFrame.MyWin._m.getString("ok"));
   private int             choosen=-1,sNamesSel=0;
   private LineBorder      selBorder=new LineBorder(PoddyFrame.highlight);
   private boolean         editMode=false;
   String[]  sNames={"",
                     "3sat",              "ARD",               "Arena",   "Arte",
                     "BR-3",
                     "Das Vierte",        "Deutsche Welle TV", "DSF",
                     "EinsExtra",         "EinsFestival",      "EinsPlus","Euronews","Eurosport",
                     "Kabel 1",
                     "MDR",               "MTV",
                     "N24",               "NDR",               "Nord-3",  "n-tv",
                     "ORF1",              "ORF2",
                     "Pro 7",
                     "RBB",
                     "RTL",               "RTL 2",
                     "SAT 1",             "SF1",               "SF2",     "SWR-3",   "SR-3",      "Super-RTL",
                     "Tele 5",
                     "VH-1",              "VIVA",              "VOX",
                     "WDR-3",
                     "ZDF"}; /** a predefined list of(german) TV station names, they are displayed as preselected values when the horizontal cursor keys are pressed within the name field */

   

  /**
   * The standard constructor creates a panel that lists all known TV stations
   * @param list the list of TV stations to be displayed
   * @param editMode when this parameter is true the list of TV stations is displayed but the
   *        station itself is not enabled, in this mode the positions of the stations within
   *        the list can be changed and the names can be edited. When this value is false the
   *        list of the stations is displayed in a small window in front of the displayed TV
   *        program, no editing is possible but the list can be used to jump to a station, it
   *        is switched to that station in this case
   */   
   public StationsPanel(StationsList list,boolean editMode)
   {
      this.list=list;
      int       i;
      JLabel    label;
      ImageIcon icon;
      
      this.editMode=editMode;
      for (i=0; i<list.stations.size(); i++)
      {
         if (!editMode)
         {
            label=new JLabel(""+(i+1));
            label.setForeground(PoddyFrame.stdentry);
            mainPanel.add(label,new GridBagConstraints(0,i,1, 1, 0.0, 0.0,
                          GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,14,1,10), 0, 0));
         }
         else
         {
            list.stations.elementAt(i).custom=new NEJTextField(NEJField.MODE_INPUT|NEJField.MODE_LEFTRIGHT);
            ((NEJTextField)list.stations.elementAt(i).custom).setText(list.stations.elementAt(i).name);
            list.stations.elementAt(i).custom.setBackground(Color.WHITE);
            mainPanel.add(list.stations.elementAt(i).custom,new GridBagConstraints(1,i,1, 1, 1.0, 0.0,
                          GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));            
         }
         
         if (!editMode)
         {
            label=new JLabel(list.stations.elementAt(i).name);
            label.setForeground(PoddyFrame.stdentry);
            list.stations.elementAt(i).custom=new JPanel();
            ((JPanel)list.stations.elementAt(i).custom).setLayout(new BorderLayout());
            ((JPanel)list.stations.elementAt(i).custom).add(label,BorderLayout.WEST);
            ((JPanel)list.stations.elementAt(i).custom).setOpaque(false);
            mainPanel.add(list.stations.elementAt(i).custom,new GridBagConstraints(1,i,1, 1, 1.0, 0.0,
                          GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));
            icon=new ImageIcon(Toolkit.getDefaultToolkit().createImage(PoddyFrame.MyWin.sysAccess.getMainDirectory()+"/img/"+list.stations.elementAt(i).name.toLowerCase().replaceAll(" ","")+".gif"));
            if (icon.getImage()!=null)
            {
               list.stations.elementAt(i).custom2=new JButton();
               ((JButton)list.stations.elementAt(i).custom2).setBorder(null);
               ((JButton)list.stations.elementAt(i).custom2).setBackground(Color.WHITE);
               ((JButton)list.stations.elementAt(i).custom2).setIcon(icon);
               mainPanel.add(list.stations.elementAt(i).custom2,new GridBagConstraints(2,i,1, 1, 0.0, 0.0,
                             GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,14), 0, 0));
            }
         }  
      }
      mainPanel.add(okButton,new GridBagConstraints(0,list.stations.size(),3, 1, 1.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,14,14,14), 0, 0));      
      okButton.setBackground(Color.WHITE);
      switchVertical(-1); // select the first available element
/*System.out.println(com.sun.awt.AWTUtilities.isTranslucencySupported(Translucency.PERPIXEL_TRANSLUCENT));*/

      if (editMode)
      {
/*         if (com.sun.awt.AWTUtilities.isTranslucencySupported(Translucency.TRANSLUCENT))
          com.sun.awt.AWTUtilities.setWindowOpacity(PoddyFrame.MyWin,0.5f);*/
         PoddyFrame.MyWin.setBounds(new Rectangle(PoddyFrame.MyWin.config.x_offset+90,
                                               PoddyFrame.MyWin.config.y_offset+100,
                                               250,
                                               PoddyFrame.MyWin.config.y_size-120));
         PoddyFrame.MyWin.setTopPanelText("");
      }

      PoddyFrame.MyWin.toFront();
      PoddyFrame.MyWin.setAlwaysOnTop(true);
   }

   
     
   
  /**
   * This method is used in edit mode only, it exchanges the positions of two entries within the
   * list of TV stations
   * @param e1 list position of the first station that has to be exchanged with the second one 
   * @param e2 list position of the second station that has to be exchanged with the first one 
   */
   private void swapEntries(int e1,int e2)
   {
      StationInfo xInfo;
      
      if (editMode) return;

      mainPanel.add(list.stations.elementAt(e1).custom,new GridBagConstraints(1,e2,1, 1, 1.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));
      mainPanel.add(list.stations.elementAt(e2).custom,new GridBagConstraints(1,e1,1, 1, 1.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0));
      
      mainPanel.add(list.stations.elementAt(e1).custom2,new GridBagConstraints(2,e2,1, 1, 0.0, 0.0,
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,14), 0, 0));
      mainPanel.add(list.stations.elementAt(e2).custom2,new GridBagConstraints(2,e1,1, 1, 0.0, 0.0,
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,14), 0, 0));
      
      xInfo=list.stations.elementAt(e1);
      list.stations.set(e1,list.stations.elementAt(e2));
      list.stations.set(e2,xInfo);
   }
   
   
  /**
   * When the left/right cursor keys are used the behaviour of them depends on the edit mode. When it is
   * enabled they change the name of the currently selected station and switch through the list of
   * predefined station names. If editing is disabled this function jumps by 10 steps in the list of
   * stations (means it is identicall to calling switchVertical() ten times)
   * @param dir a direction value +1/-1 that specifies the direction of the pressed horizontal cursor
   *        keys 
   */
   public boolean switchHorizontal(int dir)
   {
      if (editMode)
      {
         if (selected<list.stations.size())
         {
            sNamesSel+=dir;
            if (sNamesSel<0) sNamesSel+=sNames.length;
            else if (sNamesSel>=sNames.length) sNamesSel-=sNames.length;
            ((NEJTextField)list.stations.elementAt(selected).custom).setText(sNames[sNamesSel]);
         }
      }
      else 
         {
         selected+=(dir*10);
         if (selected<=1) selected=1;
         else if (selected>=list.stations.size()-1) selected=list.stations.size()-1;
         switchVertical(-dir);
         }
      return true;
   }
   
   
  /**
   * This method is used to jump throught the list of stations in vertical direction, to select one of
   * them or to select the OK button. In editing mode here also exchanging of a selected station is handled
   * so that it can be moved through the list of stations to a new position.
   * @param dir a direction value +1/-1 that specifies the direction of the pressed horizontal cursor
   *        keys 
   */
   public boolean switchVertical(int dir)
   {
      if (dir>0)
      {
         if (selected==0) return false;
         selected--;
      }
      else if (dir<0)
      {
         if (selected>=list.stations.size()) return false;            
         selected++;
      }

      if (editMode)
      {
         if ((prevSelected>=0) && (prevSelected<list.stations.size()))
          list.stations.elementAt(prevSelected).custom.setBackground(Color.WHITE);
      }
      else
      {
         if ((prevSelected>=0) && (prevSelected<list.stations.size()))
         {
            ((JPanel)list.stations.elementAt(prevSelected).custom).setBorder(null);
            ((JPanel)list.stations.elementAt(prevSelected).custom).setBackground(Color.WHITE);
            ((JPanel)list.stations.elementAt(prevSelected).custom).setOpaque(false);
         }
      }
      okButton.setBackground(Color.WHITE);
      if (selected<list.stations.size())
      {
         if (editMode)
         {
            if (editMode) PoddyFrame.MyWin.sysAccess.switchTVChannel(list.stations.elementAt(selected));            
            list.stations.elementAt(selected).custom.setBackground(PoddyFrame.highlight);
            sNames[0]=((NEJTextField)list.stations.elementAt(selected).custom).getText();
            sNamesSel=0;
         }
         else
         {
            if ((choosen>=0) && (choosen<list.stations.size()))
            {
               swapEntries(choosen,selected);            
               choosen=selected;
               ((JPanel)list.stations.elementAt(selected).custom).setOpaque(true);
               ((JPanel)list.stations.elementAt(selected).custom).setBackground(PoddyFrame.highlight);
            }
            else ((JPanel)list.stations.elementAt(selected).custom).setBorder(selBorder);
            revalidate();
         }
         scrollComponentToVisible(list.stations.elementAt(selected).custom);
      }
      else 
      {
         okButton.setBackground(PoddyFrame.highlight);
         scrollComponentToVisible(okButton);
      }
      if (dir!=0) prevSelected=selected;
      return true;
   }

   

  /**
   * This method handles all other, non-specific key events. Here the text input functionality for
   * station names is implemented
   * @param keyCode the VK_xxx key constant of the pressed key
   * @param keyChar the printable character of the pressed key; this value is set to null when a key
   *        is pressed that does not have a printable character
   */   
   public void otherKeyEvent(int keyCode,char keyChar)
   {
      super.otherKeyEvent(keyCode, keyChar);
      
      if (editMode)
      {
         JTextComponent focusedInput=null;
         for (int i=0; i<list.stations.size(); i++) if (selected==i)
         {
            focusedInput=((NEJField)list.stations.elementAt(i).custom).getInputField();
            break;
         }
         if (focusedInput==null) return;
      
         if (keyCode==KeyEvent.VK_SHIFT) return;
         if (keyCode==KeyEvent.VK_BACK_SPACE)
         {
            if (focusedInput instanceof XIntegerField)
            {
               ((XIntegerField)focusedInput).setValue(((XIntegerField)focusedInput).getValue()/10);                        
            }
            else
            {
               if (focusedInput.getText().length()>0)
                focusedInput.setText(focusedInput.getText().substring(0,focusedInput.getText().length()-1));            
            }
         }
         else
         {
            if (focusedInput instanceof XIntegerField) try
            {
               int value;
            
               value=Integer.parseInt(""+keyChar);
               ((XIntegerField)focusedInput).setValue((((XIntegerField)focusedInput).getValue()*10)+value);            
            }
            catch (Exception e)
            {
               e.printStackTrace();
            }
            else
            {
               focusedInput.setText(focusedInput.getText()+keyChar);
            }
         }
      }
   }

   
   
  /**
   * This method is called before the StationsPanel is removed from the main application, here the
   * new list of stations is saved within the database, synchronization with a possibly existing remote
   * account is started and new/changed station logos are downloaded.
   */
   public boolean closePanel()
   {
      super.closePanel();
      if ((selected==list.stations.size()) || (editMode))
      {
         if (editMode)
         {
/*            if (com.sun.awt.AWTUtilities.isTranslucencySupported(Translucency.TRANSLUCENT))
             com.sun.awt.AWTUtilities.setWindowOpacity(PoddyFrame.MyWin,1f);*/
         }
         PoddyFrame.MyWin.setBounds(new Rectangle(PoddyFrame.MyWin.config.x_offset,
               PoddyFrame.MyWin.config.y_offset,
               PoddyFrame.MyWin.config.x_size,
               PoddyFrame.MyWin.config.y_size));
         PoddyFrame.MyWin.setTopPanelText("Poddy");
         PoddyFrame.MyWin.invalidate();
         PoddyFrame.MyWin.repaint();
         PoddyFrame.MyWin.setAlwaysOnTop(false);
         if (editMode) for (int i=0; i<list.stations.size(); i++)
          list.stations.elementAt(i).name=((NEJTextField)list.stations.elementAt(i).custom).getText();
         list.downloadStationLogos();
         PoddyFrame.MyWin.saveStations(list);
         if (editMode) PoddyFrame.MyWin.sysAccess.turnOffTV();
         PoddyFrame.MyWin.invalidate();
         return true;
      }
      if (choosen<0) choosen=selected;
      else choosen=-1;
      switchVertical(0);
      return false;
   }
   
   
  /**
   * Unconditioned leaving of a station panel is allowed in non-editing mode only. The result of
   * this method signals the calling main application if the panel can be left or not.
   * @return true in case aborting of the panel is allowed
   */
   public boolean canBeAborted()
   {
      super.canBeAborted();
      return (!editMode);
   }
   
   
   
}
