/*
 * TrackInfo.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

/**
 * This is a storage class for a single track within a CD album.
 */
public class TrackInfo
{
   public String name=null;
   public int    trackTime=0,currentTrackTime=0,trackNum=0,cnt=-1,cdidx=-1;
   
   
   
  /**
   * Constructor for a new object of type TrackInfo
   * @param trackNum the number of this track; this has to be a value greater than 0
   */
   public TrackInfo(int trackNum)
   {
      this.trackNum=trackNum;
   }
   
   
   
  /**
   * Constructor for a new object of type TrackInfo
   * @param track the name of the track; this value normally is the title of the song at this track
   * @param cnt the usage counter for this track, it holds the information how often this track was
   *        played; this value is stored within the database
   * @param cdidx a unique identifier for this track as it is used by the database
   */
   public TrackInfo(String track,int cnt,int cdidx)
   {
      try
      {
         this.trackNum=Integer.parseInt(track.substring(0,2));
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
      this.name=track.substring(5);
      this.cnt=cnt;
      this.cdidx=cdidx;
   }
   
   
   
  /**
   * This method can be used to securely set new track information. When a member of the
   * new track data is not used/initialized the own member is left unchanged. Only when a
   * member contains new data, the own member is overwritten.
   * @param copy the new TrackInfo object that contains the datas to be copied to the own
   *        ones 
   */
   public void set(TrackInfo copy)
   {
      if (copy.name!=null) name=copy.name;
      if (copy.trackTime!=0) trackTime=copy.trackTime;
      if (copy.trackNum!=0) trackNum=copy.trackNum;
      if (copy.cnt>0) cnt=copy.cnt;
   }
   
}

