package com.vwp.awt.image;

/*
 * ImageUtils.java 1.343 25.09.200
 *
 * Copyright (c) 2002, 2003 Virtual Worlds Productions
 * Email: virtual_worlds@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds@gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
Reviewed:
09. 06. 2003
*/

import java.awt.color.*;
import java.awt.image.*;
import java.awt.*;

import com.vwp.util.*;



/** This class contains a collection of useful functions related to image
    manipulation and handling. Some of them are useful especially together with
   Java 3D */
public class ImageUtils
   {



  /*************************************************************************
   * Main constructor
   */
   public ImageUtils()
      {
      }


/* The following function bases on a method originally taken from
 * com.sun.j3d.utils.image.TextureLoader(). Therefore the following copyright
 *  notice replaces the copyright of the whole class for the following function:
 */
/*
 *	@(#)TextureLoader.java 1.35 01/06/26 08:53:13
 *
 * Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

   /****************************************************************************
    * Creates a BufferedImage out of a picture (.gif, .jpg, ...)
    *
    * @param b the loaded picture file inside a byte-array
    * @param scalingFactor factor which has to be used to prescale the image
    * @return the BufferedImage created out of the picture
    */
   public synchronized BufferedImage createBufferedImage(byte[] b,double scalingFactor)
      {
      return createBufferedImage(b,scalingFactor,false,null);
      }



   /****************************************************************************
    * Creates a BufferedImage out of a picture (.gif, .jpg, ...)
    *
    * @param b the loaded picture file inside a byte-array
    * @param scalingFactor factor which has to be used to prescale the image
    * @return the BufferedImage created out of the picture
    */
   public synchronized BufferedImage createBufferedImage(byte[] b,double scalingFactor,boolean textureScaling)
      {
      return createBufferedImage(b,scalingFactor,textureScaling,null);
      }



   /****************************************************************************
    * Creates a BufferedImage out of a picture (.gif, .jpg, ...) This method
    * bases on a method from the original com.sun.j3d.utils.image.TextureLoader
    *
    * @param b the loaded picture file inside a byte-array
    * @param scalingFactor factor which has to be used to prescale the image
    * @param textureScaling scale to a size which fits to the J3D texture
    *         properties (width and height have to be a value equal to a power
    *         of two
    * @return the BufferedImage created out of the picture
    */
   public synchronized BufferedImage createBufferedImage(byte[] b,double scalingFactor,boolean textureScaling,Insets cropTexture)
      {
      Image     img;
      Toolkit MyToolkit=Toolkit.getDefaultToolkit();
      int[] bandOffset = { 0, 1, 2, 3}, nBits = {8, 8, 8, 8};
      ComponentColorModel colorModel = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB), nBits, true, false, Transparency.BITMASK, 0);

      if ((b==null) || (scalingFactor<=0)) throw new IllegalArgumentException(b+" "+scalingFactor);

      img=MyToolkit.createImage(b);
      {
      Container observer=new Container();
      observer.prepareImage(img, null);
      if (!waitForObserver(observer,img)) return null;
      }
      if (cropTexture!=null)
         {
         int newWidth=cropTexture.right-cropTexture.left,newHeight=cropTexture.bottom-cropTexture.top;
         if (scalingFactor==1)
            {
            WritableRaster wr=Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,newWidth,newHeight,newWidth*4,4,bandOffset,null);
            BufferedImage img2=new BufferedImage(colorModel,wr,false,null);
            Graphics g=img2.getGraphics();
            if ((cropTexture.left<0) || (cropTexture.top<0) || (newWidth>img.getWidth(null)) || (newHeight>img.getHeight(null)))
               {
               g.setColor(Color.black);
               g.fillRect(0,0,newWidth,newHeight);
               }
            g.drawImage(img,0,0,newWidth,newHeight,cropTexture.left,cropTexture.top,cropTexture.right,cropTexture.bottom,null);
            return img2;
            }
         BufferedImage img2=new BufferedImage(newWidth,newHeight,BufferedImage.TYPE_INT_RGB);
         img2.getGraphics().drawImage(img,0,0,newWidth,newHeight,cropTexture.left,cropTexture.top,cropTexture.right,cropTexture.bottom,null);
         img=img2;
         }
      if (scalingFactor!=1)
         {
         int newWidth,newHeight;

         newWidth=(int)(img.getWidth(null)*scalingFactor);   if (newWidth<2) newWidth=2;
         newHeight=(int)(img.getHeight(null)*scalingFactor); if (newHeight<2) newHeight=2;
         if (textureScaling)
            {
            newWidth=getClosestPowerOf2(newWidth);
            newHeight=getClosestPowerOf2(newHeight);
            }
         if ((newWidth!=img.getWidth(null)) || (newHeight!=img.getHeight(null)))
            {
            WritableRaster wr=Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE, newWidth, newHeight, newWidth*4, 4, bandOffset, null);
            BufferedImage bImage = new BufferedImage(colorModel, wr, false, null);
            Graphics g=bImage.getGraphics();
            g.drawImage(img,0,0,newWidth,newHeight,null);
            img.flush();
            return bImage;
            }
         }
      int width=img.getWidth(null);
      WritableRaster wr=Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE, width, img.getHeight(null) , width * 4, 4, bandOffset, null);
      BufferedImage bImage = new BufferedImage(colorModel, wr, false, null);
      Graphics g=bImage.getGraphics();
      if (g==null) return null;
      g.drawImage(img,0,0,null);
      img.flush();
      return bImage;
      }



  /*****************************************************************************
   * Evaluates a value which is a power of two and nearest to the given value
   * @param value - input value to which the power of two value has to be
   *          searched
   * @return the nearest power of two value
   */
   public synchronized int getClosestPowerOf2(int value)
      {
      if (value<=2) return value;

      int powerValue = 2,minBound;

      while (true)
         {
         if (value==powerValue) return value;
         if (value < powerValue)
            {
            // Found max bound of power, determine which is closest
            minBound = powerValue/2;
            if ((powerValue - value) > (value - minBound)) return minBound;
            else return powerValue;
            }
         powerValue *= 2;
         }
      }



  /*********************************************************************************
   * Scales a BufferedImage to a new size using a smooth scaling algorithm
   * (Image.SCALE_SMOOTH)
   * @param inBImg input BufferedImage which has to be scaled
   * @param w new width of the image
   * @param h nwe height of the image
   * @return the new, scaled BufferedImage
   */
   public BufferedImage scaleBufferedImage(BufferedImage inBImg,int w, int h)
      {
      Image         img2;
      Container     observer=new Container();
      BufferedImage bImage;

      img2=inBImg.getScaledInstance(w,h,Image.SCALE_SMOOTH);
      observer.prepareImage(img2,null);
      if (!waitForObserver(observer,img2)) return null;
      bImage = new BufferedImage(w,h,BufferedImage.TYPE_INT_ARGB);
      bImage.getGraphics().drawImage(img2, 0, 0, observer);
      if (!waitForObserver(observer,bImage)) return null;
      return bImage;
      }



   private boolean waitForObserver(Container observer,Image img)
      {
      int   status=0,timeout=0;

      if (img==null) return false;
      while (timeout<60)
         {
         timeout++;
         status = observer.checkImage(img, null);
         if ((status & ImageObserver.ERROR) == ImageObserver.ERROR) return false;
         else if ((status & ImageObserver.ALLBITS) == ImageObserver.ALLBITS) return true;
         else if (status==31) return true;
         else if (status==23) return true;
         Tools.threadSleep(110);
         }
      System.err.println("com.vwp.awt.image.ImageUtils.waitForObserver() timeout, status: "+status);
      return false;
      }
   }
