/*
 *	IconWidget.java 1.0 12.01.2009
 *
 * Copyright (c) 2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 */

package com.vwp.jLCD;

/**
 * A icon widget is able to display one single icon with a size of one single character.
 * 
 */
public class IconWidget extends Widget 
{
	public static final int ICON_BLOCK_FILLED=1;
	public static final int ICON_HEART_FILLED=2;
	public static final int ICON_HEART_OPEN=3;
	public static final int ICON_ARROW_UP=4;
	public static final int ICON_ARROW_DOWN=5;
	public static final int ICON_ARROW_LEFT=6;
	public static final int ICON_ARROW_RIGHT=7;
	public static final int ICON_CHECKBOX_OFF=8;
	public static final int ICON_CHECKBOX_ON=9;
	public static final int ICON_CHECKBOX_GRAY=10;
	public static final int ICON_SELECTOR_AT_LEFT=11;
	public static final int ICON_SELECTOR_AT_RIGHT=12;
	public static final int ICON_ELLIPSIS=13;
	public static final int ICON_STOP=14;
	public static final int ICON_PAUSE=15;
	public static final int ICON_PLAY=16;
	public static final int ICON_PLAYR=17;
	public static final int ICON_FF=18;
	public static final int ICON_FR=19;
	public static final int ICON_NEXT=20;
	public static final int ICON_PREV=21;
	public static final int ICON_REC=22;
	
    int    x;    /** left position the icon is positioned at, this value is 1-based */
    int    y;    /** top position the icon is positioned at, this value is 1-based */
    int    icon; /** the ID of the icon to be displayed */ 

    
    
    /**
     * The default constructor
     */
    IconWidget()
    {
    }
    
    
    
    /**
     * Constructor for a new widget; here the top and left position have to be given.
     * These parameters are 1-based, so a position of 1,1 specifies the upper left corner
     * of the display
     * @param x position of the text in horizontal direction in unit cells
     * @param y position of the text in vertical direction in unit cells
     * @param icon the ID of the icon that has to be displayed at this position
     */
    public IconWidget(int x, int y, int icon) {
        this.x = x;
        this.y = y;
        this.icon = icon;
    }
    
    

    /**
     * @return the current horizontal position in unit cells
     */
    public int getX() {
        return x;
    }

    
    
    /**
     * @param x the new horizontal position where the text has to be displayed at
     */
    public void setX(int x) {
        this.x = x;
        update();
    }

    
    
    /**
     * @return the current vertical position in unit cells
     */
    public int getY() {
        return y;
    }

    
    
    /**
     * @param y the new vertical position where the text has to be displayed at
     */
    public void setY(int y) {
        this.y = y;
        update();
    }

    
    
    /**
     * @return the icon ID that is currently displayed by this widget
     */
    public int getIcon() {
        return icon;
    }

    
    
    /**
     * @param icon a new icons ID that has to be displayed left-aligned by this widget
     */
    public void setIcon(int icon) {
        this.icon = icon;
        update();
    }
    
    
    
    /**
     * Refreshes the icon on the display: if was is overwritten by a different widget the call
     * to refresh updates the icon 
     */
    public void refresh()
    {
       update();
    }
    

    protected String getData() 
    {
    	String text;
    	
    	switch (icon)
    	{
    	case ICON_BLOCK_FILLED:
    		text="BLOCK_FILLED";
    		break;
    	case ICON_HEART_FILLED:
    		text="HEART_FILLED";
    		break;
    	case ICON_HEART_OPEN:
    		text="HEART_OPEN";
    		break;
    	case ICON_ARROW_UP:
    		text="ARROW_UP";
    		break;
    	case ICON_ARROW_DOWN:
    		text="ARROW_DOWN";
    		break;
    	case ICON_ARROW_LEFT:
    		text="ARROW_LEFT";
    		break;
    	case ICON_ARROW_RIGHT:
    		text="ARROW_RIGHT";
    		break;
    	case ICON_CHECKBOX_OFF:
    		text="CHECKBOX_OFF";
    		break;
    	case ICON_CHECKBOX_ON:
    		text="CHECKBOX_ON";
    		break;
    	case ICON_CHECKBOX_GRAY:
    		text="CHECKBOX_GRAY";
    		break;
    	case ICON_SELECTOR_AT_LEFT:
    		text="SELECTOR_AT_LEFT";
    		break;
	    case ICON_SELECTOR_AT_RIGHT:
	    	text="SELECTOR_AT_RIGHT";
	    	break;
	    case ICON_ELLIPSIS:
	    	text="ELLIPSIS";
	    	break;
	    case ICON_STOP:
	    	text="STOP";
	    	break;
	    case ICON_PAUSE:
	    	text="PAUSE";
	    	break;
	    case ICON_PLAY:
	    	text="PLAY";
	    	break;
	    case ICON_PLAYR:
	    	text="PLAYR";
	    	break;
	    case ICON_FF:
	    	text="FF";
	    	break;
	    case ICON_FR:
	    	text="FR";
	    	break;
	    case ICON_NEXT:
	    	text="NEXT";
	    	break;
	    case ICON_PREV:
	    	text="PREV";
	    	break;
	    case ICON_REC:
	    	text="REC";
	    	break;
    	default:
    		throw new RuntimeException("Illegal icon ID "+icon);
    	}    	
        return x + " " + y + " \"" + text + "\"";
    }

    
    
    public String getType() {
        return "string";
    }
    
}
