/*
 * 	LCD.java 1.0 29.12.2008
 *
 * Copyright (c) 2004 Gavin & Georgina Sallery
 * Copyright (c) 2008 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 */

package com.vwp.jLCD;

import com.vwp.io.*;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * This class is the starting point for all connections to an lcdproc server. It has to be used to
 * instantiate a connection to it and afterwards add a screen for displaying widgets on it.
 * 
 * Following an example is given that demonstrates the usage of the jLCD package. It was originally
 * developed in 2004 by Gavin & Georgina Sallery and is now incorporated into the Java X-Tools to ensure a further
 * development of it 
<PRE>
import com.vwp.jLCD.*;

public class TestCom 
{

  public static void main(String[] args) {
    int lcdPort = 13666;
    LCD myLCD;

    try
    {
       myLCD = new LCD(lcdPort, "90.0.0.30"); // replace this one by your servers IP!
    }
    catch (Exception e)
    {
    	e.printStackTrace();
    	return;
    }
    System.out.println("Found an LCD. Width = " + myLCD.getWidth() + ", height = " + myLCD.getHeight());
    // create a new screen and add it to the display
    Screen myScreen = new Screen();
    myScreen.setName("My first screen");
    myLCD.addScreen(myScreen);
    System.out.println("Screen Width = " + myScreen.getWidth() + ", height = " + myScreen.getHeight());
    
    // create a new graphical screen and add it to the display
    Screen myGraphScreen = new Screen();
    myGraphScreen.setName("My Graphical Screen");
    myLCD.addScreen(myGraphScreen);
    
    System.out.println("Added screens");
    
    // enable the heartbeat 
    myScreen.setHeartbeat(true);

    // let the backlight flesh 
    myLCD.flashBacklight(1000, 1000, 50000);

    // Add Widgets
    StringWidget stringWidget = new StringWidget(1, 1, "Hello World!");
    myScreen.addWidget(stringWidget);
    System.out.println(stringWidget.getX()+"  "+stringWidget.getX());

    ScrollerWidget scrollWidget = new ScrollerWidget(2, 3, 5, 4, "h", -5, "So you are a coward after all!, kneel" +
                                                                          " you fool, kneel before my whip and I shall..make...you...SCREAM!!!:-) Muuuhhhhaaa haaaaa!...Thwack");
    myScreen.addWidget(scrollWidget);

    TitleWidget titWidget = new TitleWidget("Title - Submission");
    myScreen.addWidget(titWidget);

    HBarWidget hbarWidget = new HBarWidget(20, 2, 7);
    HBarWidget hbarWidget1 = new HBarWidget(20, 3, 10);
    myScreen.addWidget(hbarWidget);
    myScreen.addWidget(hbarWidget1);

    VBarWidget verticalGraph = new VBarWidget(4, 3, 20);
    VBarWidget verticalGraph1 = new VBarWidget(6, 3, 15);
    VBarWidget verticalGraph2 = new VBarWidget(8, 3, 10);
    VBarWidget verticalGraph3 = new VBarWidget(10, 3, 18);
    myGraphScreen.addWidget(verticalGraph);
    myGraphScreen.addWidget(verticalGraph1);
    myGraphScreen.addWidget(verticalGraph2);
    myGraphScreen.addWidget(verticalGraph3);


    System.out.println("Added widget");
    while (true) {
      try {
        Thread.sleep(1000);
        //        stringWidget.setText("System time: " + System.currentTimeMillis());
      } catch (InterruptedException e) {
        e.printStackTrace();  //To change body of catch statement use Options | File Templates.
      }
    }
  } //end main
}
</PRE>
 * 
 * 
 */
public class LCD {

	class FlashThread extends Thread {
		int onPeriod;
		int offPeriod;
		int duration;
		long start;

		public FlashThread(int onPeriod, int offPeriod, int duration) {
			this.onPeriod = onPeriod;
			this.offPeriod = offPeriod;
			this.duration = duration;
			start = System.currentTimeMillis();
		}

		public void run() {
			try {
				while (System.currentTimeMillis() < start + duration) {
					setBacklight(true);
					Thread.sleep(onPeriod);
					setBacklight(false);
					Thread.sleep(offPeriod);
				}
				setBacklight(backlight);
			} catch (InterruptedException e) {
			}
		}
	}

	class ScreenListener extends Thread {
		private SafeBufferedReader in = null;
		private String lastLine = null;

		public ScreenListener(SafeBufferedReader in) {
			this.in = in;
		}

		public void run() {
			while (running) {
				try {
					String newLine = null;
					synchronized (in) {
						newLine = in.readLine();
					}
          if (DEBUG) {
            System.out.println("-- " + newLine);
          }
					if (newLine.length() > 6) {
						if (newLine.substring(0, 6).equals("listen")) {
              int screenID = Integer.parseInt(newLine.substring(7));
              Screen s = (Screen)screens.get(new Integer(screenID));
							if (s != null)
								s.listen();
							else
								System.out.println("Why is LCDd telling me that it's listening to screen " + screenID + "? It's not " +
								                   "one of mine...");
						} else if (newLine.substring(0, 6).equals("ignore")) {
              int screenID = Integer.parseInt(newLine.substring(7));
              Screen s = (Screen)screens.get(new Integer(screenID));
							if (s != null)
								s.ignore();
							else
								System.out.println("Why is LCDd telling me that it's ignoring screen " + screenID + "? It's not " +
								                   "one of mine...");
						} else
            lastLine = newLine;
					} else
						lastLine = newLine;
				} catch (IOException e) {
					e.printStackTrace();  //To change body of catch statement use Options | File Templates.
				}
			}
		}

		public String getLastLine() { // Non-blocking
			return lastLine;
		}

		public String getLine() { // This should block
			while (lastLine == null) {
        try {
          Thread.sleep(100);
        } catch (InterruptedException ie) {}
			}
			String result = lastLine;
			lastLine = null;
			return result;
		}
	}

	private static final boolean DEBUG = false;
	private HashMap screens = null;
	private int width, height;
	private int cellWidth, cellHeight;
	private int screenIDCnt;        //Counter for screen id's
	private PrintWriter out = null;
	protected SafeBufferedReader in = null;
	protected boolean running = false;
	private boolean backlight;

	/**
	 * The standard constructor for a new LCD object, it tries to establish a connection to the lcdproc
	 * server that itself controls the display
	 * @param port the port number the lcdproc server is listening at; default is 13666
	 * @param hostname the hostname or IP the lcdproc server is running at; please make sure
	 *        that the server is able to access connections from outside in case jLCD and lcdproc
	 *        are NOT running on the same machine!
	 */
	public LCD(int port, String hostname)
	throws IOException
	{
		Socket lcdSocket = null;
		screens = new HashMap();

		lcdSocket = new Socket(hostname, port);
		InetAddress host = lcdSocket.getInetAddress();
		in = new SafeBufferedReader(new InputStreamReader(lcdSocket.getInputStream()));
		out = new PrintWriter(new OutputStreamWriter(lcdSocket.getOutputStream(),"ISO-8859-1"));

		running = true;
		ScreenListener listener = new ScreenListener(in);
		listener.start();

		write("hello");
		String response = listener.getLine();

		double protocol = Double.parseDouble(response.substring(response.indexOf("protocol") + 8, response.indexOf(" lcd")));
		if (protocol != 0.3) 
			new RuntimeException("jLCD can only operate with LCDd protocol version 0.3. Exiting.");

		width = Integer.parseInt(response.substring(response.indexOf("wid") + 4, response.indexOf(" hgt")));
		height = Integer.parseInt(response.substring(response.indexOf("hgt") + 4, response.indexOf(" cellwid")));
		cellWidth = Integer.parseInt(response.substring(response.indexOf("cellwid") + 8, response.indexOf(" cellhgt")));
		cellHeight = Integer.parseInt(response.substring(response.indexOf("cellhgt") + 8, response.length()));
	}

	/**
	 * Get the width of the display in unit cells
	 * @return the width of the display
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * Get the height of the display in unit cells
	 * @return the height of the display
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * Add a screen to the current display. Screens are the base objects that have to be used for displaying
	 * @param s the screen to be added to the display
	 */
	public void addScreen(Screen s) {
		write("screen_add " + screenIDCnt);
		synchronized (screens) {
			screens.put(new Integer(screenIDCnt), s);
		}
		s.added(this, screenIDCnt);
		screenIDCnt++;
	}

	/**
	 * Remove a screen from the display
	 * @param s the screen that has t obe removed
	 */
	public void removeScreen(Screen s) {
		write("screen_del " + s.getID());
		synchronized (screens) {
			screens.remove(s.getID());
		}
	}

	/**
	 * Switch the backlight of the display (as far as supported by it)
	 * @param state true to turn on the backlight or false otherwise
	 */
	public void setBacklight(boolean state) {
		write("backlight " + (state?"on":"off"));
		backlight = state;
	}

	/**
	 * Gets the current state of the backlight (as far as supported by the display)
	 * @return true if backlight is turned on or false otherwise
	 */
	public boolean getBacklight() {
		return backlight;
	}

	/**
	 * Flash the backlight using the given pattern (only possible when backlight control is
	 * supported by the used display controller)
	 * @param onPeriod period in milliseconds the backlight has to be turned on
	 * @param offPeriod period in milliseconds the backlight has t obe turned off
	 * @param duration time in milliseconds the backlight flashing has to be used
	 */
	public void flashBacklight(int onPeriod, int offPeriod, int duration) {
		Thread th = new FlashThread(onPeriod, offPeriod, duration);
		th.start();
	}

	public void finalize() {
		running = false;
		try {
			if (in != null) in.close();
			if (out != null) out.close();
		} catch (IOException e) {
			e.printStackTrace();  //To change body of catch statement use Options | File Templates.
		}
	}

	synchronized void write(String s) {
//		System.out.println("Server write: " + s);
		out.println(s);
		out.flush();
	}

	/**
	 * Gives the height of one single cell of the display
	 * @return the height in pixels
	 */
	public int getCellHeight() {
		return cellHeight;
	}

	/**
	 * Gives the width of one single cell of the display
	 * @return the width in pixels
	 */
	public int getCellWidth() {
		return cellWidth;
	}

/*  public static String rightAlign(String input, int width) {
    String result = new String();
    if (input.length() > width)
      return input.substring(0, width);
    else {
      for (int i=0; i<((width - 1) - input.length()); i++)
        result += " ";
      result += input;
    }
    return result;
  }*/
}
