/*
 *	ScrollerWidget.java 1.2 05.01.2009
 *
 * Copyright (c) 2004 Gavin & Georgina Sallery
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 */

package com.vwp.jLCD;



public class ScrollerWidget extends StringWidget {
   public static final int SCROLL_HORIZONTAL=1; /** scroll direction: scroll the widget in horizontal direction */ 
   public static final int SCROLL_VERTICAL=2;   /** scroll direction: scroll the widget in vertical direction */
   
    private int right;
    private int bottom;
    private String direction;
    private int speed;

    /**
     * This creates a text widget that scrolls its contents continuously
     * @param left left position of text
     * @param top top position of text
     * @param right right end of text to display
     * @param bottom bottom end of text to display
     * @param direction scroll direction SCROLL_HORIZONTAL or SCROLL_VERTICAL
     * @param speed the scroll speed, this number defines the number of frames that have to elapse before a scroll movement is done
     * @param text the text to display
     */
    public ScrollerWidget(int x, int y, int right, int bottom, int direction, int speed, String text) {
        this.x = x;
        this.y = y;
        this.right = right;
        this.bottom = bottom;
        if (direction==SCROLL_HORIZONTAL) this.direction ="h";
        else this.direction="v";
        this.speed = speed;
        this.text = text;
    }

    public int getRight() {
        return right;
    }

    public void setRight(int right) {
        this.right = right;
    }

    public int getBottom() {
        return bottom;
    }

    public void setBottom(int bottom) {
        this.bottom = bottom;
    }

    public String getDirection() {
        return direction;
    }

    public void setDirection(String direction) {  //will need Enumeration value of h or v
        this.direction = direction;
    }

    public int getSpeed() {
        return speed;
    }

    public void setSpeed(int speed) {
        this.speed = speed;
    }

    /**
     * @deprecated use getX() instead
     */
    public int getLeft() {
        return x;
    }

    /**
     * @deprecated use setX() instead
     */
    public void setLeft(int left) {
        this.x = left;
        update();
    }

    /**
     * @deprecated use getY() instead
     */    
    public int getTop() {
        return y;
    }

    /**
     * @deprecated use setX() instead
     */
    public void setTop(int top) {
        this.y = top;
        update();
    }

    

    /**
     * Set a new text left-aligned to the widget
     * @param text the new text to set
     */
    public void setText(String text) 
    {
       super.setText(text);
        update();
    }


    /**
     * Set a new text to the widget
     * @param text the new text to set
     * @param align the text alignment to use
     */
    public void setText(String text,int align) 
    {
       super.setText(text,align);
        update();
    }


    protected String getData() {
        return x + " " + y + " " + right + " " + bottom + " " + "\"" + direction + "\"" + " " + speed + " " +
                " \"" + text + "\"";
    }

    public String getType() {
        return "scroller";
    }
}
