/*
 *	StringWidget.java 1.1 05.01.2009
 *
 * Copyright (c) 2004 Gavin & Georgina Sallery
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 */

package com.vwp.jLCD;

/**
 * A string widget is able to handle a simple static text 
 */
public class StringWidget extends Widget 
{
    int x;
    int y;
    String text;

    /**
     * The default constructor
     */
    StringWidget()
    {
    }
    
    /**
     * Constructor for a new widget
     * @param x position of the text in horizontal direction in unit cells
     * @param y position of the text in vertical direction in unit cells
     * @param text the text that has to be displayed at this position
     */
    public StringWidget(int x, int y, String text) {
        this.x = x;
        this.y = y;
        this.text = text;
    }

    /**
     * @return the current horizontal position in unit cells
     */
    public int getX() {
        return x;
    }

    /**
     * @param x the new horizontal position where the text has to be displayed at
     */
    public void setX(int x) {
        this.x = x;
        update();
    }

    /**
     * @return the current vertical position in unit cells
     */
    public int getY() {
        return y;
    }

    /**
     * @param y the new vertical position where the text has to be displayed at
     */
    public void setY(int y) {
        this.y = y;
        update();
    }

    /**
     * @return the text that is currently displayed by this widget
     */
    public String getText() {
        return text;
    }

    /**
     * @param text a new text string that has to be displayed left-aligned by this widget
     */
    public void setText(String text) {
        this.text = text;
        update();
    }
    
    
    
    /**
     * Set a new text to the widget
     * @param text the new text to set
     * @param align the text alignment to use
     */
    public void setText(String text,int align) {
       if (align==ALIGN_CENTER)
       {
          while (text.length()<lcd.getWidth()) text=" "+text+" ";
          if ((text.length()>lcd.getWidth()) && (text.startsWith(" ")))
           text=text.substring(1);
       }
       else if (align==ALIGN_RIGHT)
       {
          while (text.length()<lcd.getWidth()) text=" "+text;
       }          
        this.text = text;
        update();
    }
    

    protected String getData() {
        return x + " " + y + " \"" + text + "\"";
    }

    public String getType() {
        return "string";
    }
}
