package com.vwp.util;

/*
 *	Tools.java 1.6 15.08.2004
 *
 * Copyright (c) 2002-2004 Virtual Worlds Productions
 * Email: virtual_worlds@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds@gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
Reviewed:
11. 06. 2003
*/

import java.util.zip.*;



/**
 * The Tools class is intended to be a container for functionalities which don't
 * fit anywhere else.
 */
public class Tools
   {
   private CRC32   CSum=new CRC32();
   private Adler32 ASum=new Adler32();


  /*****************************************************************************
   * Constructs a new Tools object. This is necessary to initialize and
   * instantiate objects and values which are used within this class.
   * Independend from this fact, all Tools objects method can be (re)used
   * afterwards without the necessarity to reinitialize or to create a new
   * instance of it.
   */
   public Tools()
      {
      }



  /*****************************************************************************
   * Calculates some kind of hash or checksum out of a String. This checksum
   * should be unique for different contents of this String, also if they differ
   * only in some characters. Identically Strings of course would result in
   * equal hash values.
   * @param instr The input string to calculate the hash value from
   * @return The unique has value which belongs to the input String
   */
   public long calculateHash(String instr)
      {
      CSum.reset();
      CSum.update(instr.getBytes());
      ASum.reset();
      ASum.update(instr.getBytes());
      return ((CSum.getValue()<<32) & 0xFFFFFFFF00000000L) | (ASum.getValue());
      }



  /*****************************************************************************
   * Lets the current Thread sleep for the specified amount of milliseconds.
   * This function encapsulates the InterruptedException, so only one simple
   * method call is necessary.
   * @param msecs the amount of milliseconds the thread has to sleep
   */
   public static void threadSleep(long msecs)
      {
      try
         {
         Thread.sleep(msecs);
         }
      catch (InterruptedException ie)
         {
         }
      }

   }
