/*
 * SystemAccess.java 1.0 10.12.2008
 *
 * Copyright (c) 2008 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License v2 as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package system;

import java.util.*;
import java.awt.image.*;
import java.io.File;

import base.*;

/**
 * This class is the base class for the system access functionalities, all operating 
 * system related classes inherit from this one.
 */
public class SystemAccess extends Thread 
{
   static final int             WORK_CMD_GRAB_CD=1;  /** this working command is handled in run(), there the tracks of a CD are grabbed when workCmd is set to this value */
   static final int             WORK_CMD_GRAB_DVD=2; /** this working command is handled in run(), there the image of a DVD is grabbed when workCmd is set to this value */
                int             workCmd=0;           /** the working command that is handled within run() */
                String          path;                /** path where the DVD image has to be stored at */
                PoddyFrame      mainWin;             /** the main window object */
                Vector<CDEntry> cdList=new Vector<CDEntry>(); /** this vector stores all CD track information and needs to be emptied after grabbing of a CD has been finished */

                Process         recordProc=null;
                RecordInfo      recordInfo;

                
   class CDEntry
   {
      int     track=-1,totalTracks=-1;
      String  cdpath=null;
                   
      CDEntry(int track,String cdpath,int totalTracks)
      {
         this.track=track;
         this.cdpath=cdpath;
         this.totalTracks=totalTracks;
      }
   }
                
                
                
   /**
    * The standard constructor for this class, here the main window object is
    * handed over from the application when creating this object
    * @param mainWin the main window of the application
    */
   public SystemAccess(PoddyFrame mainWin)
   {
      this.mainWin=mainWin;
      start();
   }

   
   
   /**
    * This method retrieves CD ROM information and fetches a disc identifier
    * string that can be used from CDDB to identify this disc
    * @return the disc ID in format "DDDDDDDD t1 t2 t3 .. tn" where DDDDDDDD
    * is the ID of the whole disc and the t1..tn contain position information
    * of the different tracks
    */
   public String getCDDiscID()
	{
        return null;
	}

	
	
   /**
    * This method retrieves DVD ROM information and fetches a disc identifier
    * string that can be used from DVD databases to identify this disc
    * @return the disc ID in format "DDDDDDDD"
    */
   public long getDVDDiscID()
   {
        return 0;
   }

   
   
   /**
    * This method can be used to find out if there is currently an operation
    * in progress: as long as a workCmd is active and handled by the run()
    * method here true is returned
    * @return true when a working command is processed by the run() method,
    * false otherwise
    */
   public boolean isWorking()
	{
	   return false;
	}
	
	
	
   /**
    * This method has not to be overwritten!
    * Adds CD track information that have to be collected and used when
    * the method grabCDTracks() is called  
    * @param track the 1-based number of the track
    * @param path the path where the track data have to be stored at
    * @param totalTracks the total number of tracks
    * @return true in case the track information could be added to the
    * list successfully, false otherwise
    */
   public boolean setCDTrack(int track,String path,int totalTracks)
   {     
      cdList.add(new CDEntry(track,path,totalTracks));
      return true;
   }



   /**
    * This method has not to be overwritten as long as the workCmd is handled
    * in the overwritten run()-method!
    * Starts asynchronous grabbing and FLAC-compression of CD-tracks that
    * have been defined with setCDTrack() before
    */
	public void grabCDTracks()
	{
      workCmd=WORK_CMD_GRAB_CD;	   
	}


	
   /** 
    * This method has not to be overwritten as long as the workCmd is handled
    * in the overwritten run()-method!
    * Starts asynchronous grabbing and FLAC-compression of CD-tracks that
    * have been defined with setCDTrack() before
    */
   public void grabDVD(String mypath)
   {
      this.path=mypath;
      workCmd=WORK_CMD_GRAB_DVD;
   }
	

	
   /**
    * This method has not to be overwritten!
    * Returns the main directory as read from the settings file
    * @return path to the main directory
    */
	public String getMainDirectory()
   {
      return mainWin.config.main_dir+"/poddydata/";
   }

	
	
   /**
    * This method can be overwritten but doesn't have to
    * Returns the temporary directory that can be used for any temp data
    * @return path to the main directory
    */
   public String getTempDirectory()
   {
      File file=new File(getMainDirectory()+"tmp/");
      file.mkdirs();
      return file.toString();
   }

   
   
   /**
    * This method has not to be overwritten!
    * Returns the path to the video directory as defined in the settings file  
    * @return path to the video directory
    */
   public String getVideoDirectory()
   {
      return mainWin.config.video_dir+"/";
   }

   
   /**
    * Starts replaying of a DVD, here the external player has to be called
    * with the given path
    * @param isoPath the path to the ISO image that has to be used for replaying
    */
   public void playDVD(String isoPath)
   {
      
   }
	
	
	
   /**
    * Starts direct replaying of a DVD in the local drive
    */
   public void playDVDDirect()
   {
      
   }
   
   
   
   /**
    * Starts replaying of a video file, here the external player has to be called
    * with the given path
    * @param videoPath the path to the video file that has to be used for replaying
    */
   public void playVideo(String videoPath)
   {
      
   }

   
   /**
    * Externally stop all running videos by killing the related processes. This method
    * is mainly a workaround for video players that do not exit cleanly some times.    
    */
   public void stopVideo()
   {
   }

   
   
   public StationsList scanTV()
   {
      return null;
   }
   
   
   
   public void turnOnTV()
   {     
   }



   public void turnOffTV()
   {     
   }



   public void switchTVChannel(StationInfo info)
   {     
   }

   
   
   public void record(RecordInfo info,String channel,String recpath)
   {   
      recordInfo=info;
   }   
   
   
   public RecordInfo isRecording()
   {
      if (recordProc==null) return null;
      try
      {
         recordProc.exitValue();
      }
      catch (IllegalThreadStateException itse)
      {
         return recordInfo;
      }
      return null;
   }
   
   
   
   public void stopRecording()
   {
      recordProc.destroy();
   }

   
   
   public void powerOff()
   {
	   PoddyFrame.MyWin.dispose();
	   System.exit(0);
   }
   
   
   public boolean mountUSB(String mountpoint)
   {
      return false;  
   }

   
   
   public void unmountUSB(String mountpoint)
   {
   }

   
   public void encodeToUSB(String srcPath, CDInfo cdInfo,TrackInfo trackInfo,String mountpoint,BufferedImage coverImage)
   {
      
   }

   
}
